@@ -62,7 +62,8 @@ struct _BraseroAppPrivate
 	gint width;
 	gint height;
 
-	gboolean is_maximised;
+	guint is_maximised:1;
+	guint is_running:1;
 };
 
 #define BRASERO_APP_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BRASERO_TYPE_APP, BraseroAppPrivate))
@@ -161,6 +162,15 @@ static const gchar *description = {
 	"</ui>"
 };
 
+GtkUIManager *
+brasero_app_get_ui_manager (BraseroApp *app)
+{
+	BraseroAppPrivate *priv;
+
+	priv = BRASERO_APP_PRIVATE (app);
+	return priv->manager;
+}
+
 GtkWidget *
 brasero_app_get_statusbar1 (BraseroApp *app)
 {
@@ -169,7 +179,7 @@ brasero_app_get_statusbar1 (BraseroApp *app)
 	priv = BRASERO_APP_PRIVATE (app);
 
 	/* FIXME: change with future changes */
-	return priv->statusbar2;
+	return priv->statusbar1;
 }
 
 GtkWidget *
@@ -208,13 +218,24 @@ brasero_app_get_geometry (BraseroApp *app,
 static gboolean
 on_delete_cb (GtkWidget *window, GdkEvent *event, BraseroApp *app)
 {
+	BraseroAppPrivate *priv;
+
+	priv = BRASERO_APP_PRIVATE (app);
+	if (!priv->is_running)
+		return FALSE;
+
 	return brasero_session_save (app, TRUE, TRUE);
 }
 
 static gboolean
-on_destroy_cb (GtkWidget *window, GdkEvent *event, BraseroApp *app)
+on_destroy_cb (GtkWidget *window, BraseroApp *app)
 {
-	gtk_main_quit ();
+	BraseroAppPrivate *priv;
+
+	priv = BRASERO_APP_PRIVATE (app);
+	if (priv->is_running)
+		gtk_main_quit ();
+
 	return FALSE;
 }
 
@@ -228,7 +249,23 @@ on_exit_cb (GtkAction *action, BraseroApp *app)
 }
 
 void
-brasero_app_blank (BraseroApp *app)
+brasero_app_run (BraseroApp *app, gboolean load_default_project)
+{
+	BraseroAppPrivate *priv;
+
+	priv = BRASERO_APP_PRIVATE (app);
+	priv->is_running = TRUE;
+
+	gtk_widget_realize (GTK_WIDGET (app));
+    	brasero_session_load (app, load_default_project);
+	gtk_widget_show (GTK_WIDGET (app));
+	gtk_main ();
+}
+
+void
+brasero_app_blank (BraseroApp *app,
+		   const gchar *device,
+		   gboolean wait_and_close)
 {
 	GtkWidget *dialog;
 	GtkWidget *toplevel;
@@ -236,17 +273,39 @@ brasero_app_blank (BraseroApp *app)
 	dialog = brasero_blank_dialog_new ();
 	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (app));
 
-	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (toplevel));
-	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER_ON_PARENT);
+	if (device) {
+		BraseroDrive *drive;
+		BraseroMedium *medium;
+		BraseroMediumMonitor *monitor;
 
-	gtk_widget_show (dialog);
+		monitor = brasero_medium_monitor_get_default ();
+		drive = brasero_medium_monitor_get_drive (monitor, device);
+		g_object_unref (monitor);
+
+		medium = brasero_drive_get_medium (drive);
+
+		brasero_tool_dialog_set_medium (BRASERO_TOOL_DIALOG (dialog), medium);
+		g_object_unref (drive);
+	}
+
+	if (wait_and_close) {
+		gtk_widget_show (dialog);
+		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+	}
+	else {
+		gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (toplevel));
+		gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+		gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER_ON_PARENT);
+		
+		gtk_widget_show (dialog);
+	}
 }
 
 static void
 on_erase_cb (GtkAction *action, BraseroApp *app)
 {
-	brasero_app_blank (app);
+	brasero_app_blank (app, NULL, FALSE);
 }
 
 static void
@@ -265,8 +324,10 @@ on_eject_cb (GtkAction *action, BraseroApp *app)
 	gtk_widget_show (dialog);
 }
 
-static void
-on_integrity_check_cb (GtkAction *action, BraseroApp *app)
+void
+brasero_app_check (BraseroApp *app,
+		   const gchar *device,
+		   gboolean wait_and_close)
 {
 	GtkWidget *dialog;
 	GtkWidget *toplevel;
@@ -274,11 +335,39 @@ on_integrity_check_cb (GtkAction *action, BraseroApp *app)
 	dialog = brasero_sum_dialog_new ();
 	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (app));
 
-	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (toplevel));
-	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER_ON_PARENT);
+	if (device) {
+		BraseroDrive *drive;
+		BraseroMedium *medium;
+		BraseroMediumMonitor *monitor;
 
-	gtk_widget_show (dialog);
+		monitor = brasero_medium_monitor_get_default ();
+		drive = brasero_medium_monitor_get_drive (monitor, device);
+		g_object_unref (monitor);
+
+		medium = brasero_drive_get_medium (drive);
+
+		brasero_tool_dialog_set_medium (BRASERO_TOOL_DIALOG (dialog), medium);
+		g_object_unref (drive);
+	}
+
+	if (wait_and_close) {
+		gtk_widget_show (dialog);
+		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+	}
+	else {
+		gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (toplevel));
+		gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+		gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER_ON_PARENT);
+		
+		gtk_widget_show (dialog);
+	}
+}
+
+static void
+on_integrity_check_cb (GtkAction *action, BraseroApp *app)
+{
+	brasero_app_check (app, NULL, FALSE);
 }
 
 static void
@@ -391,7 +480,7 @@ on_help_cb (GtkAction *action, BraseroApp *app)
 		d = gtk_message_dialog_new (GTK_WINDOW (app),
 					    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
 					    GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-					    error->message);
+					    "%s", error->message);
 		gtk_dialog_run (GTK_DIALOG(d));
 		gtk_widget_destroy (d);
 		g_error_free (error);
@@ -547,6 +636,10 @@ brasero_menu_item_selected_cb (GtkMenuItem *proxy,
 				    priv->tooltip_ctx,
 				    message);
 		g_free (message);
+
+		gtk_statusbar_push (GTK_STATUSBAR (priv->statusbar1),
+				    priv->tooltip_ctx,
+				    "");
 	}
 }
 
@@ -560,6 +653,8 @@ brasero_menu_item_deselected_cb (GtkMenuItem *proxy,
 
 	gtk_statusbar_pop (GTK_STATUSBAR (priv->statusbar2),
 			   priv->tooltip_ctx);
+	gtk_statusbar_pop (GTK_STATUSBAR (priv->statusbar1),
+			   priv->tooltip_ctx);
 }
 
 static void
@@ -603,20 +698,16 @@ brasero_caps_changed_cb (BraseroPluginManager *manager,
 			 BraseroApp *app)
 {
 	BraseroAppPrivate *priv;
-	BraseroBurnCaps *caps;
 	GtkWidget *widget;
 
 	priv = BRASERO_APP_PRIVATE (app);
 
-	caps = brasero_burn_caps_get_default ();
 	widget = gtk_ui_manager_get_widget (priv->manager, "/menubar/ToolMenu/Check");
 
-	if (!brasero_burn_caps_can_checksum (caps))
+	if (!brasero_burn_library_can_checksum ())
 		gtk_widget_set_sensitive (widget, FALSE);
 	else
 		gtk_widget_set_sensitive (widget, TRUE);
-
-	g_object_unref (caps);
 }
 
 static void
@@ -625,7 +716,6 @@ brasero_app_init (BraseroApp *object)
 	GtkWidget *hbox;
 	GtkWidget *menubar;
 	GError *error = NULL;
-	BraseroBurnCaps *caps;
 	BraseroAppPrivate *priv;
 	GtkAccelGroup *accel_group;
 	GtkActionGroup *action_group;
@@ -633,7 +723,7 @@ brasero_app_init (BraseroApp *object)
 
 	/* New window */
 	priv = BRASERO_APP_PRIVATE (object);
-	g_set_application_name (_("Brasero Disc Burning"));
+	g_set_application_name (_("Brasero Disc Burner"));
 
 	gtk_window_set_default_icon_name ("brasero");
 	gtk_window_set_icon_name (GTK_WINDOW (object), "brasero");
@@ -707,14 +797,12 @@ brasero_app_init (BraseroApp *object)
 	gtk_ui_manager_ensure_update (priv->manager);
 
 	/* check if we can use checksums (we need plugins enabled) */
-	caps = brasero_burn_caps_get_default ();
-	if (!brasero_burn_caps_can_checksum (caps)) {
+	if (!brasero_burn_library_can_checksum ()) {
 		GtkWidget *widget;
 
 		widget = gtk_ui_manager_get_widget (priv->manager, "/menubar/ToolMenu/Check");
 		gtk_widget_set_sensitive (widget, FALSE);
 	}
-	g_object_unref (caps);
 
 	plugin_manager = brasero_plugin_manager_get_default ();
 	g_signal_connect (plugin_manager,
