@@ -61,11 +61,16 @@
 #include "brasero-disc-message.h"
 #include "brasero-rename.h"
 #include "brasero-notify.h"
+#include "brasero-session-cfg.h"
+
+#include "brasero-app.h"
+#include "brasero-project-manager.h"
 
 #include "burn-debug.h"
 #include "burn-basics.h"
 #include "burn-track.h"
 #include "burn-session.h"
+#include "burn-volume-obj.h"
 
 
 typedef struct _BraseroDataDiscPrivate BraseroDataDiscPrivate;
@@ -80,6 +85,7 @@ struct _BraseroDataDiscPrivate
 
 	GtkUIManager *manager;
 	GtkActionGroup *disc_group;
+	GtkActionGroup *import_group;
 
 	gint press_start_x;
 	gint press_start_y;
@@ -93,15 +99,15 @@ struct _BraseroDataDiscPrivate
 	guint editing:1;
 	guint reject_files:1;
 
+	guint overburning:1;
+
 	guint G2_files:1;
 	guint deep_directory:1;
 };
 
 #define BRASERO_DATA_DISC_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BRASERO_TYPE_DATA_DISC, BraseroDataDiscPrivate))
 
-static void
-brasero_data_disc_import_session_cb (GtkToggleAction *action,
-				     BraseroDataDisc *disc);
+
 static void
 brasero_data_disc_new_folder_clicked_cb (GtkButton *button,
 					 BraseroDataDisc *disc);
@@ -132,18 +138,12 @@ static GtkActionEntry entries [] = {
 	 G_CALLBACK (brasero_data_disc_new_folder_clicked_cb)},
 };
 
-static GtkToggleActionEntry toggle_entries [] = {
-	{"ImportSession", "drive-optical", N_("Import Session"), NULL, N_("Import session"),
-	 G_CALLBACK (brasero_data_disc_import_session_cb), FALSE},
-};
-
 static const gchar *description = {
 	"<ui>"
 	"<menubar name='menubar' >"
 		"<menu action='EditMenu'>"
 		"<placeholder name='EditPlaceholder'>"
 			"<menuitem action='NewFolder'/>"
-			"<menuitem action='ImportSession'/>"
 		"</placeholder>"
 		"</menu>"
 	"</menubar>"
@@ -158,7 +158,6 @@ static const gchar *description = {
 		"<placeholder name='DiscButtonPlaceholder'>"
 			"<separator/>"
 			"<toolitem action='NewFolder'/>"
-			"<toolitem action='ImportSession'/>"
 		"</placeholder>"
 	"</toolbar>"
 	"</ui>"
@@ -195,6 +194,18 @@ G_DEFINE_TYPE_WITH_CODE (BraseroDataDisc,
 			 G_IMPLEMENT_INTERFACE (BRASERO_TYPE_DISC,
 					        brasero_data_disc_iface_disc_init));
 
+#define BRASERO_DATA_DISC_MEDIUM		"brasero-data-disc-medium"
+#define BRASERO_DATA_DISC_MERGE_ID		"brasero-data-disc-merge-id"
+#define BRASERO_MEDIUM_GET_UDI(medium)		(brasero_drive_get_udi (brasero_medium_get_drive (medium)))
+
+BraseroMedium *
+brasero_data_disc_get_loaded_medium (BraseroDataDisc *self)
+{
+	BraseroDataDiscPrivate *priv;
+	priv = BRASERO_DATA_DISC_PRIVATE (self);
+	return brasero_data_session_get_loaded_medium (BRASERO_DATA_SESSION (priv->project));
+}
+
 /**
  * Actions callbacks
  */
@@ -203,28 +214,15 @@ static void
 brasero_data_disc_import_failure_dialog (BraseroDataDisc *disc,
 					 GError *error)
 {
-	GtkWidget *dialog;
-	GtkWidget *toplevel;
-
-	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (disc));
-	dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel),
-					 GTK_DIALOG_DESTROY_WITH_PARENT |
-					 GTK_DIALOG_MODAL,
-					 GTK_MESSAGE_WARNING,
-					 GTK_BUTTONS_CLOSE,
-					 _("The session couldn't be imported:"));
-
-	gtk_window_set_title (GTK_WINDOW (dialog), _("Session Import Error"));
-	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-						  error?error->message:_("unknown error"));
-
-	gtk_widget_show_all (dialog);
-	gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (dialog);
+	brasero_utils_message_dialog (gtk_widget_get_toplevel (GTK_WIDGET (disc)),
+				      _("The session couldn't be imported:"),
+				      error?error->message:_("unknown error"),
+				      GTK_MESSAGE_WARNING);
 }
 
 static gboolean
 brasero_data_disc_import_session (BraseroDataDisc *disc,
+				  BraseroMedium *medium,
 				  gboolean import)
 {
 	BraseroDataDiscPrivate *priv;
@@ -234,7 +232,7 @@ brasero_data_disc_import_session (BraseroDataDisc *disc,
 	if (import) {
 		GError *error = NULL;
 
-		if (!brasero_data_session_add_last (BRASERO_DATA_SESSION (priv->project), &error)) {
+		if (!brasero_data_session_add_last (BRASERO_DATA_SESSION (priv->project), medium, &error)) {
 			brasero_data_disc_import_failure_dialog (disc, error);
 			return FALSE;
 		}
@@ -249,22 +247,28 @@ brasero_data_disc_import_session (BraseroDataDisc *disc,
 
 static void
 brasero_data_disc_import_session_cb (GtkToggleAction *action,
-				     BraseroDataDisc *disc)
+				     BraseroDataDisc *self)
 {
 	BraseroDataDiscPrivate *priv;
+	BraseroMedium *medium;
 	gboolean res;
 
-	priv = BRASERO_DATA_DISC_PRIVATE (disc);
+	priv = BRASERO_DATA_DISC_PRIVATE (self);
+
+	medium = g_object_get_data (G_OBJECT (action), BRASERO_DATA_DISC_MEDIUM);
+	if (!medium)
+		return;
 
 	brasero_notify_message_remove (BRASERO_NOTIFY (priv->message), BRASERO_NOTIFY_CONTEXT_MULTISESSION);
-
-	res = brasero_data_disc_import_session (disc, gtk_toggle_action_get_active (action));
+	res = brasero_data_disc_import_session (self,
+						medium,
+						gtk_toggle_action_get_active (action));
 
 	/* make sure the button reflects the current state */
 	if (gtk_toggle_action_get_active (action) != res) {
-		g_signal_handlers_block_by_func (action, brasero_data_disc_import_session_cb, disc);
+		g_signal_handlers_block_by_func (action, brasero_data_disc_import_session_cb, self);
 		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), res);
-		g_signal_handlers_unblock_by_func (action, brasero_data_disc_import_session_cb, disc);
+		g_signal_handlers_unblock_by_func (action, brasero_data_disc_import_session_cb, self);
 	}
 }
 
@@ -574,6 +578,76 @@ brasero_data_disc_row_expanded_cb (GtkTreeView *tree,
 	brasero_data_disc_set_expand_state (self, treepath, TRUE);
 }
 
+static void
+brasero_data_disc_use_overburn_response_cb (GtkButton *button,
+					    GtkResponseType response,
+					    BraseroDataDisc *self)
+{
+	BraseroDataDiscPrivate *priv;
+
+	priv = BRASERO_DATA_DISC_PRIVATE (self);
+
+	if (response != GTK_RESPONSE_OK)
+		return;
+
+	priv->overburning = 1;
+}
+
+static void
+brasero_data_disc_project_oversized_cb (BraseroDataProject *project,
+					gboolean oversized,
+					gboolean overburn,
+					BraseroDataDisc *self)
+{
+	GtkWidget *message;
+	BraseroDataDiscPrivate *priv;
+
+	priv = BRASERO_DATA_DISC_PRIVATE (self);
+
+	if (overburn) {
+		if (priv->overburning)
+			return;
+
+		message = brasero_notify_message_add (BRASERO_NOTIFY (priv->message),
+						      _("Would you like to burn beyond the disc reported capacity?"),
+						      _("The size of the project is too large for the disc and you must remove files from the project otherwise."
+							"\nYou may want to use this option if you're using 90 or 100 min CD-R(W) which can't be properly recognised and therefore need overburn option."
+							"\nNOTE: This option might cause failure."),
+						      -1,
+						      BRASERO_NOTIFY_CONTEXT_SIZE);
+
+		brasero_disc_message_set_image (BRASERO_DISC_MESSAGE (message), GTK_STOCK_DIALOG_WARNING);
+		brasero_notify_button_add (BRASERO_NOTIFY (priv->message),
+					   BRASERO_DISC_MESSAGE (message),
+					   _("_Overburn"),
+					   _("Burn beyond the disc reported capacity"),
+					   GTK_RESPONSE_OK);
+		brasero_notify_button_add (BRASERO_NOTIFY (priv->message),
+					   BRASERO_DISC_MESSAGE (message),
+					   _("_Cancel"),
+					   _("Click here not to use overburning"),
+					   GTK_RESPONSE_CANCEL);
+		
+		g_signal_connect (BRASERO_DISC_MESSAGE (message),
+				  "response",
+				  G_CALLBACK (brasero_data_disc_use_overburn_response_cb),
+				  self);
+	}
+	else if (oversized) {
+		message = brasero_notify_message_add (BRASERO_NOTIFY (priv->message),
+						      _("Please, delete some files from the project."),
+						      _("The size of the project is too large for the disc even with the overburn option."),
+						      -1,
+						      BRASERO_NOTIFY_CONTEXT_SIZE);
+
+		brasero_disc_message_set_image (BRASERO_DISC_MESSAGE (message), GTK_STOCK_DIALOG_WARNING);
+		brasero_disc_message_add_close_button (BRASERO_DISC_MESSAGE (message));
+	}
+	else
+		brasero_notify_message_remove (BRASERO_NOTIFY (priv->message),
+					       BRASERO_NOTIFY_CONTEXT_SIZE);
+}
+
 static void
 brasero_data_disc_project_loaded_cb (BraseroDataProject *project,
 				     gint loading,
@@ -629,8 +703,6 @@ brasero_data_disc_project_loaded_cb (BraseroDataProject *project,
 
 		gtk_widget_destroy (message);
 	}
-
-	priv->loading = FALSE;
 }
 
 static void
@@ -671,6 +743,98 @@ brasero_data_disc_filtered_uri_cb (BraseroDataVFS *vfs,
 		brasero_file_filtered_remove (BRASERO_FILE_FILTERED (priv->filter), uri);
 }
 
+struct _BraseroDataDiscProjectSwitch {
+	gchar *uri;
+	BraseroDataDisc *disc;
+};
+typedef struct _BraseroDataDiscProjectSwitch BraseroDataDiscProjectSwitch;
+
+static gboolean
+brasero_data_disc_switch_to_image (gpointer data)
+{
+	GtkWidget *manager;
+	GtkWidget *toplevel;
+	BraseroDataDiscPrivate *priv;
+	BraseroDataDiscProjectSwitch *callback_data = data;
+
+	priv = BRASERO_DATA_DISC_PRIVATE (callback_data->disc);
+
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (callback_data->disc));
+	if (!BRASERO_IS_APP (toplevel))
+		return BRASERO_BURN_OK;
+
+	/* Clean up everything to avoid warning dialog */
+	brasero_data_project_reset (priv->project);
+
+	/* Tell project manager to switch */
+	manager = brasero_app_get_project_manager (BRASERO_APP (toplevel));
+	brasero_project_manager_iso (BRASERO_PROJECT_MANAGER (manager),
+				     callback_data->uri);
+
+	return FALSE;
+}
+
+static BraseroBurnResult
+brasero_data_disc_image_uri_cb (BraseroDataVFS *vfs,
+				const gchar *uri,
+				BraseroDataDisc *self)
+{
+	gint answer;
+	gchar *name;
+	GtkWidget *button;
+	GtkWidget *dialog;
+	GtkWidget *toplevel;
+	BraseroDataDiscPrivate *priv;
+	BraseroDataDiscProjectSwitch *callback_data;
+
+	priv = BRASERO_DATA_DISC_PRIVATE (self);
+
+	if (priv->loading)
+		return BRASERO_BURN_OK;
+
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (self));
+	if (!BRASERO_IS_APP (toplevel))
+		return BRASERO_BURN_OK;
+
+	name = brasero_file_node_get_uri_name (uri);
+	dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel),
+					 GTK_DIALOG_DESTROY_WITH_PARENT |
+					 GTK_DIALOG_MODAL,
+					 GTK_MESSAGE_QUESTION,
+					 GTK_BUTTONS_NONE,
+					 _("Do you want to burn \"%s\" to a disc or add it in to the data project?"),
+					 name);
+	g_free (name);
+
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+						  _("This file is the image of a disc and can therefore be burnt to disc without having to add it to a data project first."));
+
+	gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Add to project"), GTK_RESPONSE_NO);
+
+	button = brasero_utils_make_button (_("_Burn..."),
+					    NULL,
+					    "media-optical-burn",
+					    GTK_ICON_SIZE_BUTTON);
+	gtk_widget_show (button);
+	gtk_dialog_add_action_widget (GTK_DIALOG (dialog),
+				      button,
+				      GTK_RESPONSE_YES);
+
+	gtk_widget_show_all (dialog);
+	answer = gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+
+	if (answer != GTK_RESPONSE_YES)
+		return BRASERO_BURN_OK;
+
+	callback_data = g_new (BraseroDataDiscProjectSwitch, 1);
+	callback_data->disc = self;
+	callback_data->uri = g_strdup (uri);
+	g_idle_add (brasero_data_disc_switch_to_image, callback_data);
+
+	return BRASERO_BURN_CANCEL;
+}
+
 static void
 brasero_data_disc_filter_expanded_cb (GtkExpander *expander,
 				      BraseroDataDisc *self)
@@ -717,8 +881,7 @@ brasero_data_disc_unreadable_uri_cb (BraseroDataVFS *vfs,
 				     BraseroDataDisc *self)
 {
 	gchar *name;
-	GtkWidget *dialog;
-	GtkWidget *toplevel;
+	gchar *primary;
 	BraseroDataDiscPrivate *priv;
 
 	priv = BRASERO_DATA_DISC_PRIVATE (self);
@@ -731,24 +894,13 @@ brasero_data_disc_unreadable_uri_cb (BraseroDataVFS *vfs,
 		return;
 	}
 
- 	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (self));
-	dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel),
-					 GTK_DIALOG_DESTROY_WITH_PARENT |
-					 GTK_DIALOG_MODAL,
-					 GTK_MESSAGE_ERROR,
-					 GTK_BUTTONS_CLOSE,
-					 _("\"%s\" cannot be added to the selection:"),
-					 name);
+	primary = g_strdup_printf (_("\"%s\" cannot be added to the selection:"), name);
+	brasero_utils_message_dialog (gtk_widget_get_toplevel (GTK_WIDGET (self)),
+				      primary,
+				      error->message,
+				      GTK_MESSAGE_ERROR);
+	g_free (primary);
 	g_free (name);
-
-	gtk_window_set_title (GTK_WINDOW (dialog), _("Unreadable File"));
-	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-						  "%s.",
-						  error->message);
-
-	gtk_widget_show_all (dialog);
-	gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (dialog);
 }
 
 static void
@@ -757,8 +909,7 @@ brasero_data_disc_recursive_uri_cb (BraseroDataVFS *vfs,
 				    BraseroDataDisc *self)
 {
 	gchar *name;
-	GtkWidget *dialog;
-	GtkWidget *toplevel;
+	gchar *primary;
 	BraseroDataDiscPrivate *priv;
 
 	priv = BRASERO_DATA_DISC_PRIVATE (self);
@@ -774,24 +925,13 @@ brasero_data_disc_recursive_uri_cb (BraseroDataVFS *vfs,
 		return;
 	}
 
-	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (self));
-	dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel),
-					 GTK_DIALOG_DESTROY_WITH_PARENT |
-					 GTK_DIALOG_MODAL,
-					 GTK_MESSAGE_ERROR,
-					 GTK_BUTTONS_CLOSE,
-					 _("\"%s\" cannot be added to the selection:"),
-					 name);
+	primary = g_strdup_printf (_("\"%s\" cannot be added to the selection:"), name);
+	brasero_utils_message_dialog (gtk_widget_get_toplevel (GTK_WIDGET (self)),
+				      primary,
+				      _("it is a recursive symlink."),
+				      GTK_MESSAGE_ERROR);
+	g_free (primary);
 	g_free (name);
-
-	gtk_window_set_title (GTK_WINDOW (dialog), _("Recursive Symlink"));
-
-	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-						  _("it is a recursive symlink."));
-
-	gtk_widget_show_all (dialog);
-	gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (dialog);
 }
 
 static void
@@ -800,8 +940,7 @@ brasero_data_disc_unknown_uri_cb (BraseroDataVFS *vfs,
 				  BraseroDataDisc *self)
 {
 	gchar *name;
-	GtkWidget *dialog;
-	GtkWidget *toplevel;
+	gchar *primary;
 	BraseroDataDiscPrivate *priv;
 
 	priv = BRASERO_DATA_DISC_PRIVATE (self);
@@ -817,24 +956,13 @@ brasero_data_disc_unknown_uri_cb (BraseroDataVFS *vfs,
 		return;
 	}
 
-	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (self));
-	dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel),
-					 GTK_DIALOG_DESTROY_WITH_PARENT |
-					 GTK_DIALOG_MODAL,
-					 GTK_MESSAGE_ERROR,
-					 GTK_BUTTONS_CLOSE,
-					 _("\"%s\" cannot be added to the selection:"),
-					 name);
+	primary = g_strdup_printf (_("\"%s\" cannot be added to the selection:"), name);
+	brasero_utils_message_dialog (gtk_widget_get_toplevel (GTK_WIDGET (self)),
+				      primary,
+				      _("it doesn't exist at the specified location."),
+				      GTK_MESSAGE_ERROR);
+	g_free (primary);
 	g_free (name);
-
-	gtk_window_set_title (GTK_WINDOW (dialog), _("File Not Found"));
-
-	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-						  _("it doesn't exist at the specified location."));
-
-	gtk_widget_show_all (dialog);
-	gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (dialog);
 }
 
 static gboolean
@@ -991,66 +1119,213 @@ brasero_disc_disc_session_import_response_cb (GtkButton *button,
 {
 	gboolean res;
 	GtkAction *action;
+	gchar *action_name;
+	BraseroMedium *medium;
 	BraseroDataDiscPrivate *priv;
 
 	if (response != GTK_RESPONSE_OK)
 		return;
 
 	priv = BRASERO_DATA_DISC_PRIVATE (self);
-	res = brasero_data_disc_import_session (self, TRUE);
 
-	action = gtk_action_group_get_action (priv->disc_group, "ImportSession");
+	medium = g_object_get_data (G_OBJECT (button), BRASERO_DATA_DISC_MEDIUM);
+	res = brasero_data_disc_import_session (self, medium, TRUE);
+
+	action_name = g_strdup_printf ("Import_%s", BRASERO_MEDIUM_GET_UDI (medium));
+	action = gtk_action_group_get_action (priv->import_group, action_name);
+	g_free (action_name);
 
 	g_signal_handlers_block_by_func (action, brasero_data_disc_import_session_cb, self);
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), res);
 	g_signal_handlers_unblock_by_func (action, brasero_data_disc_import_session_cb, self);
 }
 
+static void
+brasero_data_disc_import_button_new (BraseroDataDisc *self,
+				     BraseroMedium *medium)
+{
+	int merge_id;
+	gchar *string;
+	gchar *tooltip;
+	GtkAction *action;
+	gchar *action_name;
+	gchar *volume_name;
+	gchar *description;
+	BraseroDataDiscPrivate *priv;
+	GtkToggleActionEntry toggle_entry = { 0, };
+
+	priv = BRASERO_DATA_DISC_PRIVATE (self);
+
+	if (!priv->manager)
+		return;
+
+	action_name = g_strdup_printf ("Import_%s", BRASERO_MEDIUM_GET_UDI (medium));
+
+	tooltip = brasero_medium_get_tooltip (medium);
+	/* Translators: %s is a string describing the type of medium and the 
+	 * drive it is in. */
+	string = g_strdup_printf (_("Import %s"), tooltip);
+	g_free (tooltip);
+	tooltip = string;
+
+	volume_name = brasero_volume_get_name (BRASERO_VOLUME (medium));
+	/* Translators: %s is the name of the volume to import */
+	string = g_strdup_printf (_("Import %s"), volume_name);
+	g_free (volume_name);
+	volume_name = string;
+
+	toggle_entry.name = action_name;
+	toggle_entry.stock_id = "drive-optical";
+	toggle_entry.label = string;
+	toggle_entry.tooltip = tooltip;
+	toggle_entry.callback = G_CALLBACK (brasero_data_disc_import_session_cb);
+
+	gtk_action_group_add_toggle_actions (priv->import_group,
+					     &toggle_entry,
+					     1,
+					     self);
+	g_free (volume_name);
+	g_free (tooltip);
+
+	action = gtk_action_group_get_action (priv->import_group, action_name);
+	if (!action) {
+		g_free (action_name);
+		return;
+	}
+
+	g_object_ref (medium);
+	g_object_set_data (G_OBJECT (action),
+			   BRASERO_DATA_DISC_MEDIUM,
+			   medium);
+
+	/* Translators: This is a verb */
+	g_object_set (action,
+		      "short-label", _("Import"), /* for toolbar buttons */
+		      NULL);
+
+	description = g_strdup_printf ("<ui>"
+				       "<menubar name='menubar'>"
+				       "<menu action='EditMenu'>"
+				       "<placeholder name='EditPlaceholder'>"
+				       "<menuitem action='%s'/>"
+				       "</placeholder>"
+				       "</menu>"
+				       "</menubar>"
+				       "<toolbar name='Toolbar'>"
+				       "<placeholder name='DiscButtonPlaceholder'>"
+				       "<toolitem action='%s'/>"
+				       "</placeholder>"
+				       "</toolbar>"
+				       "</ui>",
+				       action_name,
+				       action_name);
+
+	merge_id = gtk_ui_manager_add_ui_from_string (priv->manager,
+						      description,
+						      -1,
+						      NULL);
+	g_object_set_data (G_OBJECT( action),
+			   BRASERO_DATA_DISC_MERGE_ID,
+			   GINT_TO_POINTER (merge_id));
+
+	g_free (description);
+	g_free (action_name);
+}
+
 static void
 brasero_data_disc_session_available_cb (BraseroDataSession *session,
-					gboolean multisession,
+					BraseroMedium *medium,
+					gboolean available,
 					BraseroDataDisc *self)
 {
-	GtkAction *action;
 	BraseroDataDiscPrivate *priv;
 
 	priv = BRASERO_DATA_DISC_PRIVATE (self);
-	action = gtk_action_group_get_action (priv->disc_group, "ImportSession");
 
-	if (multisession) {
+	if (!priv->manager)
+		return;
+
+	if (available) {
+		gchar *string;
+		gchar *volume_name;
 		GtkWidget *message;
 
-		gtk_action_set_sensitive (action, TRUE);
+		/* create button and menu entry */
+		brasero_data_disc_import_button_new (self, medium);
+
+		/* ask user */
+		volume_name = brasero_volume_get_name (BRASERO_VOLUME (medium));
+		/* Translators: %s is the name of the volume to import */
+		string = g_strdup_printf (_("Do you want to import the session from \'%s\'?"), volume_name);
 		message = brasero_notify_message_add (BRASERO_NOTIFY (priv->message),
-						      _("A multisession disc is inserted:"),
-						      _("Do you want to import its contents?"),
+						      string,
+						      _("That way, old files from previous sessions will be usable after burning."),
 						      10000,
 						      BRASERO_NOTIFY_CONTEXT_MULTISESSION);
+		g_free (volume_name);
+		g_free (string);
 
-		brasero_disc_message_set_image (BRASERO_DISC_MESSAGE (message), GTK_STOCK_DIALOG_INFO);
+		brasero_disc_message_set_image (BRASERO_DISC_MESSAGE (message),
+						GTK_STOCK_DIALOG_INFO);
 
 		brasero_notify_button_add (BRASERO_NOTIFY (priv->message),
 					   BRASERO_DISC_MESSAGE (message),
 					   _("_Import Session"),
 					   _("Click here to import its contents"),
 					   GTK_RESPONSE_OK);
+
+		/* no need to ref the medium since its removal would cause the
+		 * hiding of the message it's associated with */
+		g_object_set_data (G_OBJECT (message),
+				   BRASERO_DATA_DISC_MEDIUM,
+				   medium);
+
 		g_signal_connect (BRASERO_DISC_MESSAGE (message),
 				  "response",
 				  G_CALLBACK (brasero_disc_disc_session_import_response_cb),
 				  self);
 	}
 	else {
+		int merge_id;
+		GtkAction *action;
+		gchar *action_name;
+
+		action_name = g_strdup_printf ("Import_%s", BRASERO_MEDIUM_GET_UDI (medium));
+		action = gtk_action_group_get_action (priv->import_group, action_name);
+		g_free (action_name);
+
 		brasero_notify_message_remove (BRASERO_NOTIFY (priv->message), BRASERO_NOTIFY_CONTEXT_MULTISESSION);
-		gtk_action_set_sensitive (action, FALSE);
-		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), FALSE);
+
+		merge_id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (action), BRASERO_DATA_DISC_MERGE_ID));
+		gtk_ui_manager_remove_ui (priv->manager, merge_id);
+		gtk_action_group_remove_action (priv->import_group, action);
+
+		/* unref it since we reffed it when it was associated with the action */
+		g_object_unref (medium);
 	}
 }
 
 static void
 brasero_data_disc_session_loaded_cb (BraseroDataSession *session,
+				     BraseroMedium *medium,
 				     gboolean loaded,
 				     BraseroDataDisc *self)
 {
+	BraseroDataDiscPrivate *priv;
+	gchar *action_name;
+	GtkAction *action;
+
+	priv = BRASERO_DATA_DISC_PRIVATE (self);
+
+	action_name = g_strdup_printf ("Import_%s", BRASERO_MEDIUM_GET_UDI (medium));
+	action = gtk_action_group_get_action (priv->import_group, action_name);
+	g_free (action_name);
+
+	g_signal_handlers_block_by_func (action, brasero_data_disc_import_session_cb, self);
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), loaded);
+	g_signal_handlers_unblock_by_func (action, brasero_data_disc_import_session_cb, self);
+
+	/* Update buttons states */
 	if (loaded)
 		brasero_disc_flags_changed (BRASERO_DISC (self), BRASERO_BURN_FLAG_MERGE);
 	else
@@ -1065,16 +1340,14 @@ static void
 brasero_data_disc_clear (BraseroDisc *disc)
 {
 	BraseroDataDiscPrivate *priv;
-	GtkAction *action;
 
 	priv = BRASERO_DATA_DISC_PRIVATE (disc);
 
 	if (priv->loading)
 		return;
 
-	action = gtk_action_group_get_action (priv->disc_group, "ImportSession");
-	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)))
-		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), FALSE);
+	if (brasero_data_session_get_loaded_medium (BRASERO_DATA_SESSION (priv->project)))
+		brasero_data_session_remove_last (BRASERO_DATA_SESSION (priv->project));
 
 	if (priv->load_errors) {
 		g_slist_foreach (priv->load_errors, (GFunc) g_free , NULL);
@@ -1082,6 +1355,7 @@ brasero_data_disc_clear (BraseroDisc *disc)
 		priv->load_errors = NULL;
 	}
 
+	priv->overburning = FALSE;
 	priv->G2_files = FALSE;
 	priv->deep_directory = FALSE;
 
@@ -1100,13 +1374,19 @@ static void
 brasero_data_disc_reset (BraseroDisc *disc)
 {
 	BraseroDataDiscPrivate *priv;
-	GtkAction *action;
 
 	priv = BRASERO_DATA_DISC_PRIVATE (disc);
 
-	action = gtk_action_group_get_action (priv->disc_group, "ImportSession");
-	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)))
-		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), FALSE);
+	/* Unload session */
+	if (brasero_data_session_get_loaded_medium (BRASERO_DATA_SESSION (priv->project)))
+		brasero_data_session_remove_last (BRASERO_DATA_SESSION (priv->project));
+
+	/* Hide all toggle actions for session importing */
+	if (gtk_action_group_get_visible (priv->import_group))
+		gtk_action_group_set_visible (priv->import_group, FALSE);
+
+	if (gtk_action_group_get_visible (priv->disc_group))
+		gtk_action_group_set_visible (priv->disc_group, FALSE);
 
 	if (priv->load_errors) {
 		g_slist_foreach (priv->load_errors, (GFunc) g_free , NULL);
@@ -1116,6 +1396,8 @@ brasero_data_disc_reset (BraseroDisc *disc)
 
 	brasero_data_project_reset (priv->project);
 
+	priv->overburning = FALSE;
+
 	priv->loading = FALSE;
 	priv->G2_files = FALSE;
 	priv->deep_directory = FALSE;
@@ -1242,6 +1524,7 @@ static BraseroDiscResult
 brasero_data_disc_set_session_param (BraseroDisc *self,
 				     BraseroBurnSession *session)
 {
+	GValue *value;
 	BraseroFileNode *root;
 	BraseroTrackType type;
 	BraseroImageFS fs_type;
@@ -1266,20 +1549,39 @@ brasero_data_disc_set_session_param (BraseroDisc *self,
 	if (brasero_data_project_is_video_project (priv->project))
 		fs_type |= BRASERO_IMAGE_FS_VIDEO;
 
-	type.type = BRASERO_TRACK_TYPE_DATA;
-	type.subtype.fs_type = fs_type;
-	brasero_burn_session_set_input_type (session, &type);
+	value = g_new0 (GValue, 1);
+	g_value_init (value, G_TYPE_INT64);
+	g_value_set_int64 (value, brasero_data_project_get_size (priv->project));
+	brasero_burn_session_tag_add (session,
+				      BRASERO_DATA_TRACK_SIZE_TAG,
+				      value);
 
 	/* set multisession options */
 	if (brasero_data_session_get_loaded_medium (BRASERO_DATA_SESSION (priv->project))) {
+		BraseroDrive *drive;
+		BraseroMedium *medium;
+
+		medium = brasero_data_session_get_loaded_medium (BRASERO_DATA_SESSION (priv->project));
+		if (medium)
+			drive = brasero_medium_get_drive (medium);
+		else
+			drive = NULL;
+
+		if (priv->overburning)
+			brasero_burn_session_add_flag (session, BRASERO_BURN_FLAG_OVERBURN);
+
 		/* remove the following flag just in case */
 		brasero_burn_session_remove_flag (session,
 						  BRASERO_BURN_FLAG_FAST_BLANK|
 						  BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE);
 		brasero_burn_session_add_flag (session, BRASERO_BURN_FLAG_MERGE);
-		brasero_burn_session_set_burner (session, brasero_data_session_get_loaded_medium (BRASERO_DATA_SESSION (priv->project)));
+		brasero_burn_session_set_burner (session, drive);
 	}
 
+	type.type = BRASERO_TRACK_TYPE_DATA;
+	type.subtype.fs_type = fs_type;
+	brasero_burn_session_set_input_type (session, &type);
+
 	return BRASERO_DISC_OK;
 }
 
@@ -1371,7 +1673,6 @@ brasero_data_disc_load_track (BraseroDisc *disc,
 	priv->loading = brasero_data_project_load_contents (priv->project,
 							    track->contents.data.grafts,
 							    track->contents.data.excluded);
-
 	if (!priv->loading) {
 		gtk_widget_set_sensitive (GTK_WIDGET (priv->tree), TRUE);
 		gtk_widget_set_sensitive (GTK_WIDGET (priv->filter), TRUE);
@@ -1411,15 +1712,18 @@ brasero_data_disc_get_status (BraseroDisc *disc)
 	BraseroDataDiscPrivate *priv;
 
 	priv = BRASERO_DATA_DISC_PRIVATE (disc);
+
 	if (priv->loading)
 		return BRASERO_DISC_LOADING;
 
-	if (brasero_data_project_is_empty (priv->project))
-		return BRASERO_DISC_ERROR_EMPTY_SELECTION;
-	
+	/* This one goes before the next since a node may be loading but not
+	 * yet in the project and therefore project will look empty */
 	if (brasero_data_vfs_is_active (BRASERO_DATA_VFS (priv->project)))
 		return BRASERO_DISC_NOT_READY;
 
+	if (brasero_data_project_is_empty (priv->project))
+		return BRASERO_DISC_ERROR_EMPTY_SELECTION;
+
 	return BRASERO_DISC_OK;
 }
 
@@ -1441,16 +1745,6 @@ brasero_data_disc_get_selected_uri (BraseroDisc *disc,
 	return TRUE;
 }
 
-static void
-brasero_data_disc_set_drive (BraseroDisc *disc, BraseroDrive *drive)
-{
-	BraseroDataDiscPrivate *priv;
-
-	priv = BRASERO_DATA_DISC_PRIVATE (disc);
-
-	brasero_data_session_set_drive (BRASERO_DATA_SESSION (priv->project), drive);
-}
-
 static guint
 brasero_data_disc_add_ui (BraseroDisc *disc,
 			  GtkUIManager *manager,
@@ -1462,7 +1756,6 @@ brasero_data_disc_add_ui (BraseroDisc *disc,
 	guint merge_id;
 
 	priv = BRASERO_DATA_DISC_PRIVATE (disc);
-
 	if (priv->message) {
 		g_object_unref (priv->message);
 		priv->message = NULL;
@@ -1478,39 +1771,56 @@ brasero_data_disc_add_ui (BraseroDisc *disc,
 					      entries,
 					      G_N_ELEMENTS (entries),
 					      disc);
-		gtk_action_group_add_toggle_actions (priv->disc_group,
-						     toggle_entries,
-						     G_N_ELEMENTS (toggle_entries),
-						     disc);
 		gtk_ui_manager_insert_action_group (manager,
 						    priv->disc_group,
 						    0);
+
+		merge_id = gtk_ui_manager_add_ui_from_string (manager,
+							      description,
+							      -1,
+							      &error);
+		if (!merge_id) {
+			BRASERO_BURN_LOG ("Adding ui elements failed: %s", error->message);
+			g_error_free (error);
+			return 0;
+		}
+
+		action = gtk_action_group_get_action (priv->disc_group, "NewFolder");
+		g_object_set (action,
+			      "short-label", _("New Folder"), /* for toolbar buttons */
+			      NULL);
+	
+		priv->manager = manager;
+		g_object_ref (manager);
 	}
+	else
+		gtk_action_group_set_visible (priv->disc_group, TRUE);
 
-	merge_id = gtk_ui_manager_add_ui_from_string (manager,
-						      description,
-						      -1,
-						      &error);
-	if (!merge_id) {
-		BRASERO_BURN_LOG ("Adding ui elements failed: %s", error->message);
-		g_error_free (error);
-		return 0;
+	/* Now let's take care of all the available sessions */
+	if (!priv->import_group) {
+		GSList *iter;
+		GSList *list;
+
+		priv->import_group = gtk_action_group_new ("session_import_group");
+		gtk_action_group_set_translation_domain (priv->import_group, GETTEXT_PACKAGE);
+		gtk_ui_manager_insert_action_group (manager,
+						    priv->import_group,
+						    0);
+
+		list = brasero_data_session_get_available_media (BRASERO_DATA_SESSION (priv->project));
+		for (iter = list; iter; iter = iter->next) {
+			BraseroMedium *medium;
+
+			medium = iter->data;
+			brasero_data_disc_import_button_new (BRASERO_DATA_DISC (disc), medium);
+		}
+		g_slist_foreach (list, (GFunc) g_object_unref, NULL);
+		g_slist_free (list);
 	}
+	else
+		gtk_action_group_set_visible (priv->import_group, TRUE);
 
-	action = gtk_action_group_get_action (priv->disc_group, "ImportSession");
-	gtk_action_set_sensitive (action, FALSE);
-	g_object_set (action,
-		      "short-label", _("Import"), /* for toolbar buttons */
-		      NULL);
-
-	action = gtk_action_group_get_action (priv->disc_group, "NewFolder");
-	g_object_set (action,
-		      "short-label", _("New Folder"), /* for toolbar buttons */
-		      NULL);
-
-	priv->manager = manager;
-	g_object_ref (manager);
-	return merge_id;
+	return -1;
 }
 
 /**
@@ -1627,6 +1937,7 @@ brasero_data_disc_rename_activated (BraseroDataDisc *disc)
 		gtk_tree_path_free (treepath);
 	}
 	else {
+		gchar *string;
 		GtkWidget *frame;
 		GtkWidget *dialog;
 		GtkWidget *rename;
@@ -1644,7 +1955,10 @@ brasero_data_disc_rename_activated (BraseroDataDisc *disc)
 		brasero_rename_set_show_keep_default (BRASERO_RENAME (rename), FALSE);
 		gtk_widget_show (rename);
 
-		frame = brasero_utils_pack_properties (_("<b>Renaming mode</b>"), rename, NULL);
+		string = g_strdup_printf ("<b>%s</b>", _("Renaming mode"));
+		frame = brasero_utils_pack_properties (string, rename, NULL);
+		g_free (string);
+
 		gtk_widget_show (frame);
 
 		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), frame, TRUE, TRUE, 0);
@@ -2027,6 +2341,11 @@ brasero_data_disc_init (BraseroDataDisc *object)
 			  "project-loaded",
 			  G_CALLBACK (brasero_data_disc_project_loaded_cb),
 			  object);
+	g_signal_connect (priv->project,
+			  "oversize",
+			  G_CALLBACK (brasero_data_disc_project_oversized_cb),
+			  object);
+
 	g_signal_connect (priv->project,
 			  "row-inserted",
 			  G_CALLBACK (brasero_data_disc_contents_added_cb),
@@ -2044,6 +2363,10 @@ brasero_data_disc_init (BraseroDataDisc *object)
 			  "filtered-uri",
 			  G_CALLBACK (brasero_data_disc_filtered_uri_cb),
 			  object);
+	g_signal_connect (priv->project,
+			  "image-uri",
+			  G_CALLBACK (brasero_data_disc_image_uri_cb),
+			  object);
 	g_signal_connect (priv->project,
 			  "unreadable-uri",
 			  G_CALLBACK (brasero_data_disc_unreadable_uri_cb),
@@ -2270,7 +2593,6 @@ brasero_data_disc_iface_disc_init (BraseroDiscIface *iface)
 	iface->get_status = brasero_data_disc_get_status;
 	iface->get_selected_uri = brasero_data_disc_get_selected_uri;
 	iface->add_ui = brasero_data_disc_add_ui;
-	iface->set_drive = brasero_data_disc_set_drive;
 }
 
 static void
