@@ -212,13 +212,16 @@ brasero_data_tree_model_iter_has_child (GtkTreeModel *model,
 	}
 
 	node = iter->user_data;
-	if (node->is_file)
+
+	/* This is a workaround for a warning in gailtreeview.c line 2946 where
+	 * gail uses the GtkTreePath and not a copy which if the node inserted
+	 * declares to have children and is not expanded leads to the path being
+	 * upped and therefore wrong. */
+	if (node->is_inserting)
 		return FALSE;
 
-	if (!BRASERO_FILE_NODE_CHILDREN (node)) {
-		/* It has children but only a bogus one. */
-		return TRUE;
-	}
+	if (node->is_file)
+		return FALSE;
 
 	/* always return TRUE here when it's a directory since even if
 	 * it's empty we'll add a row written empty underneath it
@@ -241,7 +244,7 @@ brasero_data_tree_model_iter_children (GtkTreeModel *model,
 
 		/* This is for the top directory */
 		root = brasero_data_project_get_root (BRASERO_DATA_PROJECT (model));
-		if (!BRASERO_FILE_NODE_CHILDREN (root))
+		if (!root || !BRASERO_FILE_NODE_CHILDREN (root))
 			return FALSE;
 
 		iter->stamp = priv->stamp;
@@ -344,8 +347,14 @@ brasero_data_tree_model_node_shown (GtkTreeModel *model,
 
 	node->is_visible ++;
 
-	if (node->is_imported)
+	if (node->is_imported) {
+		if (node->is_fake && !node->is_file) {
+			/* we don't load all nodes when importing a session do it now */
+			brasero_data_session_load_directory_contents (BRASERO_DATA_SESSION (model), node, NULL);
+		}
+
 		return;
+	}
 
 	if (node->is_visible > 1)
 		return;
@@ -638,30 +647,6 @@ brasero_data_tree_model_get_value (GtkTreeModel *model,
 	return;
 }
 
-/**
- * This is a function mainly used at project load time. In this context there
- * can be nodes that have been added to the data project tree but not added 
- * through the model. Don't count those nodes.
- */
-static guint
-brasero_data_tree_model_node_index (BraseroFileNode *node)
-{
-	BraseroFileNode *parent;
-	BraseroFileNode *peers;
-	guint pos = 0;
-
-	parent = node->parent;
-	for (peers = BRASERO_FILE_NODE_CHILDREN (parent); peers; peers = peers->next) {
-		if (peers == node)
-			break;
-		if (!peers->is_visible)
-			continue;
-		pos ++;
-	}
-
-	return pos;
-}
-
 GtkTreePath *
 brasero_data_tree_model_node_to_path (BraseroDataTreeModel *self,
 				      BraseroFileNode *node)
@@ -675,7 +660,7 @@ brasero_data_tree_model_node_to_path (BraseroDataTreeModel *self,
 	for (; node->parent && !node->is_root; node = node->parent) {
 		guint nth;
 
-		nth = brasero_data_tree_model_node_index (node);
+		nth = brasero_file_node_get_pos_as_child (node);
 		gtk_tree_path_prepend_index (path, nth);
 	}
 
@@ -753,6 +738,10 @@ brasero_data_tree_model_get_iter (GtkTreeModel *model,
 	depth = gtk_tree_path_get_depth (path);
 
 	root = brasero_data_project_get_root (BRASERO_DATA_PROJECT (model));
+	/* NOTE: if we're in reset, then root won't exist anymore */
+	if (!root)
+		return FALSE;
+		
 	node = brasero_file_node_nth_child (root, indices [0]);
 	if (!node)
 		return FALSE;
@@ -1205,9 +1194,15 @@ brasero_data_tree_model_node_added (BraseroDataProject *project,
 	}
 
 	/* Add the row itself */
+	/* This is a workaround for a warning in gailtreeview.c line 2946 where
+	 * gail uses the GtkTreePath and not a copy which if the node inserted
+	 * declares to have children and is not expanded leads to the path being
+	 * upped and therefore wrong. */
+	node->is_inserting = 1;
 	gtk_tree_model_row_inserted (GTK_TREE_MODEL (project),
 				     path,
 				     &iter);
+	node->is_inserting = 0;
 	gtk_tree_path_free (path);
 
 	parent = node->parent;
@@ -1243,8 +1238,8 @@ brasero_data_tree_model_node_added (BraseroDataProject *project,
 	/* we also have to set the is_visible property as all nodes added to 
 	 * root are always visible but ref_node is not necessarily called on
 	 * these nodes. */
-	if (parent->is_root)
-		node->is_visible = TRUE;
+//	if (parent->is_root)
+//		node->is_visible = TRUE;
 
 end:
 	/* chain up this function */
@@ -1352,7 +1347,19 @@ brasero_data_tree_model_node_changed (BraseroDataProject *project,
 		/* emit child-toggled. Thanks to bogus rows we only need to emit
 		 * this signal once since a directory will always have a child
 		 * in the tree */
-		gtk_tree_model_row_has_child_toggled (GTK_TREE_MODEL (project), path, &iter);
+		gtk_tree_model_row_has_child_toggled (GTK_TREE_MODEL (project),
+						      path,
+						      &iter);
+
+		/* The problem is that without that, the folder contents on disc
+		 * won't be added to the tree if the node it replaced was
+		 * already visible. */
+		if (node->is_imported
+		&&  node->is_visible
+		&&  node->is_fake)
+			brasero_data_session_load_directory_contents (BRASERO_DATA_SESSION (project),
+								      node,
+								      NULL);
 
 		/* add the row */
 		if (!BRASERO_FILE_NODE_CHILDREN (node))  {
