@@ -71,6 +71,8 @@ struct BraseroFileChooserPrivate {
 	GtkFileFilter *filter_any;
 	GtkFileFilter *filter_audio;
 	GtkFileFilter *filter_video;
+
+	BraseroLayoutType type;
 };
 
 static GObjectClass *parent_class = NULL;
@@ -375,6 +377,9 @@ brasero_file_chooser_set_context (BraseroLayoutObject *object,
 	BraseroFileChooser *self;
 
 	self = BRASERO_FILE_CHOOSER (object);
+	if (type == self->priv->type)
+		return;
+
 	if (type == BRASERO_LAYOUT_AUDIO)
 		gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (self->priv->chooser),
 					     self->priv->filter_audio);
@@ -384,6 +389,8 @@ brasero_file_chooser_set_context (BraseroLayoutObject *object,
 	else
 		gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (self->priv->chooser),
 					     self->priv->filter_any);
+
+	self->priv->type = type;
 }
 
 static void
