@@ -97,26 +97,22 @@ G_DEFINE_TYPE (BraseroFileFiltered, brasero_file_filtered, GTK_TYPE_EXPANDER);
 static gchar *
 brasero_file_filtered_get_label_text (guint num, gboolean expanded)
 {
-	gchar *markup;
 	gchar *label;
 
 	if (expanded) {
 		if (!num)
-			label = g_strdup (_("Hide the _filtered file list (no file)"));
+			label = g_strdup (_("No file filtered"));
 		else
 			label = g_strdup_printf (ngettext ("Hide the _filtered file list (%d file)", "Hide the _filtered file list (%d files)", num), num);
 	}
 	else {
 		if (!num)
-			label = g_strdup (_("Show the _filtered file list (no file)"));
+			label = g_strdup (_("No file filtered"));
 		else
 			label = g_strdup_printf (ngettext ("Show the _filtered file list (%d file)", "Show the _filtered file list (%d files)", num), num);
 	}
 
-	markup = g_strdup_printf ("<span weight=\"bold\" size=\"medium\">%s</span>", label);
-	g_free (label);
-
-	return markup;
+	return label;
 }
 
 static void
@@ -377,6 +373,8 @@ brasero_file_filtered_restore_pressed_cb (GtkButton *button,
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->tree));
 	selected = gtk_tree_selection_get_selected_rows (selection, &model);
+
+	/* reverse the list so as to be able to delete properly item */
 	selected = g_list_reverse (selected);
 
 	for (iter = selected; iter; iter = iter->next) {
@@ -386,9 +384,14 @@ brasero_file_filtered_restore_pressed_cb (GtkButton *button,
 		gchar *uri;
 
 		treepath = iter->data;
-		gtk_tree_model_get_iter (model, &treeiter, treepath);
+		if (!gtk_tree_model_get_iter (model, &treeiter, treepath)) {
+			gtk_tree_path_free (treepath);
+			continue;
+		}
+
 		gtk_tree_path_free (treepath);
 
+		uri = NULL;
 		gtk_tree_model_get (model, &treeiter,
 				    UNESCAPED_URI_COL, &uri, 
 				    -1);
