@@ -33,6 +33,7 @@
 
 #include "brasero-jacket-view.h"
 #include "brasero-jacket-buffer.h"
+#include "brasero-utils.h"
 
 typedef struct _BraseroJacketViewPrivate BraseroJacketViewPrivate;
 struct _BraseroJacketViewPrivate
@@ -613,9 +614,15 @@ brasero_jacket_view_focus_in_cb (GtkWidget *view,
 				 GdkEventFocus *event,
 				 BraseroJacketView *self)
 {
+	GtkTextView *text_view = GTK_TEXT_VIEW (view);
 	GtkTextBuffer *buffer;
 
-	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
+	if (text_view->editable) {
+		text_view->need_im_reset = TRUE;
+		gtk_im_context_focus_in (text_view->im_context);
+	}
+
+	buffer = gtk_text_view_get_buffer (text_view);
 	brasero_jacket_buffer_show_default_text (BRASERO_JACKET_BUFFER (buffer), FALSE);
 
 	g_signal_emit (self,
@@ -628,9 +635,15 @@ brasero_jacket_view_focus_out_cb (GtkWidget *view,
 				  GdkEventFocus *event,
 				  BraseroJacketView *self)
 {
+	GtkTextView *text_view = GTK_TEXT_VIEW (view);
 	GtkTextBuffer *buffer;
 
-	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
+	if (text_view->editable) {
+		text_view->need_im_reset = TRUE;
+		gtk_im_context_focus_out (text_view->im_context);
+	}
+
+	buffer = gtk_text_view_get_buffer (text_view);
 	brasero_jacket_buffer_show_default_text (BRASERO_JACKET_BUFFER (buffer), TRUE);
 
 	g_signal_emit (self,
@@ -1078,17 +1091,11 @@ brasero_jacket_view_set_image (BraseroJacketView *self,
 
 	image = gdk_pixbuf_new_from_file (path, &error);
 	if (error) {
-		GtkWidget *message;
-
-		message = gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (self))),
-						  GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT,
-						  GTK_MESSAGE_ERROR,
-						  GTK_BUTTONS_CLOSE,
-						  _("The image couldn't be loaded:"));
-		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
-							  error->message);
-		gtk_dialog_run (GTK_DIALOG (message));
-		gtk_widget_destroy (message);
+		brasero_utils_message_dialog (gtk_widget_get_toplevel (GTK_WIDGET (self)),
+					      _("The image couldn't be loaded:"),
+					      error->message,
+					      GTK_MESSAGE_ERROR);
+		g_error_free (error);
 		return priv->image_path;
 	}
 
@@ -1442,9 +1449,6 @@ brasero_jacket_view_container_forall (GtkContainer *container,
 {
 	BraseroJacketViewPrivate *priv;
 
-	if (!include_internals)
-		return;
-
 	priv = BRASERO_JACKET_VIEW_PRIVATE (container);
 	if (priv->edit)
 		callback (priv->edit, callback_data);
@@ -1520,7 +1524,9 @@ brasero_jacket_view_init (BraseroJacketView *object)
 			  G_CALLBACK (brasero_jacket_view_scrolled_cb),
 			  priv->edit);
 
-	gtk_widget_set_scroll_adjustments (priv->edit, GTK_ADJUSTMENT (hadj), GTK_ADJUSTMENT (vadj));
+	gtk_widget_set_scroll_adjustments (priv->edit,
+					   GTK_ADJUSTMENT (hadj),
+					   GTK_ADJUSTMENT (vadj));
 }
 
 static void
