@@ -66,9 +66,10 @@
 #include "brasero-player.h"
 #endif
 
+#include "brasero-app.h"
 #include "brasero-project.h"
+#include "brasero-session-cfg.h"
 #include "brasero-jacket-edit.h"
-#include "brasero-project-size.h"
 #include "brasero-project-type-chooser.h"
 #include "brasero-disc.h"
 #include "brasero-data-disc.h"
@@ -82,6 +83,8 @@
 #include "brasero-disc-message.h"
 #include "brasero-file-chooser.h"
 #include "brasero-notify.h"
+#include "brasero-burn-options.h"
+#include "brasero-project-name.h"
 
 static void brasero_project_class_init (BraseroProjectClass *klass);
 static void brasero_project_init (BraseroProject *sp);
@@ -104,9 +107,6 @@ brasero_project_empty_cb (GtkAction *action, BraseroProject *project);
 static void
 brasero_project_burn_cb (GtkAction *action, BraseroProject *project);
 
-static void
-brasero_project_disc_changed_cb (BraseroProjectSize *size,
-				 BraseroProject *project);
 static void
 brasero_project_size_changed_cb (BraseroDisc *disc,
 			         gint64 size,
@@ -156,7 +156,7 @@ typedef enum {
 } BraseroProjectSave;
 
 struct BraseroProjectPrivate {
-	GtkWidget *size_display;
+	GtkWidget *name_display;
 	GtkWidget *discs;
 	GtkWidget *audio;
 	GtkWidget *data;
@@ -166,6 +166,8 @@ struct BraseroProjectPrivate {
 
 	GtkUIManager *manager;
 
+	guint status_ctx;
+
 	/* header */
 	GtkWidget *burn;
 
@@ -191,8 +193,9 @@ struct BraseroProjectPrivate {
 	guint modified:1;
 	guint has_focus:1;
 	guint oversized:1;
-	guint ask_overburn:1;
 	guint selected_uris:1;
+
+	guint merging:1;
 };
 
 static GtkActionEntry entries [] = {
@@ -204,6 +207,7 @@ static GtkActionEntry entries [] = {
 	 N_("Add files to the project"), G_CALLBACK (brasero_project_add_uris_cb)},
 	{"DeleteProject", GTK_STOCK_REMOVE, N_("_Remove Files"), NULL,
 	 N_("Remove the selected files from the project"), G_CALLBACK (brasero_project_remove_selected_uris_cb)},
+	/* Translators: "empty" is a verb here */
 	{"DeleteAll", GTK_STOCK_CLEAR, N_("E_mpty Project"), NULL,
 	 N_("Remove all files from the project"), G_CALLBACK (brasero_project_empty_cb)},
 	{"Burn", "media-optical-burn", N_("_Burn..."), NULL,
@@ -250,14 +254,8 @@ static const gchar *description = {
 
 static GObjectClass *parent_class = NULL;
 
-#define BRASERO_PROJECT_SPACING			6
 #define BRASERO_PROJECT_SIZE_WIDGET_BORDER	1
 
-#define KEY_DEFAULT_DATA_BURNING_APP		"/desktop/gnome/volume_manager/autoburn_data_cd_command"
-#define KEY_DEFAULT_AUDIO_BURNING_APP		"/desktop/gnome/volume_manager/autoburn_audio_cd_command"
-#define KEY_ASK_DEFAULT_BURNING_AUDIO		"/apps/brasero/ask_default_audio"
-#define KEY_ASK_DEFAULT_BURNING_DATA		"/apps/brasero/ask_default_data"
-
 #define BRASERO_KEY_SHOW_PREVIEW		"/apps/brasero/display/viewer"
 
 #define BRASERO_PROJECT_VERSION "0.2"
@@ -338,8 +336,10 @@ brasero_project_get_proportion (BraseroLayoutObject *object,
 				gint *center,
 				gint *footer)
 {
-	*footer = BRASERO_PROJECT (object)->priv->size_display->allocation.height + 
-		  BRASERO_PROJECT_SPACING * 2 + BRASERO_PROJECT_SIZE_WIDGET_BORDER * 2;
+	if (!BRASERO_PROJECT (object)->priv->name_display)
+		return;
+
+	*footer = BRASERO_PROJECT (object)->priv->name_display->parent->allocation.height;
 }
 
 static void
@@ -360,13 +360,29 @@ static void
 brasero_project_set_add_button_state (BraseroProject *project)
 {
 	GtkAction *action;
+	GtkWidget *widget;
 	gboolean sensitive;
+	GtkWidget *toplevel;
 
 	sensitive = ((!project->priv->current_source || !project->priv->has_focus) &&
 		      !project->priv->oversized);
 
 	action = gtk_action_group_get_action (project->priv->project_group, "Add");
 	gtk_action_set_sensitive (action, sensitive);
+
+	/* set the Add button to be the default for the whole window. That fixes 
+	 * #465175 – Location field not working. GtkFileChooser needs a default
+	 * widget to be activated. */
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (project));
+	if (!sensitive) {
+		gtk_window_set_default (GTK_WINDOW (toplevel), NULL);
+		return;
+	}
+
+	widget = gtk_ui_manager_get_widget (project->priv->manager, "/Toolbar/Add");
+	widget = gtk_bin_get_child (GTK_BIN (widget));
+	GTK_WIDGET_SET_FLAGS (widget, GTK_CAN_DEFAULT);
+	gtk_window_set_default (GTK_WINDOW (toplevel), widget);
 }
 
 static void
@@ -383,12 +399,28 @@ brasero_project_focus_changed_cb (GtkContainer *container,
 	brasero_project_set_add_button_state (project);
 }
 
+static void
+brasero_project_name_changed_cb (BraseroProjectName *name,
+				 BraseroProject *project)
+{
+	GtkAction *action;
+
+	project->priv->modified = TRUE;
+
+	/* the state of the following depends on the existence of an opened project */
+	action = gtk_action_group_get_action (project->priv->project_group, "Save");
+	if (project->priv->modified)
+		gtk_action_set_sensitive (action, TRUE);
+	else
+		gtk_action_set_sensitive (action, FALSE);
+}
+
 static void
 brasero_project_init (BraseroProject *obj)
 {
 	GtkSizeGroup *size_group;
 	GtkWidget *alignment;
-	GtkWidget *vbox;
+	GtkWidget *label;
 	GtkWidget *box;
 
 	obj->priv = g_new0 (BraseroProjectPrivate, 1);
@@ -404,24 +436,29 @@ brasero_project_init (BraseroProject *obj)
 
 	/* bottom */
 	box = gtk_hbox_new (FALSE, 6);
+	gtk_container_set_border_width (GTK_CONTAINER (box), 4);
 	gtk_widget_show (box);
-	gtk_box_pack_end (GTK_BOX (obj), box, FALSE, FALSE, BRASERO_PROJECT_SPACING);
+	gtk_box_pack_end (GTK_BOX (obj), box, FALSE, TRUE, 0);
 
-	/* size widget */
-	vbox = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (vbox);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), BRASERO_PROJECT_SIZE_WIDGET_BORDER);
-	gtk_box_pack_start (GTK_BOX (box), vbox, TRUE, TRUE, 0);
+	/* Name widget */
+	label = gtk_label_new_with_mnemonic (_("_Name:"));
+	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+	gtk_misc_set_padding (GTK_MISC (label), 6, 0);
+	gtk_widget_show (label);
+	gtk_box_pack_start (GTK_BOX (box), label, FALSE, TRUE, 0);
 
-	obj->priv->size_display = brasero_project_size_new ();
-	gtk_widget_show (obj->priv->size_display);
-	g_signal_connect (G_OBJECT (obj->priv->size_display), 
-			  "disc-changed",
-			  G_CALLBACK (brasero_project_disc_changed_cb),
-			  obj);
-	gtk_box_pack_start (GTK_BOX (vbox), obj->priv->size_display, TRUE, TRUE, 0);
+	obj->priv->name_display = brasero_project_name_new ();
+	gtk_widget_show (obj->priv->name_display);
+	gtk_box_pack_start (GTK_BOX (box), obj->priv->name_display, TRUE, TRUE, 0);
 	obj->priv->empty = 1;
-	
+
+	g_signal_connect (obj->priv->name_display,
+			  "name-changed",
+			  G_CALLBACK (brasero_project_name_changed_cb),
+			  obj);
+
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), obj->priv->name_display);
+
 	/* burn button set insensitive since there are no files in the selection */
 	size_group = gtk_size_group_new (GTK_SIZE_GROUP_BOTH);
 	obj->priv->burn = brasero_utils_make_button (_("_Burn..."),
@@ -439,10 +476,10 @@ brasero_project_init (BraseroProject *obj)
 				     _("Start to burn the contents of the selection"));
 	gtk_size_group_add_widget (GTK_SIZE_GROUP (size_group), obj->priv->burn);
 
-	alignment = gtk_alignment_new (1.0, 0.0, 0.0, 0.0);
+	alignment = gtk_alignment_new (1.0, 0.5, 0.0, 0.0);
 	gtk_widget_show (alignment);
 	gtk_container_add (GTK_CONTAINER (alignment), obj->priv->burn);
-	gtk_box_pack_end (GTK_BOX (box), alignment, FALSE, FALSE, 0);
+	gtk_box_pack_end (GTK_BOX (box), alignment, FALSE, TRUE, 0);
 
 	/* The three panes to put into the notebook */
 	obj->priv->audio = brasero_audio_disc_new ();
@@ -542,115 +579,51 @@ brasero_project_new ()
 
 /********************************** size ***************************************/
 static void
-brasero_project_error_size_dialog (BraseroProject *project)
+brasero_project_update_project_size (BraseroProject *project,
+				     guint64 sectors)
 {
-	GtkWidget *message;
+	GtkWidget *toplevel;
+	GtkWidget *status;
+	gchar *string;
+	gchar *size;
 
-	message = brasero_notify_message_add (BRASERO_NOTIFY (project->priv->message),
-					      _("Please, remove some files from the project."),
-					      _("The size of the project is too large for the disc even with the overburn option."),
-					      -1,
-					      BRASERO_NOTIFY_CONTEXT_SIZE);
-	brasero_disc_message_set_image (BRASERO_DISC_MESSAGE (message), GTK_STOCK_DIALOG_WARNING);
-	brasero_disc_message_add_close_button (BRASERO_DISC_MESSAGE (message));
-}
-
-static gboolean
-brasero_project_overburn_dialog (BraseroProject *project)
-{
-	GtkWidget *dialog, *toplevel;
-	gint result;
-
-	/* get the current CD length and make sure selection is not too long */
 	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (project));
-	dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel),
-					 GTK_DIALOG_DESTROY_WITH_PARENT|
-					 GTK_DIALOG_MODAL,
-					 GTK_MESSAGE_WARNING,
-					 GTK_BUTTONS_NONE,
-					 _("Would you like to activate overburn?"));
+	status = brasero_app_get_statusbar2 (BRASERO_APP (toplevel));
 
-	gtk_window_set_title (GTK_WINDOW (dialog), _("Project Size"));
+	if (!project->priv->status_ctx)
+		project->priv->status_ctx = gtk_statusbar_get_context_id (GTK_STATUSBAR (status),
+									  "size_project");
 
-	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-						  _("The size of the project is too large for the disc and you must remove files otherwise."
-						    "\nNOTE: This option might cause failure."));
+	gtk_statusbar_pop (GTK_STATUSBAR (status), project->priv->status_ctx);
 
-	gtk_dialog_add_button (GTK_DIALOG (dialog),
-			       _("_Don't use overburn"),
-			       GTK_RESPONSE_CANCEL);
-	gtk_dialog_add_button (GTK_DIALOG (dialog),
-			       _("Use _overburn"),
-			       GTK_RESPONSE_YES);
+	string = brasero_utils_get_sectors_string (sectors,
+						   !BRASERO_IS_DATA_DISC (project->priv->current),
+						   TRUE,
+						   FALSE);
+	if (project->priv->merging) {
+		gchar *medium_string;
+		BraseroMedium *medium;
+		gint64 free_space = 0;
 
-	result = gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (dialog);
-
-	if (result == GTK_RESPONSE_YES)
-		return TRUE;
-
-	return FALSE;
-}
-
-static void
-brasero_project_check_size (BraseroProject *project)
-{
-	gboolean result;
-	gboolean overburn;
-	GtkAction *action;
-	gboolean sensitive;
-
-	result = brasero_project_size_check_status (BRASERO_PROJECT_SIZE (project->priv->size_display),
-						    &overburn);
-
-	if (result) {
-		brasero_notify_message_remove (BRASERO_NOTIFY (project->priv->message), BRASERO_NOTIFY_CONTEXT_SIZE);
-		project->priv->oversized = 0;
-		goto end;
+		medium = brasero_data_disc_get_loaded_medium (BRASERO_DATA_DISC (project->priv->current));
+		brasero_medium_get_free_space (medium,
+					       &free_space,
+					       NULL);
+		/* Translators: first %s is the size of the project and the 
+		 * second %s is the remaining free space on the disc that is
+		 * used for multisession */
+		medium_string = g_format_size_for_display (free_space);
+		size = g_strdup_printf (_("Project estimated size: %s/%s"),
+					string,
+					medium_string);
+		g_free (medium_string);
 	}
+	else
+		size = g_strdup_printf (_("Project estimated size: %s"), string);
+	g_free (string);
 
-	if (project->priv->is_burning) {
-		/* we don't want to show the following dialog while burning */
-		if (overburn)
-			project->priv->oversized = 0;
-		else
-			project->priv->oversized = 1;
-		return;
-	}
-
-	if (overburn) {
-		brasero_notify_message_remove (BRASERO_NOTIFY (project->priv->message), BRASERO_NOTIFY_CONTEXT_SIZE);
-		project->priv->oversized = 0;
-		goto end;
-	}
-
-	/* avoid telling the user the same thing twice */
-	if (project->priv->oversized)
-		goto end;
-
-	project->priv->oversized = 1;
-	brasero_project_error_size_dialog (project);
-
-
-end:
-	if (project->priv->oversized) {
-		sensitive = FALSE;
-		g_object_set (G_OBJECT (project->priv->current), "reject-file", TRUE, NULL);
-	}
-	else {
-		sensitive = TRUE;
-		g_object_set (G_OBJECT (project->priv->current), "reject-file", FALSE, NULL);
-	}
-
-	brasero_project_set_add_button_state (project);
-
-	/* we need to make sure there is actually something to burn */
-	sensitive = (project->priv->empty == FALSE &&
-		     project->priv->oversized == FALSE);
-
-	action = gtk_action_group_get_action (project->priv->project_group, "Burn");
-	gtk_action_set_sensitive (action, sensitive);
-	gtk_widget_set_sensitive (project->priv->burn, sensitive);
+	gtk_statusbar_push (GTK_STATUSBAR (status), project->priv->status_ctx, size);
+	g_free (size);
 }
 
 static void
@@ -659,11 +632,7 @@ brasero_project_size_changed_cb (BraseroDisc *disc,
 			         BraseroProject *project)
 {
 	project->priv->sectors = sectors;
-
-	brasero_project_size_set_sectors (BRASERO_PROJECT_SIZE (project->priv->size_display),
-					  sectors);
-
-	brasero_project_check_size (project);
+	brasero_project_update_project_size (project, sectors);
 }
 
 static void
@@ -671,30 +640,17 @@ brasero_project_flags_changed_cb (BraseroDisc *disc,
 				  BraseroBurnFlag flags,
 				  BraseroProject *project)
 {
-	brasero_project_size_set_multisession (BRASERO_PROJECT_SIZE (project->priv->size_display),
-					      (flags & BRASERO_BURN_FLAG_MERGE) != 0);
-}
+	gboolean merging;
 
-static void
-brasero_project_disc_changed_cb (BraseroProjectSize *size,
-				 BraseroProject *project)
-{
-	BraseroMedium *medium;
-	BraseroDrive *drive;
+	merging = (flags & BRASERO_BURN_FLAG_MERGE) != 0;
 
-	brasero_project_check_size (project);
+	/* see if the project name should be updated */
+	brasero_project_name_set_multisession_medium (BRASERO_PROJECT_NAME (project->priv->name_display),
+						      brasero_data_disc_get_loaded_medium (BRASERO_DATA_DISC (disc)));
 
-	/* get the current device name and set it for the disc project in
-	 * case that is a multisession disc and a data project */
-	medium = brasero_project_size_get_active_medium (size);
-	if (!medium) {
-		brasero_disc_set_current_drive (project->priv->current, NULL);
-		return;
-	}
-
-	drive = brasero_medium_get_drive (medium);
-	brasero_disc_set_current_drive (project->priv->current, drive);
-	g_object_unref (medium);
+	/* we just need to know if MERGE flag is on */
+	project->priv->merging = merging;
+	brasero_project_update_project_size (project, project->priv->sectors);
 }
 
 /***************************** URIContainer ************************************/
@@ -844,7 +800,7 @@ brasero_project_set_cover_specifics (BraseroProject *self,
 	if (!BRASERO_IS_AUDIO_DISC (self->priv->current))
 		return;
 
-	session = brasero_burn_session_new ();
+	session = BRASERO_BURN_SESSION (brasero_session_cfg_new ());
 	brasero_disc_set_session_param (BRASERO_DISC (self->priv->current), session);
 	brasero_disc_set_session_contents (BRASERO_DISC (self->priv->current), session);
 	brasero_jacket_edit_set_audio_tracks (BRASERO_JACKET_EDIT (cover),
@@ -857,126 +813,19 @@ brasero_project_set_cover_specifics (BraseroProject *self,
 static void
 brasero_project_no_song_dialog (BraseroProject *project)
 {
-	GtkWidget *message;
-	GtkWidget *toplevel;
-
-	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (project));
-	message = gtk_message_dialog_new (GTK_WINDOW (toplevel),
-					  GTK_DIALOG_DESTROY_WITH_PARENT|
-					  GTK_DIALOG_MODAL,
-					  GTK_MESSAGE_WARNING,
-					  GTK_BUTTONS_CLOSE,
-					  _("Please add songs to the project."));
-
-	gtk_window_set_title (GTK_WINDOW (message), _("Empty Project"));
-
-	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
-						  _("The project is empty."));
-
-	gtk_dialog_run (GTK_DIALOG (message));
-	gtk_widget_destroy (message);
+	brasero_utils_message_dialog (gtk_widget_get_toplevel (GTK_WIDGET (project)),
+				      _("Please add songs to the project."),
+				      _("The project is empty."),
+				      GTK_MESSAGE_WARNING);
 }
 
 static void
 brasero_project_no_file_dialog (BraseroProject *project)
 {
-	GtkWidget *message;
-	GtkWidget *toplevel;
-
-	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (project));
-	message = gtk_message_dialog_new (GTK_WINDOW (toplevel),
-					  GTK_DIALOG_DESTROY_WITH_PARENT|
-					  GTK_DIALOG_MODAL,
-					  GTK_MESSAGE_WARNING,
-					  GTK_BUTTONS_CLOSE,
-					  _("Please add files to the project."));
-
-	gtk_window_set_title (GTK_WINDOW (message), _("Empty Project"));
-
-	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
-						  _("The project is empty."));
-
-	gtk_dialog_run (GTK_DIALOG (message));
-	gtk_widget_destroy (message);
-}
-
-static void
-brasero_project_check_default_burning_app (BraseroProject *project,
-					   const gchar *primary,
-					   const gchar *key,
-					   const gchar *key_ask,
-					   const gchar *default_command)
-{
-	GtkResponseType response;
-	GConfClient *client;
-	GtkWidget *toplevel;
-	GtkWidget *message;
-	gchar *command;
-	gboolean ask;
-	gchar *text;
-
-	client = gconf_client_get_default ();
-	command = gconf_client_get_string (client,
-					   key,
-					   NULL);
-
-	if (command && g_str_has_prefix (command, "brasero")) {
-		g_object_unref (client);
-		g_free (command);
-		return;
-	}
-
-	ask = gconf_client_get_bool (client,
-				     key_ask,
-				     NULL);
-	if (ask) {
-		g_object_unref (client);
-		g_free (command);
-		return;
-	}
-
-	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (project));
-	message = gtk_message_dialog_new (GTK_WINDOW (toplevel),
-					  GTK_DIALOG_MODAL|
-					  GTK_DIALOG_DESTROY_WITH_PARENT,
-					  GTK_MESSAGE_WARNING,
-					  GTK_BUTTONS_NONE,
-					  primary);
-
-	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
-						  _("This is a first time running dialog that won't be shown again. If you change your mind, you can change your choice later in the Removable Drives and Media Preferences whether or not you chose brasero."));
-
-	if (command)
-		/* NOTE for translators the %s is the old application name */
-		text = g_strdup_printf (_("_Keep using \"%s\""), command);
-	else
-		text = g_strdup (GTK_STOCK_CANCEL);
-
-	gtk_dialog_add_button (GTK_DIALOG (message),
-			       text,
-			       GTK_RESPONSE_CANCEL);
-	g_free (text);
-	g_free (command);
-
-	gtk_dialog_add_button (GTK_DIALOG (message),
-			       _("_Use Brasero next time"),
-			       GTK_RESPONSE_YES);
-
-	response = gtk_dialog_run (GTK_DIALOG (message));
-	gconf_client_set_bool (client,
-			       key_ask,
-			       TRUE,
-			       NULL);
-
-	gtk_widget_destroy (message);
-
-	if (response == GTK_RESPONSE_YES)
-		gconf_client_set_string (client,
-					 key,
-					 default_command,
-					 NULL);
-
-	g_object_unref (client);
+	brasero_utils_message_dialog (gtk_widget_get_toplevel (GTK_WIDGET (project)),
+				      _("Please add files to the project."),
+				      _("The project is empty."),
+				      GTK_MESSAGE_WARNING);
 }
 
 void
@@ -986,23 +835,9 @@ brasero_project_burn (BraseroProject *project)
 	BraseroDiscResult result;
 	GtkWidget *toplevel;
 	GtkWidget *dialog;
-	gboolean overburn;
 	gboolean destroy;
 	gboolean success;
 
-	overburn = FALSE;
-	result = brasero_project_size_check_status (BRASERO_PROJECT_SIZE (project->priv->size_display),
-						    &overburn);
-	if (result == FALSE) {
-		if (!overburn) {
-			brasero_project_error_size_dialog (project);
-			return;
-		}
-
-		if (!brasero_project_overburn_dialog (project))
-			return;
-	}
-
 	result = brasero_project_check_status (project, project->priv->current);
 	if (result == BRASERO_DISC_CANCELLED)
 		return;
@@ -1046,8 +881,8 @@ brasero_project_burn (BraseroProject *project)
 	session = brasero_disc_option_dialog_get_session (BRASERO_DISC_OPTION_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
 
-	if (overburn)
-		brasero_burn_session_add_flag (session, BRASERO_BURN_FLAG_OVERBURN);
+	/* set the label for the session */
+	brasero_burn_session_set_label (session, gtk_entry_get_text (GTK_ENTRY (project->priv->name_display)));
 
 	/* now setup the burn dialog */
 	dialog = brasero_burn_dialog_new ();
@@ -1064,21 +899,6 @@ brasero_project_burn (BraseroProject *project)
 
     	project->priv->burnt = success;
 
-	if (success) {
-		if (BRASERO_IS_AUDIO_DISC (project->priv->current))
-			brasero_project_check_default_burning_app (project,
-								   _("Would you like to use Brasero in the future to burn audio discs?"),
-								   KEY_DEFAULT_AUDIO_BURNING_APP,
-								   KEY_ASK_DEFAULT_BURNING_AUDIO,
-								   "brasero -a");
-		else
-			brasero_project_check_default_burning_app (project,
-								   _("Would you like to use Brasero in the future to burn data discs?"),
-								   KEY_DEFAULT_DATA_BURNING_APP,
-								   KEY_ASK_DEFAULT_BURNING_DATA,
-								   "brasero -d");
-	}
-
 end:
 
 	project->priv->is_burning = 0;
@@ -1104,10 +924,9 @@ brasero_project_switch (BraseroProject *project, BraseroProjectType type)
 
 	project->priv->empty = 1;
     	project->priv->burnt = 0;
+	project->priv->merging = 0;
 	project->priv->modified = 0;
 
-	brasero_project_size_set_sectors (BRASERO_PROJECT_SIZE (project->priv->size_display), 0);
-
 	if (project->priv->current)
 		brasero_disc_reset (project->priv->current);
 
@@ -1118,7 +937,7 @@ brasero_project_switch (BraseroProject *project, BraseroProjectType type)
 
 	client = gconf_client_get_default ();
 
-	/* rempove the buttons from the "toolbar" */
+	/* remove the buttons from the "toolbar" */
 	if (project->priv->merge_id)
 		gtk_ui_manager_remove_ui (project->priv->manager,
 					  project->priv->merge_id);
@@ -1130,7 +949,7 @@ brasero_project_switch (BraseroProject *project, BraseroProjectType type)
 							       project->priv->message);
 
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (project->priv->discs), 0);
-		brasero_project_size_set_context (BRASERO_PROJECT_SIZE (project->priv->size_display), BRASERO_PROJECT_TYPE_AUDIO);
+		brasero_project_update_project_size (project, 0);
 	}
 	else if (type == BRASERO_PROJECT_TYPE_DATA) {
 		project->priv->current = BRASERO_DISC (project->priv->data);
@@ -1139,7 +958,7 @@ brasero_project_switch (BraseroProject *project, BraseroProjectType type)
 							       project->priv->message);
 
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (project->priv->discs), 1);
-		brasero_project_size_set_context (BRASERO_PROJECT_SIZE (project->priv->size_display), BRASERO_PROJECT_TYPE_DATA);
+		brasero_project_update_project_size (project, 0);
 	}
 	else if (type == BRASERO_PROJECT_TYPE_VIDEO) {
 		project->priv->current = BRASERO_DISC (project->priv->video);
@@ -1148,7 +967,7 @@ brasero_project_switch (BraseroProject *project, BraseroProjectType type)
 							       project->priv->message);
 
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (project->priv->discs), 2);
-		brasero_project_size_set_context (BRASERO_PROJECT_SIZE (project->priv->size_display), BRASERO_PROJECT_TYPE_VIDEO);
+		brasero_project_update_project_size (project, 0);
 	}
 
 	brasero_notify_message_remove (BRASERO_NOTIFY (project->priv->message), BRASERO_NOTIFY_CONTEXT_SIZE);
@@ -1167,6 +986,8 @@ brasero_project_switch (BraseroProject *project, BraseroProjectType type)
 	gtk_action_set_sensitive (action, TRUE);
 	action = gtk_action_group_get_action (project->priv->project_group, "Save");
 	gtk_action_set_sensitive (action, FALSE);
+
+	brasero_project_name_set_type (BRASERO_PROJECT_NAME (project->priv->name_display), type);
 }
 
 void
@@ -1183,7 +1004,8 @@ brasero_project_set_audio (BraseroProject *project, GSList *uris)
 }
 
 void
-brasero_project_set_data (BraseroProject *project, GSList *uris)
+brasero_project_set_data (BraseroProject *project,
+			  GSList *uris)
 {
 	brasero_project_switch (project, BRASERO_PROJECT_TYPE_DATA);
 
@@ -1274,6 +1096,8 @@ void
 brasero_project_set_none (BraseroProject *project)
 {
 	GtkAction *action;
+	GtkWidget *status;
+	GtkWidget *toplevel;
 
 	if (project->priv->project) {
 		g_free (project->priv->project);
@@ -1305,6 +1129,12 @@ brasero_project_set_none (BraseroProject *project)
 	if (project->priv->merge_id)
 		gtk_ui_manager_remove_ui (project->priv->manager,
 					  project->priv->merge_id);
+
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (project));
+	status = brasero_app_get_statusbar2 (BRASERO_APP (toplevel));
+
+	if (project->priv->status_ctx)
+		gtk_statusbar_pop (GTK_STATUSBAR (status), project->priv->status_ctx);
 }
 
 /********************* update the appearance of menus and buttons **************/
@@ -1443,6 +1273,7 @@ brasero_project_file_chooser_activated_cb (GtkWidget *chooser,
 	g_slist_free (uris);
 
 	gtk_widget_destroy (GTK_WIDGET (project->priv->chooser));
+	project->priv->chooser = NULL;
 }
 
 static void
@@ -1455,6 +1286,7 @@ brasero_project_file_chooser_response_cb (GtkWidget *chooser,
 
 	if (response != BRASERO_RESPONSE_ADD) {
 		gtk_widget_destroy (chooser);
+		project->priv->chooser = NULL;
 		return;
 	}
 
@@ -1469,6 +1301,7 @@ brasero_project_file_chooser_response_cb (GtkWidget *chooser,
 	g_slist_free (uris);
 
 	gtk_widget_destroy (GTK_WIDGET (project->priv->chooser));
+	project->priv->chooser = NULL;
 }
 
 static void
@@ -1492,7 +1325,8 @@ brasero_project_update_preview (GtkFileChooser *chooser,
 }
 
 static void
-brasero_project_add_uris_cb (GtkAction *action, BraseroProject *project)
+brasero_project_add_uris_cb (GtkAction *action,
+			     BraseroProject *project)
 {
 	GtkWidget *toplevel;
 	GtkFileFilter *filter;
@@ -1523,6 +1357,8 @@ brasero_project_add_uris_cb (GtkAction *action, BraseroProject *project)
 	gtk_dialog_add_button (GTK_DIALOG (project->priv->chooser),
 			       GTK_STOCK_ADD,
 			       BRASERO_RESPONSE_ADD);
+	gtk_dialog_set_default_response (GTK_DIALOG (project->priv->chooser),
+					 BRASERO_RESPONSE_ADD);
 
 	g_signal_connect (project->priv->chooser,
 			  "file-activated",
@@ -1627,8 +1463,10 @@ brasero_project_empty_cb (GtkAction *action, BraseroProject *project)
 							    "All the work will be lost. "
 							    "Note that files will not be deleted from their own location, "
 							    "just no longer listed here."));
+		/* Translators: "empty" is a verb here */
 		gtk_dialog_add_button (GTK_DIALOG (dialog),
-				       _("_Empty Project"), GTK_RESPONSE_OK);
+				       _("E_mpty Project"),
+				       GTK_RESPONSE_OK);
 
 		answer = gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
@@ -1760,7 +1598,7 @@ brasero_project_set_uri (BraseroProject *project,
 		title = g_strdup_printf (_("Brasero - %s (Video Disc)"), name);
 	else
 		title = NULL;
-
+ 
 	g_free (name);
 
 	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (project));
@@ -1777,24 +1615,10 @@ static void
 brasero_project_invalid_project_dialog (BraseroProject *project,
 					const char *reason)
 {
-	GtkWidget *dialog;
-	GtkWidget *toplevel;
-
-	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (project));
-	dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel),
-					 GTK_DIALOG_DESTROY_WITH_PARENT |
-					 GTK_DIALOG_MODAL,
-					 GTK_MESSAGE_ERROR,
-					 GTK_BUTTONS_CLOSE,
-					 _("Error while loading the project:"));
-
-	gtk_window_set_title (GTK_WINDOW (dialog), _("Project Loading Error"));
-
-	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-						  reason);
-
-	gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (dialog);
+	brasero_utils_message_dialog (gtk_widget_get_toplevel (GTK_WIDGET (project)),
+				      _("Error while loading the project:"),
+				      reason,
+				      GTK_MESSAGE_ERROR);
 }
 
 static gboolean
@@ -2069,6 +1893,7 @@ error :
 static gboolean
 brasero_project_open_project_xml (BraseroProject *proj,
 				  const gchar *uri,
+				  gchar **label,
 				  BraseroDiscTrack **track,
 				  gboolean warn_user)
 {
@@ -2112,6 +1937,13 @@ brasero_project_open_project_xml (BraseroProject *proj,
 		if (!xmlStrcmp (item->name, (const xmlChar *) "version")) {
 			/* simply ignore it */
 		}
+		else if (!xmlStrcmp (item->name, (const xmlChar *) "label")) {
+			*label = (gchar *) xmlNodeListGetString (project,
+								 item->xmlChildrenNode,
+								 1);
+			if (!(*label))
+				goto error;
+		}
 		else if (!xmlStrcmp (item->name, (const xmlChar *) "track")) {
 			if (track_node)
 				goto error;
@@ -2147,15 +1979,15 @@ brasero_project_open_project (BraseroProject *project,
 {
 	BraseroDiscTrack *track = NULL;
 	BraseroProjectType type;
+	gchar *label = NULL;
 
 	if (!uri || *uri =='\0')
 		return BRASERO_PROJECT_TYPE_INVALID;
 
-	if (!brasero_project_open_project_xml (project, uri, &track, TRUE))
+	if (!brasero_project_open_project_xml (project, uri, &label, &track, TRUE))
 		return BRASERO_PROJECT_TYPE_INVALID;
 
-	brasero_project_size_set_sectors (BRASERO_PROJECT_SIZE (project->priv->size_display),
-					  0);
+	brasero_project_update_project_size (project, 0);
 
 	if (track->type == BRASERO_DISC_TRACK_AUDIO)
 		type = BRASERO_PROJECT_TYPE_AUDIO;
@@ -2170,6 +2002,16 @@ brasero_project_open_project (BraseroProject *project,
 
 	brasero_project_switch (project, type);
 
+	g_signal_handlers_block_by_func (project->priv->name_display,
+					 brasero_project_name_changed_cb,
+					 project);
+	gtk_entry_set_text (GTK_ENTRY (project->priv->name_display), label);
+	g_free (label);
+
+	g_signal_handlers_unblock_by_func (project->priv->name_display,
+					   brasero_project_name_changed_cb,
+					   project);
+
 	brasero_disc_load_track (project->priv->current, track);
 	brasero_track_free (track);
 
@@ -2181,6 +2023,20 @@ brasero_project_open_project (BraseroProject *project,
 
 #ifdef BUILD_PLAYLIST
 
+static void
+brasero_project_playlist_playlist_started (TotemPlParser *parser,
+					   const gchar *uri,
+					   GHashTable *metadata,
+					   gpointer user_data)
+{
+	gchar *string;
+	gchar **retval = user_data;
+
+	string = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_TITLE);
+	if (string)
+		*retval = g_strdup (string);
+}
+
 static void
 brasero_project_playlist_entry_parsed (TotemPlParser *parser,
 				       const gchar *uri,
@@ -2202,6 +2058,7 @@ brasero_project_playlist_entry_parsed (TotemPlParser *parser,
 static gboolean
 brasero_project_open_audio_playlist_project (BraseroProject *proj,
 					     const gchar *uri,
+					     gchar **label,
 					     BraseroDiscTrack **track,
 					     gboolean warn_user)
 {
@@ -2218,6 +2075,11 @@ brasero_project_open_audio_playlist_project (BraseroProject *proj,
 		      "disable-unsafe", TRUE,
 		      NULL);
 
+	g_signal_connect (parser,
+			  "playlist-started",
+			  G_CALLBACK (brasero_project_playlist_playlist_started),
+			  &label);
+
 	g_signal_connect (parser,
 			  "entry-parsed",
 			  G_CALLBACK (brasero_project_playlist_entry_parsed),
@@ -2243,17 +2105,26 @@ brasero_project_open_playlist (BraseroProject *project,
 			       const gchar *uri) /* escaped */
 {
 	BraseroDiscTrack *track = NULL;
-	BraseroProjectType type;
+	gchar *label = NULL;
 
 	if (!uri || *uri =='\0')
 		return BRASERO_PROJECT_TYPE_INVALID;
 
-	if (!brasero_project_open_audio_playlist_project (project, uri, &track, TRUE))
+	if (!brasero_project_open_audio_playlist_project (project, uri, &label, &track, TRUE))
 		return BRASERO_PROJECT_TYPE_INVALID;
 
-	brasero_project_size_set_sectors (BRASERO_PROJECT_SIZE (project->priv->size_display), 0);
-	brasero_project_switch (project, TRUE);
-	type = BRASERO_PROJECT_TYPE_AUDIO;
+	brasero_project_update_project_size (project, 0);
+	brasero_project_switch (project, BRASERO_PROJECT_TYPE_AUDIO);
+
+	if (label) {
+		g_signal_handlers_block_by_func (project->priv->name_display,
+						 brasero_project_name_changed_cb,
+						 project);
+		gtk_entry_set_text (GTK_ENTRY (project->priv->name_display), (gchar *) label);
+		g_signal_handlers_unblock_by_func (project->priv->name_display,
+						   brasero_project_name_changed_cb,
+						   project);
+	}
 
 	brasero_disc_load_track (project->priv->current, track);
 	brasero_track_free (track);
@@ -2261,7 +2132,7 @@ brasero_project_open_playlist (BraseroProject *project,
 	brasero_project_add_to_recents (project, uri, FALSE);
 	project->priv->modified = 0;
 
-	return type;
+	return BRASERO_PROJECT_TYPE_AUDIO;
 }
 
 #endif
@@ -2271,8 +2142,9 @@ brasero_project_load_session (BraseroProject *project, const gchar *uri)
 {
 	BraseroDiscTrack *track = NULL;
 	BraseroProjectType type;
+	gchar *label;
 
-	if (!brasero_project_open_project_xml (project, uri, &track, FALSE))
+	if (!brasero_project_open_project_xml (project, uri, &label, &track, FALSE))
 		return BRASERO_PROJECT_TYPE_INVALID;
 
 	if (track->type == BRASERO_DISC_TRACK_AUDIO)
@@ -2288,6 +2160,14 @@ brasero_project_load_session (BraseroProject *project, const gchar *uri)
 
 	brasero_project_switch (project, type);
 
+	g_signal_handlers_block_by_func (project->priv->name_display,
+					 brasero_project_name_changed_cb,
+					 project);
+	gtk_entry_set_text (GTK_ENTRY (project->priv->name_display), (gchar *) label);
+	g_signal_handlers_unblock_by_func (project->priv->name_display,
+					   brasero_project_name_changed_cb,
+					   project);
+
 	brasero_disc_load_track (project->priv->current, track);
 	brasero_track_free (track);
 
@@ -2300,31 +2180,14 @@ brasero_project_load_session (BraseroProject *project, const gchar *uri)
 static void
 brasero_project_not_saved_dialog (BraseroProject *project)
 {
-	GtkWidget *dialog;
-	xmlErrorPtr error;
-	GtkWidget *toplevel;
-
-	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (project));
-	dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel),
-					 GTK_DIALOG_DESTROY_WITH_PARENT|
-					 GTK_DIALOG_MODAL,
-					 GTK_MESSAGE_WARNING,
-					 GTK_BUTTONS_CLOSE,
-					 _("Your project has not been saved:"));
-
-	gtk_window_set_title (GTK_WINDOW (dialog), _("Unsaved Project"));
+	xmlError *error;
 
 	error = xmlGetLastError ();
-	if (error)
-		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-							  error->message);
-	else
-		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-							  _("Unknown error."));	
+	brasero_utils_message_dialog (gtk_widget_get_toplevel (GTK_WIDGET (project)),
+				      _("Your project has not been saved:"),
+				      error?error->message:_("Unknown error."),
+				      GTK_MESSAGE_ERROR);
 	xmlResetLastError ();
-
-	gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (dialog);
 }
 
 static GtkResponseType
@@ -2540,6 +2403,12 @@ brasero_project_save_project_xml (BraseroProject *proj,
 	if (success < 0)
 		goto error;
 
+	success = xmlTextWriterWriteElement (project,
+					     (xmlChar *) "label",
+					     (xmlChar *) gtk_entry_get_text (GTK_ENTRY (proj->priv->name_display)));
+	if (success < 0)
+		goto error;
+
 	success = xmlTextWriterStartElement (project, (xmlChar *) "track");
 	if (success < 0)
 		goto error;
@@ -2619,6 +2488,8 @@ brasero_project_save_audio_project_plain_text (BraseroProject *proj,
 					       BraseroDiscTrack *track,
 					       gboolean use_dialog)
 {
+	const gchar *title;
+	guint written;
 	GSList *iter;
 	gchar *path;
 	FILE *file;
@@ -2636,10 +2507,19 @@ brasero_project_save_audio_project_plain_text (BraseroProject *proj,
 		return FALSE;
 	}
 
+	/* write title */
+	title = gtk_entry_get_text (GTK_ENTRY (proj->priv->name_display));
+	written = fwrite (title, strlen (title), 1, file);
+	if (written != 1)
+		goto error;
+
+	written = fwrite ("\n", 1, 1, file);
+	if (written != 1)
+		goto error;
+
 	for (iter = track->contents.tracks; iter; iter = iter->next) {
 		BraseroDiscSong *song;
 		BraseroSongInfo *info;
-		guint written;
 		gchar *time;
 
 		song = iter->data;
@@ -2670,6 +2550,7 @@ brasero_project_save_audio_project_plain_text (BraseroProject *proj,
 			if (written != 1)
 				goto error;
 
+			/* Translators: %s is an artist */
 			string = g_strdup_printf (_(" by %s"), info->artist);
 			written = fwrite (string, 1, strlen (string), file);
 			if (written != strlen (string)) {
@@ -2780,13 +2661,14 @@ brasero_project_save_audio_project_playlist (BraseroProject *proj,
 			break;
 	}
 
-	result = totem_pl_parser_write (parser,
-					GTK_TREE_MODEL (model),
-					brasero_project_save_audio_playlist_entry,
-					path,
-					pl_type,
-					NULL,
-					NULL);
+	result = totem_pl_parser_write_with_title (parser,
+						   GTK_TREE_MODEL (model),
+						   brasero_project_save_audio_playlist_entry,
+						   path,
+						   gtk_entry_get_text (GTK_ENTRY (proj->priv->name_display)),
+						   pl_type,
+						   NULL,
+						   NULL);
 	if (!result && use_dialog)
 		brasero_project_not_saved_dialog (proj);
 
@@ -2831,7 +2713,8 @@ brasero_project_save_project_real (BraseroProject *project,
 		return FALSE;
 	}
 
-	if (save_type == BRASERO_PROJECT_SAVE_XML) {
+	if (save_type == BRASERO_PROJECT_SAVE_XML
+	||  track.type == BRASERO_DISC_TRACK_DATA) {
 		brasero_project_set_uri (project, uri, track.type);
 		if (!brasero_project_save_project_xml (project,
 						       uri ? uri : project->priv->project,
@@ -2946,7 +2829,7 @@ brasero_project_save_project (BraseroProject *project)
 gboolean
 brasero_project_save_project_as (BraseroProject *project)
 {
-	BraseroProjectSave type;
+	BraseroProjectSave type = BRASERO_PROJECT_SAVE_XML;
 	gboolean result;
 	gchar *uri;
 
