@@ -75,6 +75,8 @@ struct BraseroSearchEntryPrivate {
 	guint cxn;
 	gint search_id;
 
+	BraseroLayoutType ctx;
+
 	gchar *keywords;
 
 	GtkWidget *documents;
@@ -187,6 +189,7 @@ brasero_search_entry_separator_func (GtkTreeModel *model,
 static void
 brasero_search_entry_init (BraseroSearchEntry *obj)
 {
+	gchar *string;
 	GtkWidget *box;
 	GtkWidget *table;
 	GtkWidget *label;
@@ -202,7 +205,11 @@ brasero_search_entry_init (BraseroSearchEntry *obj)
 	box = gtk_hbox_new (FALSE, 6);
 	gtk_box_pack_start (GTK_BOX (obj), box, FALSE, FALSE, 0);
 
-	label = gtk_label_new (_("<b>Search:\t</b>"));
+	/* Translators: This string is followed later by "only in\t" */
+	string = g_strdup_printf ("<b>%s</b>", _("Search:\t"));
+	label = gtk_label_new (string);
+	g_free (string);
+
 	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
 	gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);
 
@@ -263,7 +270,11 @@ brasero_search_entry_init (BraseroSearchEntry *obj)
 
 	gtk_box_pack_end (GTK_BOX (obj), table, FALSE, FALSE, 0);
 
-	label = gtk_label_new (_("<b>only in\t</b>"));
+	/* Translators: This is the end of the previous sentence  "Search:\t" */
+	string = g_strdup_printf ("<b>%s</b>", _("only in\t"));
+	label = gtk_label_new (string);
+	g_free (string);
+
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
 	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
 	gtk_table_attach (GTK_TABLE (table),
@@ -759,12 +770,22 @@ void
 brasero_search_entry_set_context (BraseroSearchEntry *self,
 				  BraseroLayoutType type)
 {
+	if (self->priv->ctx == type)
+		return;
+
+	self->priv->ctx = type;
 	if (type == BRASERO_LAYOUT_AUDIO) {
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->priv->video), FALSE);
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->priv->music), TRUE);
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->priv->pictures), FALSE);
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->priv->documents), FALSE);
 	}
+	else if (type == BRASERO_LAYOUT_VIDEO) {
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->priv->video), TRUE);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->priv->music), FALSE);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->priv->pictures), FALSE);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->priv->documents), FALSE);
+	}
 }
 
 #endif /*BUILD_SEARCH*/
