@@ -132,29 +132,28 @@ typedef enum {
 	/* These flags should always be supported */
 	BRASERO_BURN_FLAG_EJECT			= 1,
 	BRASERO_BURN_FLAG_NOGRACE		= 1 << 1,
-	BRASERO_BURN_FLAG_DONT_CLEAN_OUTPUT	= 1 << 2,
-	BRASERO_BURN_FLAG_DONT_OVERWRITE	= 1 << 3,
-	BRASERO_BURN_FLAG_CHECK_SIZE		= 1 << 4,
+	BRASERO_BURN_FLAG_DONT_OVERWRITE	= 1 << 2,
+	BRASERO_BURN_FLAG_CHECK_SIZE		= 1 << 3,
 
 	/* These are of great importance for the result */
-	BRASERO_BURN_FLAG_MERGE			= 1 << 5,
-	BRASERO_BURN_FLAG_MULTI			= 1 << 6,
-	BRASERO_BURN_FLAG_APPEND		= 1 << 7,
+	BRASERO_BURN_FLAG_MERGE			= 1 << 4,
+	BRASERO_BURN_FLAG_MULTI			= 1 << 5,
+	BRASERO_BURN_FLAG_APPEND		= 1 << 6,
 
-	BRASERO_BURN_FLAG_BURNPROOF		= 1 << 8,
-	BRASERO_BURN_FLAG_NO_TMP_FILES		= 1 << 9,
-	BRASERO_BURN_FLAG_DUMMY			= 1 << 10,
+	BRASERO_BURN_FLAG_BURNPROOF		= 1 << 7,
+	BRASERO_BURN_FLAG_NO_TMP_FILES		= 1 << 8,
+	BRASERO_BURN_FLAG_DUMMY			= 1 << 9,
 
 	/* FIXME! this flag is more or less linked to OVERBURN one can't we do 
 	 * a single one */
-	BRASERO_BURN_FLAG_OVERBURN		= 1 << 11,
+	BRASERO_BURN_FLAG_OVERBURN		= 1 << 10,
 
-	BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE	= 1 << 12,
-	BRASERO_BURN_FLAG_FAST_BLANK		= 1 << 13,
+	BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE	= 1 << 11,
+	BRASERO_BURN_FLAG_FAST_BLANK		= 1 << 12,
 
 	/* NOTE: these two are contradictory? */
-	BRASERO_BURN_FLAG_DAO			= 1 << 14,
-	BRASERO_BURN_FLAG_RAW			= 1 << 15,
+	BRASERO_BURN_FLAG_DAO			= 1 << 13,
+	BRASERO_BURN_FLAG_RAW			= 1 << 14,
 
 	BRASERO_BURN_FLAG_LAST
 } BraseroBurnFlag;
@@ -181,6 +180,9 @@ brasero_burn_library_init (void);
 GSList *
 brasero_burn_library_get_plugins_list (void);
 
+gboolean
+brasero_burn_library_can_checksum (void);
+
 void
 brasero_burn_library_shutdown (void);
 
@@ -191,7 +193,7 @@ brasero_burn_library_shutdown (void);
 /**
  * Define the audio streams for a DVD
  */
-#define BRASERO_DVD_AUDIO_STREAMS		"DVD-audio-format"
+#define BRASERO_DVD_AUDIO_STREAMS		"session::DVD::audio::format"
 
 /**
  * Define the format: whether VCD or SVCD
@@ -202,7 +204,7 @@ enum {
 	BRASERO_VCD_V2,
 	BRASERO_SVCD
 };
-#define BRASERO_VCD_TYPE			"VCD-format"
+#define BRASERO_VCD_TYPE			"session::VCD::format"
 
 /**
  * This is the video format that should be used.
@@ -212,7 +214,7 @@ enum {
 	BRASERO_VIDEO_FRAMERATE_NTSC,
 	BRASERO_VIDEO_FRAMERATE_PAL_SECAM
 };
-#define BRASERO_VIDEO_OUTPUT_FRAMERATE		"video-framerate"
+#define BRASERO_VIDEO_OUTPUT_FRAMERATE		"session::video::framerate"
 
 /**
  * Aspect ratio
@@ -222,7 +224,7 @@ enum {
 	BRASERO_VIDEO_ASPECT_4_3,
 	BRASERO_VIDEO_ASPECT_16_9
 };
-#define BRASERO_VIDEO_OUTPUT_ASPECT		"video-aspect"
+#define BRASERO_VIDEO_OUTPUT_ASPECT		"session::video::aspect"
 
 
 G_END_DECLS
