@@ -61,21 +61,6 @@ GType brasero_burn_caps_get_type();
 
 BraseroBurnCaps *brasero_burn_caps_get_default ();
 
-gint
-brasero_burn_caps_register_plugin_group (BraseroBurnCaps *caps,
-					 const gchar *name);
-
-/**
- *
- */
-
-BraseroMedia
-brasero_burn_caps_media_capabilities (BraseroBurnCaps *caps,
-				      BraseroMedia media);
-
-gboolean
-brasero_burn_caps_can_checksum (BraseroBurnCaps *caps);
-
 /**
  * Returns a GSList * of BraseroTask * for a given session
  */
@@ -93,6 +78,22 @@ brasero_burn_caps_new_checksuming_task (BraseroBurnCaps *caps,
 					BraseroBurnSession *session,
 					GError **error);
 
+/**
+ * Test the supported or compulsory flags for a given session
+ */
+
+BraseroBurnResult
+brasero_burn_caps_get_flags (BraseroBurnCaps *caps,
+			     BraseroBurnSession *session,
+			     BraseroBurnFlag *supported,
+			     BraseroBurnFlag *compulsory);
+
+BraseroBurnResult
+brasero_burn_caps_get_blanking_flags (BraseroBurnCaps *caps,
+				      BraseroBurnSession *session,
+				      BraseroBurnFlag *supported,
+				      BraseroBurnFlag *compulsory);
+
 /**
  * Used to test the possibilities offered for a given session
  */
@@ -120,30 +121,8 @@ BraseroMedia
 brasero_burn_caps_get_required_media_type (BraseroBurnCaps *caps,
 					   BraseroBurnSession *session);
 
-/**
- * Test the supported or compulsory flags for a given session
- */
-
-BraseroBurnResult
-brasero_burn_caps_get_flags (BraseroBurnCaps *caps,
-			     BraseroBurnSession *session,
-			     BraseroBurnFlag *supported,
-			     BraseroBurnFlag *compulsory);
-
-BraseroBurnResult
-brasero_burn_caps_get_blanking_flags (BraseroBurnCaps *caps,
-				      BraseroBurnSession *session,
-				      BraseroBurnFlag *supported,
-				      BraseroBurnFlag *compulsory);
-
-BraseroBurnResult
-brasero_burn_caps_plugin_can_burn (BraseroBurnCaps *caps,
-				   BraseroPlugin *plugin);
-BraseroBurnResult
-brasero_burn_caps_plugin_can_image (BraseroBurnCaps *caps,
-				    BraseroPlugin *plugin);
-BraseroBurnResult
-brasero_burn_caps_plugin_can_convert (BraseroBurnCaps *caps,
-				      BraseroPlugin *plugin);
+BraseroImageFormat
+brasero_burn_caps_get_default_output_format (BraseroBurnCaps *caps,
+					     BraseroBurnSession *session);
 
 #endif /* BURN_CAPS_H */
