@@ -36,8 +36,10 @@
 
 #include "burn-basics.h"
 #include "burn-medium.h"
+#include "burn-volume-obj.h"
 #include "burn-drive.h"
 #include "burn-debug.h"
+#include "burn-hal-watch.h"
 
 #include "scsi-mmc1.h"
 
@@ -61,6 +63,10 @@ struct _BraseroDrivePrivate
 	gint bus;
 	gint target;
 	gint lun;
+
+	gulong hal_sig;
+
+	guint probed:1;
 };
 
 #define BRASERO_DRIVE_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BRASERO_TYPE_DRIVE, BraseroDrivePrivate))
@@ -80,8 +86,6 @@ enum {
 
 G_DEFINE_TYPE (BraseroDrive, brasero_drive, G_TYPE_OBJECT);
 
-static LibHalContext *hal_context = NULL;
-
 gboolean
 brasero_drive_get_bus_target_lun (BraseroDrive *self,
 				  guint *bus,
@@ -132,45 +136,6 @@ brasero_drive_is_fake (BraseroDrive *self)
 	return (priv->path == NULL);
 }
 
-static LibHalContext *
-brasero_drive_get_hal_context (void)
-{
-	DBusError error;
-	DBusConnection *dbus_connection;
-
-	if (hal_context)
-		return hal_context;
-
-	hal_context = libhal_ctx_new ();
-	if (hal_context == NULL) {
-		g_warning ("Cannot initialize hal library.");
-		goto error;
-	}
-
-	dbus_error_init (&error);
-	dbus_connection = dbus_bus_get (DBUS_BUS_SYSTEM, &error);
-	if (dbus_error_is_set (&error)) {
-		g_warning ("Cannot connect to DBus %s.", error.message);
-		dbus_error_free (&error);
-		goto error;
-	}
-
-	libhal_ctx_set_dbus_connection (hal_context, dbus_connection);
-	if (libhal_ctx_init (hal_context, &error) == FALSE) {
-		g_warning ("Failed to initialize hal : %s", error.message);
-		dbus_error_free (&error);
-		goto error;
-	}
-
-	return hal_context;
-
-error:
-	libhal_ctx_shutdown (hal_context, NULL);
-	libhal_ctx_free (hal_context);
-	hal_context = NULL;
-	return NULL;
-}
-
 gboolean
 brasero_drive_is_door_open (BraseroDrive *self)
 {
@@ -183,7 +148,7 @@ brasero_drive_is_door_open (BraseroDrive *self)
 	if (!priv->udi)
 		return FALSE;
 
-	handle = brasero_device_handle_open (priv->path, NULL);
+	handle = brasero_device_handle_open (priv->path, FALSE, NULL);
 	if (!handle)
 		return FALSE;
 
@@ -196,12 +161,33 @@ brasero_drive_is_door_open (BraseroDrive *self)
 	return hdr.door_open;
 }
 
+gboolean
+brasero_drive_can_use_exclusively (BraseroDrive *self)
+{
+	BraseroDeviceHandle *handle;
+	const gchar *device;
+
+#if defined(HAVE_STRUCT_USCSI_CMD)
+	device = brasero_drive_get_block_device (self);
+#else
+	device = brasero_drive_get_device (self);
+#endif
+
+	handle = brasero_device_handle_open (device, TRUE, NULL);
+	if (!handle)
+		return FALSE;
+
+	brasero_device_handle_close (handle);
+	return TRUE;
+}
+
 gboolean
 brasero_drive_lock (BraseroDrive *self,
 		    const gchar *reason,
 		    gchar **reason_for_failure)
 {
 	BraseroDrivePrivate *priv;
+	BraseroHALWatch *watch;
 	LibHalContext *ctx;
 	DBusError error;
 	gboolean result;
@@ -212,7 +198,8 @@ brasero_drive_lock (BraseroDrive *self,
 	if (!priv->udi)
 		return FALSE;
 
-	ctx = brasero_drive_get_hal_context ();
+	watch = brasero_hal_watch_get_default ();
+	ctx = brasero_hal_watch_get_ctx (watch);
 
 	dbus_error_init (&error);
 	result = libhal_device_lock (ctx,
@@ -230,6 +217,13 @@ brasero_drive_lock (BraseroDrive *self,
 	if (failure)
 		dbus_free (failure);
 
+	if (result) {
+		BRASERO_BURN_LOG ("Device locked");
+	}
+	else {
+		BRASERO_BURN_LOG ("Device failed to lock");
+	}
+
 	return result;
 }
 
@@ -237,16 +231,17 @@ gboolean
 brasero_drive_unlock (BraseroDrive *self)
 {
 	BraseroDrivePrivate *priv;
+	BraseroHALWatch *watch;
 	LibHalContext *ctx;
 	DBusError error;
 	gboolean result;
 
 	priv = BRASERO_DRIVE_PRIVATE (self);
-
 	if (!priv->udi)
 		return FALSE;
 
-	ctx = brasero_drive_get_hal_context ();
+	watch = brasero_hal_watch_get_default ();
+	ctx = brasero_hal_watch_get_ctx (watch);
 
 	dbus_error_init (&error);
 	result = libhal_device_unlock (ctx,
@@ -256,6 +251,7 @@ brasero_drive_unlock (BraseroDrive *self)
 	if (dbus_error_is_set (&error))
 		dbus_error_free (&error);
 
+	BRASERO_BURN_LOG ("Device unlocked");
 	return result;
 }
 
@@ -263,6 +259,7 @@ gchar *
 brasero_drive_get_display_name (BraseroDrive *self)
 {
 	BraseroDrivePrivate *priv;
+	BraseroHALWatch *watch;
 	LibHalContext *ctx;
 
 	priv = BRASERO_DRIVE_PRIVATE (self);
@@ -273,7 +270,8 @@ brasero_drive_get_display_name (BraseroDrive *self)
 	if (!priv->udi)
 		return g_strdup (_("Image File"));;
 
-	ctx = brasero_drive_get_hal_context ();
+	watch = brasero_hal_watch_get_default ();
+	ctx = brasero_hal_watch_get_ctx (watch);
 	return libhal_device_get_property_string (ctx,
 						  priv->udi,
 	  					  DEVICE_MODEL,
@@ -319,38 +317,13 @@ brasero_drive_get_medium (BraseroDrive *self)
 		return NULL;
 
 	priv = BRASERO_DRIVE_PRIVATE (self);
+
+	if (!priv->probed && priv->udi)
+		return NULL;
+
 	return priv->medium;
 }
 
-void
-brasero_drive_set_medium (BraseroDrive *self,
-			  BraseroMedium *medium)
-{
-	BraseroDrivePrivate *priv;
-
-	priv = BRASERO_DRIVE_PRIVATE (self);
-
-	if (priv->medium) {
-		g_signal_emit (self,
-			       drive_signals [MEDIUM_REMOVED],
-			       0,
-			       priv->medium);
-
-		g_object_unref (priv->medium);
-		priv->medium = NULL;
-	}
-
-	priv->medium = medium;
-
-	if (medium) {
-		g_object_ref (medium);
-		g_signal_emit (self,
-			       drive_signals [MEDIUM_INSERTED],
-			       0,
-			       medium);
-	}
-}
-
 BraseroDriveCaps
 brasero_drive_get_caps (BraseroDrive *self)
 {
@@ -396,37 +369,188 @@ brasero_drive_finalize (GObject *object)
 		priv->block_path = NULL;
 	}
 
-	if (priv->udi) {
-		g_free (priv->udi);
-		priv->udi = NULL;
-	}
-
 	if (priv->medium) {
 		g_object_unref (priv->medium);
 		priv->medium = NULL;
 	}
 
+	if (priv->hal_sig) {
+		BraseroHALWatch *watch;
+		LibHalContext *ctx;
+		DBusError error;
+
+		watch = brasero_hal_watch_get_default ();
+		ctx = brasero_hal_watch_get_ctx (watch);
+
+		dbus_error_init (&error);
+		libhal_device_remove_property_watch (ctx, priv->udi, &error);
+
+		g_signal_handler_disconnect (watch, priv->hal_sig);
+		priv->hal_sig = 0;
+	}
+
+	if (priv->udi) {
+		g_free (priv->udi);
+		priv->udi = NULL;
+	}
+
 	G_OBJECT_CLASS (brasero_drive_parent_class)->finalize (object);
 }
 
+static void
+brasero_drive_medium_probed (BraseroMedium *medium,
+			     BraseroDrive *self)
+{
+	BraseroDrivePrivate *priv;
+
+	priv = BRASERO_DRIVE_PRIVATE (self);
+
+	/* only when it is probed */
+	priv->probed = TRUE;
+	g_signal_emit (self,
+		       drive_signals [MEDIUM_INSERTED],
+		       0,
+		       priv->medium);
+}
+
+void
+brasero_drive_reprobe (BraseroDrive *self)
+{
+	BraseroDrivePrivate *priv;
+	BraseroMedium *medium;
+
+	priv = BRASERO_DRIVE_PRIVATE (self);
+
+	if (!priv->medium)
+		return;
+
+	BRASERO_BURN_LOG ("Reprobing inserted medium");
+
+	/* remove current medium */
+	medium = priv->medium;
+	priv->medium = NULL;
+
+	g_signal_emit (self,
+		       drive_signals [MEDIUM_REMOVED],
+		       0,
+		       medium);
+	g_object_unref (medium);
+	priv->probed = FALSE;
+
+	/* try to get a new one */
+	priv->medium = g_object_new (BRASERO_TYPE_VOLUME,
+				     "drive", self,
+				     NULL);
+	g_signal_connect (priv->medium,
+			  "probed",
+			  G_CALLBACK (brasero_drive_medium_probed),
+			  self);
+}
+
+static void
+brasero_drive_check_medium_inside (BraseroDrive *self)
+{
+	BraseroDrivePrivate *priv;
+	BraseroHALWatch *watch;
+	gboolean has_medium;
+	LibHalContext *ctx;
+	DBusError error;
+
+	priv = BRASERO_DRIVE_PRIVATE (self);
+
+	watch = brasero_hal_watch_get_default ();
+	ctx = brasero_hal_watch_get_ctx (watch);
+
+	BRASERO_BURN_LOG ("Contents changed");
+
+	dbus_error_init (&error);
+	has_medium = libhal_device_get_property_bool (ctx,
+						      priv->udi,
+						      "storage.removable.media_available",
+						      &error);
+	if (dbus_error_is_set (&error)) {
+		g_warning ("Hal connection problem :  %s\n",
+			   error.message);
+		dbus_error_free (&error);
+		return;
+	}
+
+	if (has_medium) {
+		BRASERO_BURN_LOG ("Medium inserted");
+
+		priv->probed = FALSE;
+		priv->medium = g_object_new (BRASERO_TYPE_VOLUME,
+					     "drive", self,
+					     NULL);
+
+		g_signal_connect (priv->medium,
+				  "probed",
+				  G_CALLBACK (brasero_drive_medium_probed),
+				  self);
+	}
+	else if (priv->medium) {
+		BraseroMedium *medium;
+
+		BRASERO_BURN_LOG ("Medium removed");
+
+		medium = priv->medium;
+		priv->medium = NULL;
+
+		g_signal_emit (self,
+			       drive_signals [MEDIUM_REMOVED],
+			       0,
+			       medium);
+		g_object_unref (medium);
+		priv->probed = FALSE;
+	}
+}
+
+static void
+brasero_drive_medium_inside_property_changed_cb (BraseroHALWatch *watch,
+						 const char *udi,
+						 const char *key,
+						 BraseroDrive *drive)
+{
+	BraseroDrivePrivate *priv;
+
+	priv = BRASERO_DRIVE_PRIVATE (drive);
+
+	if (key && strcmp (key, "storage.removable.media_available"))
+		return;
+
+	if (udi && strcmp (udi, priv->udi))
+		return;
+
+	brasero_drive_check_medium_inside (drive);
+}
+
 static void
 brasero_drive_init_real (BraseroDrive *drive)
 {
 	BraseroDrivePrivate *priv;
+	BraseroHALWatch *watch;
 	LibHalContext *ctx;
+	DBusError error;
 	char *parent;
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
 
-	ctx = brasero_drive_get_hal_context ();
+	watch = brasero_hal_watch_get_default ();
+	ctx = brasero_hal_watch_get_ctx (watch);
 
-	priv->path = libhal_device_get_property_string (ctx, priv->udi, BLOCK_DEVICE, NULL);
+	priv->path = libhal_device_get_property_string (ctx,
+							priv->udi,
+							BLOCK_DEVICE,
+							NULL);
 	if (priv->path [0] == '\0') {
 		g_free (priv->path);
 		priv->path = NULL;
 	}
 
-	priv->block_path = libhal_device_get_property_string (ctx, priv->udi, "block.device", NULL);
+	priv->block_path = libhal_device_get_property_string (ctx,
+							      priv->udi,
+							      "block.device",
+							      NULL);
 	if (priv->block_path [0] == '\0') {
 		g_free (priv->block_path);
 		priv->block_path = NULL;
@@ -450,32 +574,42 @@ brasero_drive_init_real (BraseroDrive *drive)
 		priv->caps |= BRASERO_DRIVE_CAPS_DVDRW_PLUS_DL;
 
 	/* Also get its parent to retrieve the bus, host, lun values */
+	priv->bus = -1;
+	priv->lun = -1;
+	priv->target = -1;
 	parent = libhal_device_get_property_string (ctx, priv->udi, "info.parent", NULL);
-	if (!parent) {
-		priv->bus = -1;
-		priv->lun = -1;
-		priv->target = -1;
-		return;
+	if (parent) {
+		/* Check it is a SCSI interface */
+		if (libhal_device_property_exists (ctx, parent, "scsi.host", NULL)
+		&&  libhal_device_property_exists (ctx, parent, "scsi.lun", NULL)
+		&&  libhal_device_property_exists (ctx, parent, "scsi.target", NULL)) {
+			priv->bus = libhal_device_get_property_int (ctx, parent, "scsi.host", NULL);
+			priv->lun = libhal_device_get_property_int (ctx, parent, "scsi.lun", NULL);
+			priv->target = libhal_device_get_property_int (ctx, parent, "scsi.target", NULL);
+		}
+
+		BRASERO_BURN_LOG ("Drive %s has bus,target,lun = %i %i %i",
+				  priv->path,
+				  priv->bus,
+				  priv->target,
+				  priv->lun);
+		libhal_free_string (parent);
 	}
 
-	/* Check it is a SCSI interface */
-	if (!libhal_device_property_exists (ctx, parent, "scsi.host", NULL)
-	||  !libhal_device_property_exists (ctx, parent, "scsi.lun", NULL)
-	||  !libhal_device_property_exists (ctx, parent, "scsi.target", NULL)) {
-		g_free (parent);
+	/* Now check for the medium */
+	brasero_drive_check_medium_inside (drive);
 
-		priv->bus = -1;
-		priv->lun = -1;
-		priv->target = -1;
-		return;
+	dbus_error_init (&error);
+	libhal_device_add_property_watch (ctx, priv->udi, &error);
+	if (dbus_error_is_set (&error)) {
+		g_warning ("Failed to watch property : %s\n", error.message);
+		dbus_error_free (&error);
 	}
 
-	priv->bus = libhal_device_get_property_int (ctx, parent, "scsi.host", NULL);
-	priv->lun = libhal_device_get_property_int (ctx, parent, "scsi.lun", NULL);
-	priv->target = libhal_device_get_property_int (ctx, parent, "scsi.target", NULL);
-
-	BRASERO_BURN_LOG ("Drive %s has bus,target,lun = %i %i %i", priv->path, priv->bus, priv->target, priv->lun);
-	libhal_free_string (parent);
+	priv->hal_sig = g_signal_connect (watch,
+					  "property-changed",
+					  G_CALLBACK (brasero_drive_medium_inside_property_changed_cb),
+					  drive);
 }
 
 static void
@@ -494,8 +628,14 @@ brasero_drive_set_property (GObject *object,
 	{
 	case PROP_UDI:
 		priv->udi = g_strdup (g_value_get_string (value));
-		if (priv->udi)
+		if (!priv->udi) {
+			priv->medium = g_object_new (BRASERO_TYPE_VOLUME,
+						     "drive", object,
+						     NULL);
+		}
+		else
 			brasero_drive_init_real (BRASERO_DRIVE (object));
+			
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -560,10 +700,10 @@ brasero_drive_class_init (BraseroDriveClass *klass)
 	g_object_class_install_property (object_class,
 	                                 PROP_UDI,
 	                                 g_param_spec_string("udi",
-	                                                      "udi",
-	                                                      "HAL udi",
-	                                                      NULL,
-	                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	                                                     "udi",
+	                                                     "HAL udi",
+	                                                     NULL,
+	                                                     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
 
 BraseroDrive *
