@@ -301,11 +301,13 @@ stat_end:
 	g_free (path);
 
 	if (res == -1) {
+                int errsv = errno;
+
 		g_set_error (error,
 			     BRASERO_BURN_ERR,
 			     BRASERO_BURN_ERROR_GENERAL,
 			     _("size can't be retrieved (%s)"),
-			     strerror (errno));
+			     g_strerror (errsv));
 		return FALSE;
 	}
 
@@ -380,16 +382,19 @@ stat_end:
 		g_free (tmp);
 	}
 
-	/* if size is skipped then g_lstat () the file */
-	res = g_lstat (path, &buffer);
+	/* if size is skipped then g_stat () the file */
+	/* NOTE: follow symlink if any */
+	res = g_stat (path, &buffer);
 	g_free (path);
 
 	if (res == -1) {
+                int errsv = errno;
+
 		g_set_error (error,
 			     BRASERO_BURN_ERR,
 			     BRASERO_BURN_ERROR_GENERAL,
 			     _("size can't be retrieved (%s)"),
-			     strerror (errno));
+			     g_strerror (errsv));
 		return FALSE;
 	}
 
@@ -414,11 +419,13 @@ brasero_image_format_get_cdrdao_size (gchar *path,
 	 * multiple files. Which is great but not for us ... */
 	file = fopen (path, "r");
 	if (!file) {
+                int errsv = errno;
+
 		g_set_error (error,
 			     BRASERO_BURN_ERR,
 			     BRASERO_BURN_ERROR_GENERAL,
 			     _("size can't be retrieved (%s)"),
-			     strerror (errno));
+			     g_strerror (errsv));
 		return FALSE;
 	}
 
@@ -541,11 +548,13 @@ brasero_image_format_get_cue_size (gchar *path,
 	 * multiple files. Which is great but not for us ... */
 	file = fopen (path, "r");
 	if (!file) {
+                int errsv = errno;
+
 		g_set_error (error,
 			     BRASERO_BURN_ERR,
 			     BRASERO_BURN_ERROR_GENERAL,
 			     _("size can't be retrieved (%s)"),
-			     strerror (errno));
+			     g_strerror (errsv));
 		return FALSE;
 	}
 
@@ -580,14 +589,17 @@ brasero_image_format_get_cue_size (gchar *path,
 				g_free (tmp);
 			}
 
-			res = g_lstat (file_path, &buffer);
+			/* NOTE: follow symlink if any */
+			res = g_stat (path, &buffer);
 			if (res == -1) {
+                                int errsv = errno;
+
 				g_set_error (error,
 					     BRASERO_BURN_ERR,
 					     BRASERO_BURN_ERROR_GENERAL,
 					     _("size can't be retrieved for %s: %s"),
 					     file_path,
-					     strerror (errno));
+					     g_strerror (errsv));
 				g_free (file_path);
 				return FALSE;
 			}
@@ -724,6 +736,9 @@ brasero_image_format_get_iso_size (gchar *path,
 	struct stat buffer;
 	int res;
 
+	if (!path)
+		return FALSE;
+
 	/* a simple stat () will do. That means of course that the image must be
 	 * local. Now if local-track is enabled, it will always run first and we
 	 * don't need that size before it starts. During the GET_SIZE phase of 
@@ -735,13 +750,16 @@ brasero_image_format_get_iso_size (gchar *path,
 	 * since it'll be local and stat() will work.
 	 * if local-track is not enabled we can't use non-local images anyway so
 	 * there is no need to have a function set_size */
-	res = g_lstat (path, &buffer);
+	/* NOTE: follow symlink if any */
+	res = g_stat (path, &buffer);
 	if (res == -1) {
+                int errsv = errno;
+
 		g_set_error (error,
 			     BRASERO_BURN_ERR,
 			     BRASERO_BURN_ERROR_GENERAL,
 			     _("size can't be retrieved (%s)"),
-			     strerror (errno));
+			     g_strerror (errsv));
 
 		return FALSE;
 	}
@@ -776,13 +794,16 @@ brasero_image_format_get_clone_size (gchar *path,
 	 * since it'll be local and stat() will work.
 	 * if local-track is not enabled we can't use non-local images anyway so
 	 * there is no need to have a function set_size */
-	res = g_lstat (path, &buffer);
+	/* NOTE: follow symlink if any */
+	res = g_stat (path, &buffer);
 	if (res == -1) {
+                int errsv = errno;
+
 		g_set_error (error,
 			     BRASERO_BURN_ERR,
 			     BRASERO_BURN_ERROR_GENERAL,
 			     _("size can't be retrieved (%s)"),
-			     strerror (errno));
+			     g_strerror (errsv));
 
 		return FALSE;
 	}
@@ -796,3 +817,101 @@ brasero_image_format_get_clone_size (gchar *path,
 
 	return TRUE;
 }
+
+gchar *
+brasero_image_format_get_default_path (BraseroImageFormat format)
+{
+	const gchar *suffixes [] = {".iso",
+				    ".toc",
+				    ".cue",
+				    ".toc",
+				    NULL };
+	const gchar *suffix = NULL;
+	gchar *path;
+	gint i = 0;
+
+	if (format & BRASERO_IMAGE_FORMAT_BIN)
+		suffix = suffixes [0];
+	else if (format & BRASERO_IMAGE_FORMAT_CLONE)
+		suffix = suffixes [1];
+	else if (format & BRASERO_IMAGE_FORMAT_CUE)
+		suffix = suffixes [2];
+	else if (format & BRASERO_IMAGE_FORMAT_CDRDAO)
+		suffix = suffixes [3];
+
+	path = g_strdup_printf ("%s/brasero%s",
+				g_get_home_dir (),
+				suffix);
+
+	while (g_file_test (path, G_FILE_TEST_EXISTS)) {
+		g_free (path);
+
+		path = g_strdup_printf ("%s/brasero-%i%s",
+					g_get_home_dir (),
+					i,
+					suffix);
+		i ++;
+	}
+
+	return path;
+}
+
+gchar *
+brasero_image_format_fix_path_extension (BraseroImageFormat format,
+					 gboolean check_existence,
+					 gchar *path)
+{
+	gchar *dot;
+	guint i = 0;
+	gchar *retval = NULL;
+	const gchar *suffix = NULL;;
+	const gchar *suffixes [] = {".iso",
+				    ".toc",
+				    ".cue",
+				    ".toc",
+				    NULL };
+
+	/* search the last dot to check extension */
+	dot = g_utf8_strrchr (path, -1, '.');
+	if (dot && strlen (dot) < 5 && strlen (dot) > 1) {
+		if (format & BRASERO_IMAGE_FORMAT_BIN
+		&&  strcmp (suffixes [0], dot))
+			*dot = '\0';
+		else if (format & BRASERO_IMAGE_FORMAT_CLONE
+		     &&  strcmp (suffixes [1], dot))
+			*dot = '\0';
+		else if (format & BRASERO_IMAGE_FORMAT_CUE
+		     &&  strcmp (suffixes [2], dot))
+			*dot = '\0';
+		else if (format & BRASERO_IMAGE_FORMAT_CDRDAO
+		     &&  strcmp (suffixes [3], dot))
+			*dot = '\0';
+		else
+			return g_strdup (path);
+	}
+
+	/* determine the proper suffix */
+	if (format & BRASERO_IMAGE_FORMAT_BIN)
+		suffix = suffixes [0];
+	else if (format & BRASERO_IMAGE_FORMAT_CLONE)
+		suffix = suffixes [1];
+	else if (format & BRASERO_IMAGE_FORMAT_CUE)
+		suffix = suffixes [2];
+	else if (format & BRASERO_IMAGE_FORMAT_CDRDAO)
+		suffix = suffixes [3];
+	else
+		return g_strdup (path);
+
+	/* make sure the file doesn't exist */
+	retval = g_strdup_printf ("%s%s", path, suffix);
+	if (!check_existence)
+		return retval;
+
+	while (g_file_test (retval, G_FILE_TEST_EXISTS)) {
+		g_free (retval);
+		retval = g_strdup_printf ("%s-%i%s", path, i, suffix);
+		i ++;
+	}
+
+	return retval;
+}
