@@ -461,23 +461,18 @@ brasero_job_check_output_volume_space (BraseroJob *self,
 	brasero_job_get_session_output_size (BRASERO_JOB (self), NULL, &output_size);
 
 	/* Now check the filesystem type: the problem here is that some
-	 * filesystems have a maximum file size limit of 4 Gio and more than
-	 * often we need a temporary file size of 4 Gio or more. */
+	 * filesystems have a maximum file size limit of 4 GiB and more than
+	 * often we need a temporary file size of 4 GiB or more. */
 	filesystem = g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_FILESYSTEM_TYPE);
 	BRASERO_BURN_LOG ("%s filesystem detected", filesystem);
 
 	if (output_size >= 2147483648ULL
 	&&  filesystem
 	&& !strcmp (filesystem, "msdos")) {
-		/* FIXME: This string should mention that the location is on the
-		 * hard drive and not the medium itself to prevent any confusion
-		 * as seen in #533149 */
-		/* FIXME: change this string to something more appropriate when
-		 * we're not in string freeze. */
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_DISK_SPACE,
-			     _("The filesystem you chose to store the temporary image on cannot hold files with a size over 2 Gio."));
+			     _("The filesystem you chose to store the temporary image on cannot hold files with a size over 2 GiB."));
 		return BRASERO_BURN_ERR;
 	}
 
@@ -489,13 +484,10 @@ brasero_job_check_output_volume_space (BraseroJob *self,
 
 	/* it's fine here to check size in bytes */
 	if (output_size > vol_size) {
-		/* FIXME: This string should mention that the location is on the
-		 * hard drive and not the medium itself to prevent any confusion
-		 * as seen in #533149 */
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_DISK_SPACE,
-			     _("the selected location does not have enough free space to store the disc image (%ld MiB needed)"),
+			     _("The location you chose to store the temporary image on does not have enough free space for the disc image (%ld MiB needed)"),
 			     (unsigned long) output_size / 1048576);
 		return BRASERO_BURN_ERR;
 	}
@@ -506,7 +498,8 @@ brasero_job_check_output_volume_space (BraseroJob *self,
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_DISK_SPACE,
-			     strerror (errno));
+			     "%s",
+			     g_strerror (errno));
 		return BRASERO_BURN_ERR;
 	}
 
@@ -549,29 +542,89 @@ brasero_job_set_output_file (BraseroJob *self,
 
 	/* no next job so we need a file pad */
 	session = brasero_task_ctx_get_session (priv->ctx);
+	flags = brasero_burn_session_get_flags (session);
 	if (priv->type.type == BRASERO_TRACK_TYPE_IMAGE) {
 		BraseroImageFormat format;
 
 		/* check if that's the last task */
 		format = brasero_burn_session_get_output_format (session);
-		if (priv->type.subtype.img_format == format)
+		if (priv->type.subtype.img_format == format) {
+			BraseroTrackType input = { 0, };
+
 			result = brasero_burn_session_get_output (session,
 								  &image,
 								  &toc,
 								  error);
-		else
+
+			/* check paths are set */
+			if (!image
+			|| (priv->type.subtype.img_format != BRASERO_IMAGE_FORMAT_BIN && !toc)) {
+				g_set_error (error,
+					     BRASERO_BURN_ERROR,
+					     BRASERO_BURN_ERROR_GENERAL,
+					     _("no path"));
+				return BRASERO_BURN_ERR;
+			}
+
+			brasero_burn_session_get_input_type (session,
+							     &input);
+
+			/* if input is the same as output, then that's a
+			 * processing task and there's no need to check if the
+			 * output already exists since it will (but that's OK) */
+			if (input.type == BRASERO_TRACK_TYPE_IMAGE
+			&&  input.subtype.img_format == priv->type.subtype.img_format) {
+				BRASERO_BURN_LOG ("Processing task, skipping check size");
+				priv->output = g_new0 (BraseroJobOutput, 1);
+				priv->output->image = image;
+				priv->output->toc = toc;
+				return BRASERO_BURN_OK;
+			}
+
+			/* There must be an image at least */
+			if ((flags & BRASERO_BURN_FLAG_DONT_OVERWRITE)
+			&&   g_file_test (image, G_FILE_TEST_EXISTS)) {
+				BRASERO_BURN_LOG ("Problem with image existence");
+				g_set_error (error,
+					     BRASERO_BURN_ERROR,
+					     BRASERO_BURN_ERROR_GENERAL,
+					     _("%s already exists"),
+					     image);
+				g_free (toc);
+				g_free (image);
+				return BRASERO_BURN_ERR;
+			}
+
+			if (priv->type.subtype.img_format != BRASERO_IMAGE_FORMAT_BIN) {
+				/* There must a toc file in this case */
+				if ((flags & BRASERO_BURN_FLAG_DONT_OVERWRITE)
+				&&   g_file_test (toc, G_FILE_TEST_EXISTS)) {
+					BRASERO_BURN_LOG ("Problem with toc existence");
+					g_set_error (error,
+						     BRASERO_BURN_ERROR,
+						     BRASERO_BURN_ERROR_GENERAL,
+						     _("%s already exists"),
+						     toc);
+					g_free (toc);
+					g_free (image);
+					return BRASERO_BURN_ERR;
+				}
+			}
+		}
+		else {
+			/* NOTE: no need to check for the existence here */
 			result = brasero_burn_session_get_tmp_image (session,
 								     priv->type.subtype.img_format,
 								     &image,
 								     &toc,
 								     error);
-
-		if (result != BRASERO_BURN_OK)
-			return result;
+			if (result != BRASERO_BURN_OK)
+				return result;
+		}
 
 		BRASERO_JOB_LOG (self, "output set (IMAGE) image = %s toc = %s",
 				 image,
-		    toc ? toc : "nil");
+				 toc ? toc : "none");
 	}
 	else if (priv->type.type == BRASERO_TRACK_TYPE_AUDIO) {
 		/* NOTE: this one can only a temporary file */
@@ -579,8 +632,7 @@ brasero_job_set_output_file (BraseroJob *self,
 							    ".cdr",
 							    &image,
 							    error);
-		BRASERO_JOB_LOG (self, "Output set (AUDIO) image = %s",
-				 image);
+		BRASERO_JOB_LOG (self, "Output set (AUDIO) image = %s", image);
 	}
 	else /* other types don't need an output */
 		return BRASERO_BURN_OK;
@@ -592,8 +644,6 @@ brasero_job_set_output_file (BraseroJob *self,
 	priv->output->image = image;
 	priv->output->toc = toc;
 
-	flags = brasero_burn_session_get_flags (session);
-
 	if (flags & BRASERO_BURN_FLAG_CHECK_SIZE)
 		return brasero_job_check_output_volume_space (self, error);
 
@@ -677,11 +727,13 @@ brasero_job_item_start (BraseroTaskItem *item,
 
 		/* setup a pipe */
 		if (pipe (fd)) {
+                        int errsv = errno;
+
 			g_set_error (error,
 				     BRASERO_BURN_ERROR,
 				     BRASERO_BURN_ERROR_GENERAL,
 				     _("the pipe couldn't be created (%s)"),
-				     strerror (errno));
+				     g_strerror (errsv));
 
 			return BRASERO_BURN_ERR;
 		}
@@ -1518,9 +1570,11 @@ brasero_job_get_speed (BraseroJob *self, guint *speed)
 	rate = brasero_burn_session_get_rate (session);
 
 	media = brasero_burn_session_get_dest_media (session);
-	if (media & (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_DVD_DL))
+	if (media & BRASERO_MEDIUM_DVD)
 		*speed = BRASERO_RATE_TO_SPEED_DVD (rate);
-	else 
+	else if (media & BRASERO_MEDIUM_BD)
+		*speed = BRASERO_RATE_TO_SPEED_BD (rate);
+	else
 		*speed = BRASERO_RATE_TO_SPEED_CD (rate);
 
 	return BRASERO_BURN_OK;
@@ -1570,9 +1624,11 @@ brasero_job_get_max_speed (BraseroJob *self, guint *speed)
 	medium = brasero_drive_get_medium (drive);
 	rate = brasero_medium_get_max_write_speed (medium);
 	media = brasero_medium_get_status (medium);
-	if (media & (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_DVD_DL))
+	if (media & BRASERO_MEDIUM_DVD)
 		*speed = BRASERO_RATE_TO_SPEED_DVD (rate);
-	else 
+	else if (media & BRASERO_MEDIUM_BD)
+		*speed = BRASERO_RATE_TO_SPEED_BD (rate);
+	else
 		*speed = BRASERO_RATE_TO_SPEED_CD (rate);
 
 	return BRASERO_BURN_OK;
@@ -1673,7 +1729,8 @@ brasero_job_start_progress (BraseroJob *self,
 {
 	BraseroJobPrivate *priv;
 
-	BRASERO_JOB_DEBUG (self);
+	/* Turn this off as otherwise it floods bug reports */
+	// BRASERO_JOB_DEBUG (self);
 
 	priv = BRASERO_JOB_PRIVATE (self);
 	if (priv->next)
@@ -1686,29 +1743,15 @@ brasero_job_start_progress (BraseroJob *self,
  * these should be used to set the different values of the task by the jobs
  */
 
-BraseroBurnResult
-brasero_job_add_wrong_checksum (BraseroJob *self,
-				const gchar *path)
-{
-	BraseroJobPrivate *priv;
-	BraseroBurnSession *session;
-
-	BRASERO_JOB_DEBUG (self);
-
-	priv = BRASERO_JOB_PRIVATE (self);
-	session = brasero_task_ctx_get_session (priv->ctx);
-
-	brasero_burn_session_add_wrong_checksum (session, path);
-	return BRASERO_BURN_OK;
-}
-
 BraseroBurnResult
 brasero_job_set_progress (BraseroJob *self,
 			  gdouble progress)
 {
 	BraseroJobPrivate *priv;
 
-	BRASERO_JOB_LOG (self, "Called brasero_job_set_progress (%lf)", progress);
+	/* Turn this off as it floods bug reports */
+	//BRASERO_JOB_LOG (self, "Called brasero_job_set_progress (%lf)", progress);
+
 	priv = BRASERO_JOB_PRIVATE (self);
 	if (priv->next)
 		return BRASERO_BURN_ERR;
@@ -1769,7 +1812,8 @@ brasero_job_set_rate (BraseroJob *self,
 {
 	BraseroJobPrivate *priv;
 
-	BRASERO_JOB_DEBUG (self);
+	/* Turn this off as otherwise it floods bug reports */
+	// BRASERO_JOB_DEBUG (self);
 
 	priv = BRASERO_JOB_PRIVATE (self);
 	if (priv->next)
@@ -1810,7 +1854,8 @@ brasero_job_set_written_track (BraseroJob *self,
 {
 	BraseroJobPrivate *priv;
 
-	BRASERO_JOB_DEBUG (self);
+	/* Turn this off as otherwise it floods bug reports */
+	// BRASERO_JOB_DEBUG (self);
 
 	priv = BRASERO_JOB_PRIVATE (self);
 	if (priv->next)
@@ -1825,7 +1870,8 @@ brasero_job_set_written_session (BraseroJob *self,
 {
 	BraseroJobPrivate *priv;
 
-	BRASERO_JOB_DEBUG (self);
+	/* Turn this off as otherwise it floods bug reports */
+	// BRASERO_JOB_DEBUG (self);
 
 	priv = BRASERO_JOB_PRIVATE (self);
 	if (priv->next)
