@@ -28,6 +28,7 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <unistd.h>
 
 #include <glib.h>
 #include <glib/gi18n-lib.h>
@@ -52,14 +53,14 @@
 #include "burn-drive.h"
 
 const gchar *icons [] = { 	"iso-image-new",
-				"media-optical-cd",
-				"media-optical-cd",
-				"media-optical-cd",
-				"media-optical-dvd",
-				"media-optical-dvd",
-				"media-optical-dvd",
-				"media-optical-dvd",
-				"media-optical-dvd",
+				"media-optical",
+				"media-optical",
+				"media-optical",
+				"media-optical",
+				"media-optical",
+				"media-optical",
+				"media-optical",
+				"media-optical",
 				NULL };
 const gchar *types [] = {	N_("file"),
 				N_("CDROM"),
@@ -88,15 +89,14 @@ typedef enum {
 typedef struct _BraseroMediumPrivate BraseroMediumPrivate;
 struct _BraseroMediumPrivate
 {
-	gint retry_id;
+	GThread *probe;
+	gint probe_id;
 
 	GSList *tracks;
 
 	const gchar *type;
 	const gchar *icon;
 
-	gchar *udi;
-
 	gchar *id;
 
 	gint max_rd;
@@ -114,10 +114,14 @@ struct _BraseroMediumPrivate
 	BraseroMedia info;
 	BraseroDrive *drive;
 
+	gchar *CD_TEXT_title;
+
 	/* Do we really need both? */
 	guint dummy_sao:2;
 	guint dummy_tao:2;
 	guint burnfree:2;
+
+	guint probe_cancelled:1;
 };
 
 #define BRASERO_MEDIUM_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BRASERO_TYPE_MEDIUM, BraseroMediumPrivate))
@@ -135,11 +139,82 @@ enum
 {
 	PROP_0,
 	PROP_DRIVE,
-	PROP_UDI
 };
 
+enum {
+	PROBED,
+	LAST_SIGNAL
+};
+static gulong medium_signals [LAST_SIGNAL] = {0, };
+
+#define BRASERO_MEDIUM_OPEN_ATTEMPTS			5
+
 static GObjectClass* parent_class = NULL;
 
+gchar *
+brasero_medium_get_tooltip (BraseroMedium *self)
+{
+	BraseroMediumPrivate *priv;
+	BraseroDrive *drive;
+	BraseroMedia media;
+	const gchar *type;
+	gchar *label;
+	gchar *name;
+
+	priv = BRASERO_MEDIUM_PRIVATE (self);
+
+	media = brasero_medium_get_status (BRASERO_MEDIUM (self));
+	if (media & BRASERO_MEDIUM_FILE) {
+		/* Translators: This is a fake drive, a file, and means that
+		 * when we're writing, we're writing to a file and create an
+		 * image on the hard drive. */
+		return g_strdup (_("Image File"));
+	}
+
+	type = brasero_medium_get_type_string (BRASERO_MEDIUM (self));
+	drive = brasero_medium_get_drive (BRASERO_MEDIUM (self));
+	name = brasero_drive_get_display_name (drive);
+
+	if (media & BRASERO_MEDIUM_BLANK) {
+		/* NOTE for translators: the first %s is the disc type and the
+		 * second %s the name of the drive this disc is in. */
+		label = g_strdup_printf (_("Blank %s in %s"),
+					 type,
+					 name);
+	}
+	else if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_HAS_AUDIO|BRASERO_MEDIUM_HAS_DATA)) {
+		/* NOTE for translators: the first %s is the disc type and the
+		 * second %s the name of the drive this disc is in. */
+		label = g_strdup_printf (_("Audio and data %s in %s"),
+					 type,
+					 name);
+	}
+	else if (media & BRASERO_MEDIUM_HAS_AUDIO) {
+		/* NOTE for translators: the first %s is the disc type and the
+		 * second %s the name of the drive this disc is in. */
+		label = g_strdup_printf (_("Audio %s in %s"),
+					 type,
+					 name);
+	}
+	else if (media & BRASERO_MEDIUM_HAS_DATA) {
+		/* NOTE for translators: the first %s is the disc type and the
+	 	* second %s the name of the drive this disc is in. */
+		label = g_strdup_printf (_("Data %s in %s"),
+					 type,
+					 name);
+	}
+	else {
+		/* NOTE for translators: the first %s is the disc type and the
+	 	* second %s the name of the drive this disc is in. */
+		label = g_strdup_printf (_("%s in %s"),
+					 type,
+					 name);
+	}
+
+	g_free (name);
+	return label;
+}
+
 const gchar *
 brasero_medium_get_type_string (BraseroMedium *medium)
 {
@@ -903,11 +978,31 @@ brasero_medium_get_capacity_DVD_RW (BraseroMedium *self,
 		return BRASERO_BURN_ERR;
 	}
 
+	/* NOTE: for BD-RE there is a slight problem to determine the exact
+	 * capacity of the medium when it is unformatted. Indeed the final size
+	 * of the User Data Area will depend on the size of the Spare areas.
+	 * On the other hand if it's formatted then that's OK, just take the 
+	 * current one.
+	 * NOTE: that could work also for BD-R SRM+POW and BD-R RRM */
+
 	/* see if the media is already formatted */
 	current = hdr->max_caps;
 	if (!(current->type & BRASERO_SCSI_DESC_FORMATTED)) {
 		BRASERO_BURN_LOG ("Unformatted media");
 		priv->info |= BRASERO_MEDIUM_UNFORMATTED;
+
+		/* if unformatted, a DVD-RAM will return its maximum formattable
+		 * size in this descriptor and that's what we're looking for. */
+		if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVD_RAM)) {
+			priv->block_num = BRASERO_GET_32 (current->blocks_num);
+			priv->block_size = 2048;
+			goto end;
+		}
+	}
+	else if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_BDRE)) {
+		priv->block_num = BRASERO_GET_32 (current->blocks_num);
+		priv->block_size = 2048;
+		goto end;
 	}
 
 	max = (hdr->len - 
@@ -929,12 +1024,24 @@ brasero_medium_get_capacity_DVD_RW (BraseroMedium *self,
 				break;
 			}
 		}
+		else if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_BDRE)) {
+			/* This is for unformatted BDRE: since we can't know the
+			 * size of the Spare Area in advance, we take the vendor
+			 * preferred one. Always following are the smallest one
+			 * and the biggest one. */
+			if (desc->format_type == BRASERO_SCSI_BDRE_FORMAT) {
+				priv->block_num = BRASERO_GET_32 (desc->blocks_num);
+				break;
+			}
+		}
 		else if (desc->format_type == BRASERO_SCSI_MAX_PACKET_SIZE_FORMAT) {
 			priv->block_num = BRASERO_GET_32 (desc->blocks_num);
 			break;
 		}
 	}
 
+end:
+
 	BRASERO_BURN_LOG ("Format capacity %lli %lli",
 			  priv->block_num,
 			  priv->block_size);
@@ -952,7 +1059,7 @@ brasero_medium_get_capacity_by_type (BraseroMedium *self,
 
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 
-	/* For DVDs that's always that block size */
+	/* For DVDs/BDs that's always that block size */
 	priv->block_size = 2048;
 
 	if (!(priv->info & BRASERO_MEDIUM_REWRITABLE))
@@ -1282,7 +1389,7 @@ brasero_medium_track_get_info (BraseroMedium *self,
 	/* at this point we know the type of the disc that's why we set the 
 	 * size according to this type. That may help to avoid outrange address
 	 * errors. */
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVD_DL|BRASERO_MEDIUM_WRITABLE))
+	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DUAL_L|BRASERO_MEDIUM_WRITABLE))
 		size = 48;
 	else if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_PLUS|BRASERO_MEDIUM_WRITABLE))
 		size = 40;
@@ -1408,8 +1515,7 @@ brasero_medium_track_set_leadout (BraseroMedium *self,
 
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS)
-	||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_RESTRICTED)) {
+	if (BRASERO_MEDIUM_RANDOM_WRITABLE (priv->info)) {
 		BRASERO_BURN_LOG ("Overwritable medium  => skipping");
 		return BRASERO_BURN_OK;
 	}
@@ -1422,28 +1528,33 @@ brasero_medium_track_set_leadout (BraseroMedium *self,
 						   &data,
 						   &size,
 						   code);
-	if (result != BRASERO_SCSI_OK) {
+	if (result == BRASERO_SCSI_OK) {
+		wrt_page = (BraseroScsiWritePage *) &data->page;
+
+		BRASERO_BURN_LOG ("Former write mode %d", wrt_page->write_type);
+		wrt_page->write_type = BRASERO_SCSI_WRITE_TAO;
+
+		result = brasero_spc1_mode_select (handle, data, size, code);
 		g_free (data);
 
-		BRASERO_BURN_LOG ("MODE SENSE failed");
-		return BRASERO_BURN_ERR;
+		if (result != BRASERO_SCSI_OK) {
+			BRASERO_BURN_LOG ("MODE SELECT failed");
+
+			/* This isn't necessarily a problem! we better try */
+			//	return BRASERO_BURN_ERR;
+		}
 	}
+	else {
+		BRASERO_BURN_LOG ("MODE SENSE failed");
 
-	wrt_page = (BraseroScsiWritePage *) &data->page;
-	wrt_page->write_type = BRASERO_SCSI_WRITE_TAO;
-
-	result = brasero_spc1_mode_select (handle, data, size, code);
-	g_free (data);
-
-	if (result != BRASERO_SCSI_OK) {
-		BRASERO_BURN_LOG ("MODE SELECT failed");
-		return BRASERO_BURN_ERR;
+		/* This isn't necessarily a problem we better! try the rest */
+		//	return BRASERO_BURN_ERR;
 	}
 
 	/* at this point we know the type of the disc that's why we set the 
 	 * size according to this type. That may help to avoid outrange address
 	 * errors. */
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVD_DL|BRASERO_MEDIUM_WRITABLE))
+	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DUAL_L|BRASERO_MEDIUM_WRITABLE))
 		size = 48;
 	else if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_PLUS|BRASERO_MEDIUM_WRITABLE))
 		size = 40;
@@ -1619,11 +1730,10 @@ brasero_medium_get_sessions_info (BraseroMedium *self,
 				track->type |= BRASERO_MEDIUM_TRACK_INCREMENTAL;
 		}
 
-		if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS)
-		||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_RESTRICTED)) {
+		if (BRASERO_MEDIUM_RANDOM_WRITABLE (priv->info)) {
 			BraseroBurnResult result;
 
-			/* A special case for these two kinds of media (DVD+RW)
+			/* A special case for these kinds of media (DVD+RW, ...)
 			 * which have only one track: the first. Since it's not
 			 * possible to know the amount of data that were really
 			 * written in this session, read the filesystem. */
@@ -1656,6 +1766,11 @@ brasero_medium_get_sessions_info (BraseroMedium *self,
 			continue;
 		}
 
+		if (priv->probe_cancelled) {
+			g_free (toc);
+			return BRASERO_BURN_CANCEL;
+		}
+
 		brasero_medium_track_get_info (self,
 					       multisession,
 					       track,
@@ -1664,11 +1779,15 @@ brasero_medium_get_sessions_info (BraseroMedium *self,
 					       code);
 	}
 
+	if (priv->probe_cancelled) {
+		g_free (toc);
+		return BRASERO_BURN_CANCEL;
+	}
+
 	/* put the tracks in the right order */
 	priv->tracks = g_slist_reverse (priv->tracks);
 
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS)
-	||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_RESTRICTED))
+	if (BRASERO_MEDIUM_RANDOM_WRITABLE (priv->info))
 		brasero_medium_add_DVD_plus_RW_leadout (self);
 	else if (!(priv->info & BRASERO_MEDIUM_CLOSED)) {
 		BraseroMediumTrack *leadout;
@@ -1766,9 +1885,9 @@ brasero_medium_get_contents (BraseroMedium *self,
 		priv->block_size = 2048;
 
 		priv->first_open_track = BRASERO_FIRST_TRACK_IN_LAST_SESSION (info);
+		BRASERO_BURN_LOG ("First open track %d", priv->first_open_track);
 
-		if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS)
-		||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_RESTRICTED))
+		if (BRASERO_MEDIUM_RANDOM_WRITABLE (priv->info))
 			brasero_medium_add_DVD_plus_RW_leadout (self);
 		else {
 			track = g_new0 (BraseroMediumTrack, 1);
@@ -1781,26 +1900,23 @@ brasero_medium_get_contents (BraseroMedium *self,
 							  track,
 							  code);
 		}
-
-		goto end;
 	}
-
-	if (info->status == BRASERO_SCSI_DISC_INCOMPLETE) {
+	else if (info->status == BRASERO_SCSI_DISC_INCOMPLETE) {
 		priv->info |= BRASERO_MEDIUM_APPENDABLE;
 		BRASERO_BURN_LOG ("Appendable media");
 
 		priv->first_open_track = BRASERO_FIRST_TRACK_IN_LAST_SESSION (info);
-		BRASERO_BURN_LOG ("First track in last open session %i", priv->first_open_track);
+		BRASERO_BURN_LOG ("First track in last open session %d", priv->first_open_track);
+
+		result = brasero_medium_get_sessions_info (self, handle, code);
 	}
 	else if (info->status == BRASERO_SCSI_DISC_FINALIZED) {
 		priv->info |= BRASERO_MEDIUM_CLOSED;
 		BRASERO_BURN_LOG ("Closed media");
+
+		result = brasero_medium_get_sessions_info (self, handle, code);
 	}
 
-	result = brasero_medium_get_sessions_info (self, handle, code);
-
-end:
-
 	g_free (info);
 	return result;
 }
@@ -2424,7 +2540,6 @@ brasero_medium_get_medium_type (BraseroMedium *self,
 		priv->icon = icons [5];
 		break;
 
-	/* WARNING: these types are recognized, no more */
 	case BRASERO_SCSI_PROF_BD_ROM:
 		priv->info = BRASERO_MEDIUM_BD_ROM;
 		priv->type = types [13];
@@ -2432,6 +2547,7 @@ brasero_medium_get_medium_type (BraseroMedium *self,
 		break;
 
 	case BRASERO_SCSI_PROF_BR_R_SEQUENTIAL:
+		/* check if that's a POW later */
 		priv->info = BRASERO_MEDIUM_BDR_SRM;
 		priv->type = types [14];
 		priv->icon = icons [5];
@@ -2444,7 +2560,7 @@ brasero_medium_get_medium_type (BraseroMedium *self,
 		break;
 
 	case BRASERO_SCSI_PROF_BD_RW:
-		priv->info = BRASERO_MEDIUM_BDRW;
+		priv->info = BRASERO_MEDIUM_BDRE;
 		priv->type = types [15];
 		priv->icon = icons [6];
 		break;
@@ -2454,7 +2570,8 @@ brasero_medium_get_medium_type (BraseroMedium *self,
 		priv->type = types [12];
 		priv->icon = icons [8];
 		break;
-	
+
+	/* WARNING: these types are recognized, no more */
 	case BRASERO_SCSI_PROF_NON_REMOVABLE:
 	case BRASERO_SCSI_PROF_REMOVABLE:
 	case BRASERO_SCSI_PROF_MO_ERASABLE:
@@ -2501,6 +2618,52 @@ end:
 	if (result != BRASERO_BURN_OK)
 		return result;
 
+	/* for BDs media we need to check the number of layers */
+	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_BD|BRASERO_MEDIUM_SRM)) {
+		/* check for POW */
+		hdr = NULL;
+		result = brasero_mmc2_get_configuration_feature (handle,
+								 BRASERO_SCSI_FEAT_BDR_POW,
+								 &hdr,
+								 &size,
+								 code);
+		if (result == BRASERO_SCSI_OK) {
+			if (hdr->desc->current) {
+				BRASERO_BURN_LOG ("POW formatted medium detected");
+				priv->info |= BRASERO_MEDIUM_POW;
+			}
+
+			g_free (hdr);			
+		}
+		else {
+			BraseroScsiFormatCapacitiesHdr *hdr = NULL;
+
+			/* NOTE: the disc status as far as format is concerned
+			 * is done later for all rewritable media. */
+			/* check for unformatted media (if it's POW or RANDOM
+			 * there is no need of course) */
+			result = brasero_mmc2_read_format_capacities (handle,
+								      &hdr,
+								      &size,
+								      NULL);
+			if (result == BRASERO_SCSI_OK) {
+				BraseroScsiMaxCapacityDesc *current;
+
+				current = hdr->max_caps;
+				if (!(current->type & BRASERO_SCSI_DESC_FORMATTED)) {
+					BRASERO_BURN_LOG ("Unformatted BD-R");
+					priv->info |= BRASERO_MEDIUM_UNFORMATTED;
+				}
+
+				g_free (hdr);
+			}
+		}		
+	}
+
+	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_BD)) {
+		/* check for dual layer BD */
+	}
+
 	return BRASERO_BURN_OK;
 }
 
@@ -2544,6 +2707,125 @@ brasero_medium_get_css_feature (BraseroMedium *self,
 	return BRASERO_BURN_OK;
 }
 
+static gboolean
+brasero_medium_get_CD_TEXT (BraseroMedium *medium,
+			    int type,
+			    int track_num,
+			    const char *string)
+{
+	char *utf8_string;
+	const char *charset = NULL;
+	BraseroMediumPrivate *priv;
+
+	priv = BRASERO_MEDIUM_PRIVATE (medium);
+
+	/* For the moment we're only interested in medium title but that could
+	 * be extented to all tracks information. */
+	switch (type) {
+	case BRASERO_SCSI_CD_TEXT_ALBUM_TITLE:
+		if (track_num)
+			return FALSE;
+
+		break;
+
+	case BRASERO_SCSI_CD_TEXT_PERFORMER_NAME:
+	case BRASERO_SCSI_CD_TEXT_SONGWRITER_NAME:
+	case BRASERO_SCSI_CD_TEXT_COMPOSER_NAME:
+	case BRASERO_SCSI_CD_TEXT_ARRANGER_NAME:
+	case BRASERO_SCSI_CD_TEXT_ARTIST_NAME:
+	case BRASERO_SCSI_CD_TEXT_DISC_ID_INFO:
+	case BRASERO_SCSI_CD_TEXT_GENRE_ID_INFO:
+	case BRASERO_SCSI_CD_TEXT_UPC_EAN_ISRC:
+	case BRASERO_SCSI_CD_TEXT_BLOCK_SIZE:
+	default:
+		return FALSE;
+	}
+
+	g_get_charset (&charset);
+
+	/* it's ASCII so convert to locale */
+	utf8_string = g_convert_with_fallback (string,
+					       -1,
+					       charset,
+					       "ASCII",
+					       NULL,
+					       NULL,
+					       NULL,
+					       NULL);
+
+	if (priv->CD_TEXT_title)
+		g_free (priv->CD_TEXT_title);
+
+	if (!utf8_string)
+		priv->CD_TEXT_title = g_strdup (string);
+	else
+		priv->CD_TEXT_title = utf8_string;
+
+	BRASERO_BURN_LOG ("CD-TEXT title %s", priv->CD_TEXT_title);
+	return TRUE;
+}
+
+static void
+brasero_medium_read_CD_TEXT (BraseroMedium *self,
+			     BraseroDeviceHandle *handle,
+			     BraseroScsiErrCode *code)
+{
+	int off;
+	int track_num;
+	int num, size, i;
+	char buffer [256]; /* mmc specs advise no more than 160 */
+	BraseroMediumPrivate *priv;
+	BraseroScsiCDTextData *cd_text;
+
+	BRASERO_BURN_LOG ("Getting CD-TEXT");
+	if (brasero_mmc3_read_cd_text (handle, &cd_text, &size, code) != BRASERO_SCSI_OK) {
+		BRASERO_BURN_LOG ("GET CD-TEXT failed");
+		return;
+	}
+
+	num = (BRASERO_GET_16 (cd_text->hdr->len) -
+	      (sizeof (BraseroScsiTocPmaAtipHdr) - sizeof (cd_text->hdr->len)))  /
+	       sizeof (BraseroScsiCDTextPackData);
+
+	track_num = 0;
+	off = 0;
+
+	priv = BRASERO_MEDIUM_PRIVATE (self);
+
+	for (i = 0; i < num; i ++) {
+		int j;
+
+		track_num = cd_text->pack [i].track_num;
+
+		for (j = 0; j < sizeof (cd_text->pack [i].text); j++) {
+			if (!off && cd_text->pack [i].text [j] == '\t') {
+				/* Specs say that tab character means that's the
+				 * same string as before */
+				if (buffer [0] != '\0')
+					brasero_medium_get_CD_TEXT (self,
+								    cd_text->pack [i].type,
+								    track_num,
+								    buffer);
+				track_num ++;
+				continue;
+			}
+
+			buffer [off] = cd_text->pack [i].text [j];
+			off++;
+
+			if (cd_text->pack [i].text [j] == '\0') {
+				if (buffer [0] != '\0')
+					brasero_medium_get_CD_TEXT (self,
+								    cd_text->pack [i].type,
+								    track_num,
+								    buffer);
+				track_num ++;
+				off = 0;
+			}
+		}
+	}
+}
+
 static void
 brasero_medium_init_real (BraseroMedium *object,
 			  BraseroDeviceHandle *handle)
@@ -2560,23 +2842,45 @@ brasero_medium_init_real (BraseroMedium *object,
 	BRASERO_BURN_LOG ("Initializing information for medium in %s", name);
 	g_free (name);
 
+	if (priv->probe_cancelled)
+		return;
+
 	result = brasero_medium_get_medium_type (object, handle, &code);
 	if (result != BRASERO_BURN_OK)
 		return;
 
+	if (priv->probe_cancelled)
+		return;
+
 	brasero_medium_get_capacity_by_type (object, handle, &code);
 
 	result = brasero_medium_get_contents (object, handle, &code);
 	if (result != BRASERO_BURN_OK)
 		return;
 
+	if (priv->probe_cancelled)
+		return;
+
 	/* assume that css feature is only for DVD-ROM which might be wrong but
 	 * some drives wrongly reports that css is enabled for blank DVD+R/W */
 	if (BRASERO_MEDIUM_IS (priv->info, (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_ROM)))
 		brasero_medium_get_css_feature (object, handle, &code);
 
+	if (priv->probe_cancelled)
+		return;
+
 	brasero_medium_init_caps (object, handle, &code);
 
+	if (priv->probe_cancelled)
+		return;
+
+	/* read CD-TEXT title */
+	if (priv->info & BRASERO_MEDIUM_HAS_AUDIO)
+		brasero_medium_read_CD_TEXT (object, handle, &code);
+
+	if (priv->probe_cancelled)
+		return;
+
 	BRASERO_BURN_LOG_DISC_TYPE (priv->info, "media is ");
 
 	if (!priv->wr_speeds)
@@ -2599,48 +2903,25 @@ brasero_medium_init_real (BraseroMedium *object,
 }
 
 static gboolean
-brasero_medium_retry_open (gpointer object)
+brasero_medium_probed (gpointer data)
 {
-	const gchar *path;
-	BraseroMedium *self;
-	BraseroScsiErrCode code;
 	BraseroMediumPrivate *priv;
-	BraseroDeviceHandle *handle;
 
-	self = BRASERO_MEDIUM (object);
-	priv = BRASERO_MEDIUM_PRIVATE (object);
-	path = brasero_drive_get_device (priv->drive);
+	priv = BRASERO_MEDIUM_PRIVATE (data);
 
-	BRASERO_BURN_LOG ("Retrying to open device %s", path);
-	handle = brasero_device_handle_open (path, &code);
-	if (!handle) {
-		if (code == BRASERO_SCSI_NOT_READY) {
-			BRASERO_BURN_LOG ("Device busy");
-			/* we'll retry in a second */
-			return TRUE;
-		}
-
-		BRASERO_BURN_LOG ("Open () failed");
-		priv->info = BRASERO_MEDIUM_UNSUPPORTED;
-		priv->retry_id = 0;
-		return FALSE;
-	}
-
-	BRASERO_BURN_LOG ("Open () succeeded\n");
-	priv->info = BRASERO_MEDIUM_NONE;
-	priv->icon = icons [0];
-
-	priv->retry_id = 0;
-
-	brasero_medium_init_real (self, handle);
-	brasero_device_handle_close (handle);
+	/* This signal must be emitted in the main thread */
+	g_signal_emit (data,
+		       medium_signals [PROBED],
+		       0);
 
+	priv->probe_id = 0;
 	return FALSE;
 }
 
-static void
-brasero_medium_try_open (BraseroMedium *self)
+static gpointer
+brasero_medium_probe_thread (gpointer self)
 {
+	gint counter = 0;
 	const gchar *path;
 	BraseroScsiErrCode code;
 	BraseroMediumPrivate *priv;
@@ -2649,70 +2930,76 @@ brasero_medium_try_open (BraseroMedium *self)
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 	path = brasero_drive_get_device (priv->drive);
 
+	priv->info = BRASERO_MEDIUM_BUSY;
+	priv->icon = icons [0];
+
 	/* the drive might be busy (a burning is going on) so we don't block
 	 * but we re-try to open it every second */
 	BRASERO_BURN_LOG ("Trying to open device %s", path);
-	handle = brasero_device_handle_open (path, &code);
 
-	if (!handle) {
-		if (code == BRASERO_SCSI_NOT_READY) {
-			BRASERO_BURN_LOG ("Device busy");
-			priv->info = BRASERO_MEDIUM_BUSY;
-			priv->icon = icons [0];
+	handle = brasero_device_handle_open (path, FALSE, &code);
+	while (!handle && counter <= BRASERO_MEDIUM_OPEN_ATTEMPTS) {
+		sleep (1);
 
-			priv->retry_id = g_timeout_add (BUSY_RETRY_TIME,
-							brasero_medium_retry_open,
-							self);
+		if (priv->probe_cancelled) {
+			priv->probe = NULL;
+			return NULL;
 		}
 
-		BRASERO_BURN_LOG ("Open () failed");
-		return;
+		counter ++;
+		handle = brasero_device_handle_open (path, FALSE, &code);
 	}
 
-	BRASERO_BURN_LOG ("Open () succeeded");
-	brasero_medium_init_real (self, handle);
-	brasero_device_handle_close (handle);
+	if (priv->probe_cancelled) {
+		priv->probe = NULL;
+		return NULL;
+	}
+
+	if (handle) {
+		BRASERO_BURN_LOG ("Open () succeeded");
+
+		/* NOTE: if we wanted to know the status we'd need to read the 
+		 * error code variable which is currently NULL */
+		while (brasero_spc1_test_unit_ready (handle, NULL) != BRASERO_SCSI_OK) {
+			sleep (1);
+
+			if (priv->probe_cancelled) {
+				priv->probe = NULL;
+				brasero_device_handle_close (handle);
+				BRASERO_BURN_LOG ("Device probing cancelled");
+				return NULL;
+			}
+		}
+
+		BRASERO_BURN_LOG ("Device ready");
+
+		brasero_medium_init_real (BRASERO_MEDIUM (self), handle);
+		brasero_device_handle_close (handle);
+	}
+	else
+		BRASERO_BURN_LOG ("Open () failed: medium busy");
+
+	priv->probe_id = g_idle_add (brasero_medium_probed, self);
+	priv->probe = NULL;
+	return NULL;
 }
 
-void
-brasero_medium_reload_info (BraseroMedium *self)
+static void
+brasero_medium_probe (BraseroMedium *self)
 {
 	BraseroMediumPrivate *priv;
 
-	g_return_if_fail (BRASERO_IS_MEDIUM (self));
-
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 
-	priv->max_rd = 0;
-	priv->max_wrt = 0;
-	priv->block_num = 0;
-	priv->block_size = 0;
-	priv->next_wr_add = -1;
-	priv->type = NULL;
-	priv->icon = NULL;
-	priv->info = BRASERO_MEDIUM_NONE;
-
-	if (priv->retry_id) {
-		g_source_remove (priv->retry_id);
-		priv->retry_id = 0;
-	}
-
-	if (priv->id) {
-		g_free (priv->id);
-		priv->id = NULL;
-	}
-
-	g_free (priv->rd_speeds);
-	priv->rd_speeds = NULL;
-
-	g_free (priv->wr_speeds);
-	priv->wr_speeds = NULL;
-
-	g_slist_foreach (priv->tracks, (GFunc) g_free, NULL);
-	g_slist_free (priv->tracks);
-	priv->tracks = NULL;
-
-	brasero_medium_try_open (self);
+	/* NOTE: why a thread? Because in case of a damaged medium, brasero can
+	 * block on some functions until timeout and if we do this in the main
+	 * thread then our whole UI blocks. This medium won't be exported by the
+	 * BraseroDrive that exported until it returns PROBED signal.
+	 * One (good) side effect is that it also improves start time. */
+	priv->probe = g_thread_create (brasero_medium_probe_thread,
+				       self,
+				       TRUE,
+				       NULL);
 }
 
 static void
@@ -2745,14 +3032,15 @@ brasero_medium_finalize (GObject *object)
 
 	priv = BRASERO_MEDIUM_PRIVATE (object);
 
-	if (priv->udi) {
-		g_free (priv->udi);
-		priv->udi = NULL;
+	if (priv->probe) {
+		priv->probe_cancelled = TRUE;
+		g_thread_join (priv->probe);
+		priv->probe = 0;
 	}
 
-	if (priv->retry_id) {
-		g_source_remove (priv->retry_id);
-		priv->retry_id = 0;
+	if (priv->probe_id) {
+		g_source_remove (priv->probe_id);
+		priv->probe_id = 0;
 	}
 
 	if (priv->id) {
@@ -2760,6 +3048,11 @@ brasero_medium_finalize (GObject *object)
 		priv->id = NULL;
 	}
 
+	if (priv->CD_TEXT_title) {
+		g_free (priv->CD_TEXT_title);
+		priv->CD_TEXT_title = NULL;
+	}
+
 	g_free (priv->rd_speeds);
 	priv->rd_speeds = NULL;
 
@@ -2786,9 +3079,6 @@ brasero_medium_set_property (GObject *object, guint prop_id, const GValue *value
 
 	switch (prop_id)
 	{
-	case PROP_UDI:
-		priv->udi = g_strdup (g_value_get_string (value));
-		break;
 	case PROP_DRIVE:
 		/* we don't ref the drive here as it would create a circular
 		 * dependency where the drive would hold a reference on the 
@@ -2800,7 +3090,7 @@ brasero_medium_set_property (GObject *object, guint prop_id, const GValue *value
 			break;
 		}
 
-		brasero_medium_try_open (BRASERO_MEDIUM (object));
+		brasero_medium_probe (BRASERO_MEDIUM (object));
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -2822,9 +3112,6 @@ brasero_medium_get_property (GObject *object, guint prop_id, GValue *value, GPar
 	case PROP_DRIVE:
 		g_value_set_object (value, priv->drive);
 		break;
-	case PROP_UDI:
-		g_value_set_string (value, g_strdup (priv->udi));
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -2843,6 +3130,16 @@ brasero_medium_class_init (BraseroMediumClass *klass)
 	object_class->set_property = brasero_medium_set_property;
 	object_class->get_property = brasero_medium_get_property;
 
+	medium_signals[PROBED] =
+		g_signal_new ("probed",
+		              G_OBJECT_CLASS_TYPE (klass),
+		              G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE,
+		              0,
+		              NULL, NULL,
+		              g_cclosure_marshal_VOID__VOID,
+		              G_TYPE_NONE, 0,
+		              G_TYPE_NONE);
+
 	g_object_class_install_property (object_class,
 	                                 PROP_DRIVE,
 	                                 g_param_spec_object ("drive",
@@ -2850,13 +3147,6 @@ brasero_medium_class_init (BraseroMediumClass *klass)
 	                                                      "drive in which medium is inserted",
 	                                                      BRASERO_TYPE_DRIVE,
 	                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-	g_object_class_install_property (object_class,
-	                                 PROP_UDI,
-	                                 g_param_spec_string ("udi",
-	                                                      "udi",
-	                                                      "HAL udi",
-	                                                      NULL,
-	                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
 
 gboolean
@@ -2948,15 +3238,6 @@ brasero_medium_get_drive (BraseroMedium *self)
 	return priv->drive;
 }
 
-const gchar *
-brasero_medium_get_udi (BraseroMedium *self)
-{
-	BraseroMediumPrivate *priv;
-
-	priv = BRASERO_MEDIUM_PRIVATE (self);
-	return priv->udi;
-}
-
 const gchar *
 brasero_medium_get_id (BraseroMedium *self)
 {
@@ -2966,6 +3247,16 @@ brasero_medium_get_id (BraseroMedium *self)
 	return priv->id;
 }
 
+const gchar *
+brasero_medium_get_CD_TEXT_title (BraseroMedium *self)
+{
+	BraseroMediumPrivate *priv;
+
+	priv = BRASERO_MEDIUM_PRIVATE (self);
+	return priv->CD_TEXT_title;;
+
+}
+
 GType
 brasero_medium_get_type (void)
 {
