@@ -78,8 +78,6 @@ struct _BraseroBurnSessionPrivate {
 	FILE *session;
 	gchar *session_path;
 
-	GSList *wrong_checksums;
-
 	GSList *tmpfiles;
 
 	BraseroSessionSetting settings [1];
@@ -521,130 +519,6 @@ brasero_burn_session_get_num_copies (BraseroBurnSession *self)
 	return priv->settings->num_copies;
 }
 
-static gchar *
-brasero_burn_session_get_image_complement (BraseroBurnSession *self,
-					   BraseroImageFormat format,
-					   const gchar *path)
-{
-	gchar *retval = NULL;
-	BraseroBurnSessionPrivate *priv;
-
-	priv = BRASERO_BURN_SESSION_PRIVATE (self);
-
-	if (format == BRASERO_IMAGE_FORMAT_CLONE)
-			retval = g_strdup_printf ("%s.toc", path);
-	else if (format == BRASERO_IMAGE_FORMAT_CUE) {
-		if (g_str_has_suffix (path, ".bin"))
-			retval = g_strdup_printf ("%.*scue",
-						  strlen (path) - 3,
-						  path);
-		else
-			retval = g_strdup_printf ("%s.bin", path);
-	}
-	else if (format == BRASERO_IMAGE_FORMAT_CDRDAO) {
-		if (g_str_has_suffix (path, ".bin"))
-			retval = g_strdup_printf ("%.*stoc",
-						  strlen (path) - 3,
-						  path);
-		else
-			retval = g_strdup_printf ("%s.toc", path);
-	}
-	else
-		retval = NULL;
-
-	return retval;
-}
-
-static BraseroBurnResult
-brasero_burn_session_file_test (BraseroBurnSession *self,
-				const gchar *path,
-				GError **error)
-{
-	BraseroBurnSessionPrivate *priv;
-
-	priv = BRASERO_BURN_SESSION_PRIVATE (self);
-
-	if (!path) {
-		g_set_error (error,
-			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_GENERAL,
-			     _("no path"));
-		return BRASERO_BURN_ERR;
-	}
-
-	if (!g_file_test (path, G_FILE_TEST_EXISTS))
-		return BRASERO_BURN_OK;
-	
-	if (priv->settings->flags & BRASERO_BURN_FLAG_DONT_OVERWRITE) {
-		g_set_error (error,
-			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_GENERAL,
-			     _("%s already exists"),
-			     path);
-		return BRASERO_BURN_ERR;
-	}
-
-	return BRASERO_BURN_OK;
-}
-
-static BraseroBurnResult
-brasero_burn_session_set_image_output_retval (BraseroBurnSession *self,
-					      BraseroImageFormat format,
-					      gchar **image,
-					      gchar **toc,
-					      gchar *output,
-					      gchar *complement)
-{
-	BraseroBurnSessionPrivate *priv;
-
-	priv = BRASERO_BURN_SESSION_PRIVATE (self);
-
-	switch (format) {
-	case BRASERO_IMAGE_FORMAT_BIN:
-	case BRASERO_IMAGE_FORMAT_NONE:
-		if (image)
-			*image = output;
-		else
-			g_free (output);
-
-		if (toc)
-			*toc = NULL;
-
-		g_free (complement);
-		break;
-
-	case BRASERO_IMAGE_FORMAT_CLONE:
-		if (image)
-			*image = output;
-		else
-			g_free (output);
-
-		if (toc)
-			*toc = complement;
-		else
-			g_free (complement);
-		break;
-
-	case BRASERO_IMAGE_FORMAT_CUE:
-	case BRASERO_IMAGE_FORMAT_CDRDAO:
-		if (image)
-			*image = complement;
-		else
-			g_free (complement);
-
-		if (toc)
-			*toc = output;
-		else
-			g_free (output);
-		break;
-
-	default:
-		return BRASERO_BURN_ERR;
-	}
-
-	return BRASERO_BURN_OK;
-}
-
 /**
  * This function returns a path only if we should output to a file image
  * and not burn.
@@ -652,11 +526,12 @@ brasero_burn_session_set_image_output_retval (BraseroBurnSession *self,
 
 BraseroBurnResult
 brasero_burn_session_get_output (BraseroBurnSession *self,
-				 gchar **image,
-				 gchar **toc,
+				 gchar **image_ret,
+				 gchar **toc_ret,
 				 GError **error)
 {
-	BraseroBurnResult result;
+	gchar *toc = NULL;
+	gchar *image = NULL;
 	BraseroBurnSessionPrivate *priv;
 
 	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (self), BRASERO_BURN_ERR);
@@ -668,25 +543,21 @@ brasero_burn_session_get_output (BraseroBurnSession *self,
 		return BRASERO_BURN_ERR;
 	}
 
-	if (image) {
-		/* output paths were set so test them and returns them if OK */
-		if (priv->settings->image) {
-			result = brasero_burn_session_file_test (self,
-								 priv->settings->image,
-								 error);
-			if (result != BRASERO_BURN_OK) {
-				BRASERO_BURN_LOG ("Problem with image existence");
-				return result;
-			}
+	image = g_strdup (priv->settings->image);
+	toc = g_strdup (priv->settings->toc);
 
-			*image = g_strdup (priv->settings->image);
-		}
-		else if (priv->settings->toc) {
+	if (!image && !toc)
+		return BRASERO_BURN_ERR;
+
+	if (image_ret) {
+		/* output paths were set so test them and returns them if OK */
+		if (!image && toc) {
 			gchar *complement;
+			BraseroImageFormat format;
 
 			/* get the cuesheet complement */
-			complement = brasero_image_format_get_complement (priv->settings->format,
-									  priv->settings->toc);
+			format = brasero_burn_session_get_output_format (self);
+			complement = brasero_image_format_get_complement (format, toc);
 			if (!complement) {
 				BRASERO_BURN_LOG ("no output specified");
 
@@ -694,19 +565,15 @@ brasero_burn_session_get_output (BraseroBurnSession *self,
 					     BRASERO_BURN_ERROR,
 					     BRASERO_BURN_ERROR_GENERAL,
 					     _("no output specified"));
+
+				g_free (toc);
 				return BRASERO_BURN_ERR;
 			}
 
-			result = brasero_burn_session_file_test (self,
-								 complement,
-								 error);
-			if (result != BRASERO_BURN_OK) {
-				BRASERO_BURN_LOG ("Problem with image existence");
-				return result;
-			}
-
-			*image = complement;
+			*image_ret = complement;
 		}
+		else if (image)
+			*image_ret = image;
 		else {
 			BRASERO_BURN_LOG ("no output specified");
 
@@ -717,20 +584,13 @@ brasero_burn_session_get_output (BraseroBurnSession *self,
 			return BRASERO_BURN_ERR;
 		}
 	}
+	else
+		g_free (image);
 
-	if (toc) {
-		if (priv->settings->toc) {
-			result = brasero_burn_session_file_test (self,
-								 priv->settings->toc,
-								 error);
-			if (result != BRASERO_BURN_OK) {
-				BRASERO_BURN_LOG ("Problem with toc existence");
-				return result;
-			}
-		}
-
-		*toc = g_strdup (priv->settings->toc);
-	}
+	if (toc_ret)
+		*toc_ret = toc;
+	else
+		g_free (toc);
 
 	return BRASERO_BURN_OK;
 }
@@ -767,23 +627,24 @@ brasero_burn_session_set_image_output_full (BraseroBurnSession *self,
 
 	priv = BRASERO_BURN_SESSION_PRIVATE (self);
 
-	if (!BRASERO_BURN_SESSION_WRITE_TO_FILE (priv)) {
-		BraseroMediumMonitor *monitor;
-		BraseroDrive *drive;
-		GSList *list;
-
-		monitor = brasero_medium_monitor_get_default ();
-		list = brasero_medium_monitor_get_media (monitor, BRASERO_MEDIA_TYPE_FILE);
-		drive = brasero_medium_get_drive (list->data);
-		brasero_burn_session_set_burner (self, drive);
-		g_object_unref (monitor);
-		g_slist_free (list);
-	}
-
 	if (priv->settings->format == format
 	&&  BRASERO_STR_EQUAL (image, priv->settings->image)
-	&&  BRASERO_STR_EQUAL (toc, priv->settings->toc))
+	&&  BRASERO_STR_EQUAL (toc, priv->settings->toc)) {
+		if (!BRASERO_BURN_SESSION_WRITE_TO_FILE (priv)) {
+			BraseroMediumMonitor *monitor;
+			BraseroDrive *drive;
+			GSList *list;
+
+			monitor = brasero_medium_monitor_get_default ();
+			list = brasero_medium_monitor_get_media (monitor, BRASERO_MEDIA_TYPE_FILE);
+			drive = brasero_medium_get_drive (list->data);
+			brasero_burn_session_set_burner (self, drive);
+			g_object_unref (monitor);
+			g_slist_free (list);
+		}
+
 		return BRASERO_BURN_OK;
+	}
 
 	if (priv->settings->image)
 		g_free (priv->settings->image);
@@ -803,9 +664,23 @@ brasero_burn_session_set_image_output_full (BraseroBurnSession *self,
 
 	priv->settings->format = format;
 
-	g_signal_emit (self,
-		       brasero_burn_session_signals [OUTPUT_CHANGED_SIGNAL],
-		       0);
+	if (!BRASERO_BURN_SESSION_WRITE_TO_FILE (priv)) {
+		BraseroMediumMonitor *monitor;
+		BraseroDrive *drive;
+		GSList *list;
+
+		monitor = brasero_medium_monitor_get_default ();
+		list = brasero_medium_monitor_get_media (monitor, BRASERO_MEDIA_TYPE_FILE);
+		drive = brasero_medium_get_drive (list->data);
+		brasero_burn_session_set_burner (self, drive);
+		g_object_unref (monitor);
+		g_slist_free (list);
+	}
+	else
+		g_signal_emit (self,
+			       brasero_burn_session_signals [OUTPUT_CHANGED_SIGNAL],
+			       0);
+
 	return BRASERO_BURN_OK;
 }
 
@@ -870,12 +745,14 @@ brasero_burn_session_get_tmp_dir (BraseroBurnSession *self,
 
 	*path = mkdtemp (tmp);
 	if (*path == NULL) {
+                int errsv = errno;
+
 		g_free (tmp);
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
 			     _("a temporary directory could not be created (%s)"),
-			     strerror (errno));
+			     g_strerror (errsv));
 		return BRASERO_BURN_ERR;
 	}
 
@@ -918,12 +795,14 @@ brasero_burn_session_get_tmp_file (BraseroBurnSession *self,
 
 	fd = g_mkstemp (tmp);
 	if (fd == -1) {
+                int errsv = errno;
+
 		g_free (tmp);
 		g_set_error (error, 
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
 			     _("a temporary file can't be created: %s"),
-			     strerror (errno));
+			     g_strerror (errsv));
 		return BRASERO_BURN_ERR;
 	}
 
@@ -936,6 +815,40 @@ brasero_burn_session_get_tmp_file (BraseroBurnSession *self,
 	return BRASERO_BURN_OK;
 }
 
+static gchar *
+brasero_burn_session_get_image_complement (BraseroBurnSession *self,
+					   BraseroImageFormat format,
+					   const gchar *path)
+{
+	gchar *retval = NULL;
+	BraseroBurnSessionPrivate *priv;
+
+	priv = BRASERO_BURN_SESSION_PRIVATE (self);
+
+	if (format == BRASERO_IMAGE_FORMAT_CLONE)
+		retval = g_strdup_printf ("%s.toc", path);
+	else if (format == BRASERO_IMAGE_FORMAT_CUE) {
+		if (g_str_has_suffix (path, ".bin"))
+			retval = g_strdup_printf ("%.*scue",
+						  strlen (path) - 3,
+						  path);
+		else
+			retval = g_strdup_printf ("%s.cue", path);
+	}
+	else if (format == BRASERO_IMAGE_FORMAT_CDRDAO) {
+		if (g_str_has_suffix (path, ".bin"))
+			retval = g_strdup_printf ("%.*stoc",
+						  strlen (path) - 3,
+						  path);
+		else
+			retval = g_strdup_printf ("%s.toc", path);
+	}
+	else
+		retval = NULL;
+
+	return retval;
+}
+
 BraseroBurnResult
 brasero_burn_session_get_tmp_image (BraseroBurnSession *self,
 				    BraseroImageFormat format,
@@ -953,7 +866,10 @@ brasero_burn_session_get_tmp_image (BraseroBurnSession *self,
 	priv = BRASERO_BURN_SESSION_PRIVATE (self);
 
 	/* Image tmp file */
-	result = brasero_burn_session_get_tmp_file (self, NULL, &path, error);
+	result = brasero_burn_session_get_tmp_file (self,
+						    (format == BRASERO_IMAGE_FORMAT_CLONE)? NULL:".bin",
+						    &path,
+						    error);
 	if (result != BRASERO_BURN_OK)
 		return result;
 
@@ -961,12 +877,10 @@ brasero_burn_session_get_tmp_image (BraseroBurnSession *self,
 		/* toc tmp file */
 		complement = brasero_burn_session_get_image_complement (self, format, path);
 		if (complement) {
-			result = brasero_burn_session_file_test (self,
-								 complement,
-								 error);
-			if (result != BRASERO_BURN_OK) {
+			/* That shouldn't happen ... */
+			if (g_file_test (complement, G_FILE_TEST_EXISTS)) {
 				g_free (complement);
-				return result;
+				return BRASERO_BURN_ERR;
 			}
 		}
 	}
@@ -975,12 +889,15 @@ brasero_burn_session_get_tmp_image (BraseroBurnSession *self,
 		priv->tmpfiles = g_slist_prepend (priv->tmpfiles,
 						  g_strdup (complement));
 
-	brasero_burn_session_set_image_output_retval (self,
-						      format,
-						      image,
-						      toc,
-						      path,
-						      complement);
+	if (image)
+		*image = path;
+	else
+		g_free (path);
+
+	if (toc)
+		*toc = complement;
+	else
+		g_free (complement);
 
 	return BRASERO_BURN_OK;
 }
@@ -1406,37 +1323,6 @@ brasero_burn_session_same_src_dest_drive (BraseroBurnSession *self)
 }
 
 
-/**
- *
- */
-
-void
-brasero_burn_session_add_wrong_checksum (BraseroBurnSession *self,
-					 const gchar *path)
-{
-	BraseroBurnSessionPrivate *priv;
-
-	priv = BRASERO_BURN_SESSION_PRIVATE (self);
-	priv->wrong_checksums = g_slist_prepend (priv->wrong_checksums, g_strdup (path));
-}
-
-GSList *
-brasero_burn_session_get_wrong_checksums (BraseroBurnSession *self)
-{
-	BraseroBurnSessionPrivate *priv;
-	GSList *retval;
-
-	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (self), NULL);
-
-	priv = BRASERO_BURN_SESSION_PRIVATE (self);
-
-	/* reset our list so it will return only the new ones next time */
-	retval = priv->wrong_checksums;
-	priv->wrong_checksums = NULL;
-
-	return retval;
-}
-
 /****************************** this part is for log ***************************/
 void
 brasero_burn_session_logv (BraseroBurnSession *self,
@@ -1585,91 +1471,6 @@ brasero_burn_session_stop (BraseroBurnSession *self)
 	}
 }
 
-/**
- * Misc
- */
-
-gchar *
-brasero_burn_session_get_config_key (BraseroBurnSession *self,
-				     const gchar *property)
-{
-	BraseroMedium *medium;
-	BraseroDrive *drive;
-	gchar *display_name;
-	gchar *key = NULL;
-	gchar *disc_type;
-
-	drive = brasero_burn_session_get_burner (self);
-	if (!drive)
-		return NULL;
-
-	medium = brasero_drive_get_medium (drive);
-	if (brasero_medium_get_status (medium) == BRASERO_MEDIUM_NONE)
-		return NULL;
-	
-	/* make sure display_name doesn't contain any forbidden characters */
-	display_name = brasero_drive_get_display_name (drive);
-	g_strdelimit (display_name, " +()", '_');
-
-	disc_type = g_strdup (brasero_medium_get_type_string (medium));
-	if (!disc_type) {
-		g_free (display_name);
-		return NULL;
-	}
-
-	g_strdelimit (disc_type, " +()", '_');
-
-	display_name = display_name ? display_name : "";
-	disc_type = disc_type ? disc_type : "";
-
-	switch (brasero_burn_session_get_input_type (self, NULL)) {
-	case BRASERO_TRACK_TYPE_NONE:
-		key = g_strdup_printf ("%s/%s/none_%s/%s",
-				       BRASERO_DRIVE_PROPERTIES_KEY,
-				       display_name,
-				       disc_type,
-				       property);
-		break;
-	case BRASERO_TRACK_TYPE_DISC:
-		key = g_strdup_printf ("%s/%s/disc_%s/%s",
-				       BRASERO_DRIVE_PROPERTIES_KEY,
-				       display_name,
-				       disc_type,
-				       property);
-		break;
-
-	case BRASERO_TRACK_TYPE_DATA:
-		key = g_strdup_printf ("%s/%s/data_%s/%s",
-				       BRASERO_DRIVE_PROPERTIES_KEY,
-				       display_name,
-				       disc_type,
-				       property);
-		break;
-
-	case BRASERO_TRACK_TYPE_IMAGE:
-		key = g_strdup_printf ("%s/%s/image_%s/%s",
-				       BRASERO_DRIVE_PROPERTIES_KEY,
-				       display_name,
-				       disc_type,
-				       property);
-		break;
-
-	case BRASERO_TRACK_TYPE_AUDIO:
-		key = g_strdup_printf ("%s/%s/audio_%s/%s",
-				       BRASERO_DRIVE_PROPERTIES_KEY,
-				       display_name,
-				       disc_type,
-				       property);
-		break;
-	default:
-		break;
-	}
-
-	g_free (display_name);
-	g_free (disc_type);
-	return key;
-}
-
 /**
  *
  */
@@ -1727,12 +1528,16 @@ brasero_burn_session_clean (const gchar *path)
 	if (!path)
 		return TRUE;
 
-	if (g_file_test (path, G_FILE_TEST_IS_DIR))
+	BRASERO_BURN_LOG ("Cleaning %s", path);
+
+	/* NOTE: g_file_test follows symbolic links */
+	if (g_file_test (path, G_FILE_TEST_IS_DIR)
+	&& !g_file_test (path, G_FILE_TEST_IS_SYMLINK))
 		brasero_burn_session_clean_directory (path);
 
-	/* NOTE : we don't follow uris as certain files are simply linked by content-data */
+	/* NOTE : we don't follow paths as certain files are simply linked */
 	if (g_remove (path)) {
-		BRASERO_BURN_LOG ("Cannot remove file %s", path);
+		BRASERO_BURN_LOG ("Cannot remove file %s (%s)", path, g_strerror (errno));
 		result = FALSE;
 	}
 
@@ -1813,12 +1618,6 @@ brasero_burn_session_finalize (GObject *object)
 		priv->session_path = NULL;
 	}
 
-	if (priv->wrong_checksums) {
-		g_slist_foreach (priv->wrong_checksums, (GFunc) g_free, NULL);
-		g_slist_free (priv->wrong_checksums);
-		priv->wrong_checksums = NULL;
-	}
-
 	brasero_session_settings_clean (priv->settings);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -1842,7 +1641,7 @@ brasero_burn_session_class_init (BraseroBurnSessionClass *klass)
 	brasero_burn_session_signals [OUTPUT_CHANGED_SIGNAL] =
 	    g_signal_new ("output_changed",
 			  BRASERO_TYPE_BURN_SESSION,
-			  G_SIGNAL_RUN_LAST|G_SIGNAL_ACTION|G_SIGNAL_NO_RECURSE,
+			  G_SIGNAL_RUN_FIRST|G_SIGNAL_ACTION|G_SIGNAL_NO_RECURSE,
 			  G_STRUCT_OFFSET (BraseroBurnSessionClass, output_changed),
 			  NULL,
 			  NULL,
@@ -1853,7 +1652,7 @@ brasero_burn_session_class_init (BraseroBurnSessionClass *klass)
 	brasero_burn_session_signals [INPUT_CHANGED_SIGNAL] =
 	    g_signal_new ("input_changed",
 			  BRASERO_TYPE_BURN_SESSION,
-			  G_SIGNAL_RUN_LAST|G_SIGNAL_ACTION|G_SIGNAL_NO_RECURSE,
+			  G_SIGNAL_RUN_FIRST|G_SIGNAL_ACTION|G_SIGNAL_NO_RECURSE,
 			  G_STRUCT_OFFSET (BraseroBurnSessionClass, input_changed),
 			  NULL,
 			  NULL,
