@@ -61,9 +61,11 @@ gchar *iso_uri;
 gchar **files;
 gchar **audio_project;
 gchar **data_project;
+gchar **video_project;
 gint copy_project;
 gint empty_project;
 gint disc_blank;
+gint disc_check;
 gint open_ncb;
 gint debug;
 
@@ -92,6 +94,10 @@ static const GOptionEntry options [] = {
 	  N_("Copy a disc"),
 	  NULL },
 
+	{ "video", 'o', 0, G_OPTION_ARG_NONE, &video_project,
+	  N_("Open a video project adding the URIs given on the command line"),
+	  NULL },
+
 	/* FIXME: last argument should be defined */
 	{ "image", 'i', G_OPTION_FLAG_FILENAME, G_OPTION_ARG_STRING, &iso_uri,
 	 N_("Uri of an image file to be burnt (autodetected)"),
@@ -105,9 +111,13 @@ static const GOptionEntry options [] = {
 	  N_("Open the blank disc dialog"),
 	  NULL },
 
+	{ "check", 'k', 0, G_OPTION_ARG_NONE, &disc_check,
+	  N_("Open the check disc dialog"),
+	  NULL },
+
 	{ "ncb", 'n', 0, G_OPTION_ARG_NONE, &open_ncb,
-	  N_("Open a data project with the contents of nautilus-cd-burner"),
-          NULL },
+	  N_("Burn the contents of burn:// URI"),
+	  NULL },
 
 	{ "debug", 'g', 0, G_OPTION_ARG_NONE, &debug,
 	  N_("Display debug statements on stdout"),
@@ -160,7 +170,7 @@ brasero_app_parse_options (BraseroApp *app)
 
     	if (empty_project) {
 		brasero_project_manager_empty (BRASERO_PROJECT_MANAGER (manager));
-	    	brasero_session_load (app, FALSE);
+	    	brasero_app_run (app, FALSE);
 		return;
 	}
 
@@ -178,27 +188,18 @@ brasero_app_parse_options (BraseroApp *app)
 		nb ++;
 	if (data_project)
 		nb ++;
+	if (video_project)
+	    	nb ++;
 	if (disc_blank)
 	  	nb ++;
 	if (open_ncb)
 		nb ++;
 
 	if (nb > 1) {
-		GtkWidget *message;
-
-		message = gtk_message_dialog_new (NULL,
-						  GTK_DIALOG_MODAL |
-						  GTK_DIALOG_DESTROY_WITH_PARENT,
-						  GTK_MESSAGE_INFO,
-						  GTK_BUTTONS_CLOSE,
-						  _("Incompatible command line options used:"));
-
-		gtk_window_set_title (GTK_WINDOW (message), _("Incompatible Options"));
-		
-		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
-							  _("only one option can be given at a time."));
-		gtk_dialog_run (GTK_DIALOG (message));
-		gtk_widget_destroy (message);
+		brasero_utils_message_dialog (NULL,
+					      _("Incompatible command line options used:"),
+					      _("only one option can be given at a time."),
+					      GTK_MESSAGE_ERROR);
 
 		brasero_project_manager_empty (BRASERO_PROJECT_MANAGER (manager));
 	}
@@ -214,18 +215,24 @@ brasero_app_parse_options (BraseroApp *app)
 			device = files [0];
 		
 		/* this can't combine with any other options */
+		brasero_project_manager_set_oneshot (BRASERO_PROJECT_MANAGER (manager), TRUE);
 		brasero_project_manager_copy (BRASERO_PROJECT_MANAGER (manager), device);
+		return;
 	}
 	else if (iso_uri) {
+		brasero_project_manager_set_oneshot (BRASERO_PROJECT_MANAGER (manager), TRUE);
 		BRASERO_PROJECT_OPEN_URI (manager, brasero_project_manager_iso, iso_uri);
+		return;
 	}
 	else if (project_uri) {
+		brasero_project_manager_set_oneshot (BRASERO_PROJECT_MANAGER (manager), TRUE);
 		BRASERO_PROJECT_OPEN_URI (manager, brasero_project_manager_open_project, project_uri);
 	}
 
 #ifdef BUILD_PLAYLIST
 
 	else if (playlist_uri) {
+		brasero_project_manager_set_oneshot (BRASERO_PROJECT_MANAGER (manager), TRUE);
 		BRASERO_PROJECT_OPEN_URI (manager, brasero_project_manager_open_playlist, playlist_uri);
 	}
 
@@ -237,8 +244,36 @@ brasero_app_parse_options (BraseroApp *app)
 	else if (data_project) {
 		BRASERO_PROJECT_OPEN_LIST (manager, brasero_project_manager_data, files);
 	}
+	else if (video_project) {
+	    	BRASERO_PROJECT_OPEN_LIST (manager, brasero_project_manager_video, files);
+	}
 	else if (disc_blank) {
-		brasero_app_blank (app);
+		gchar *device = NULL;
+
+		/* make sure there is only one file in the remaining list for
+		 * specifying the source device. It could be extended to let
+		 * the user specify the destination device as well */
+		if (files
+		&&  files [0] != NULL
+		&&  files [1] == NULL)
+			device = files [0];
+
+		brasero_app_blank (app, device, TRUE);
+		return;
+	}
+	else if (disc_check) {
+		gchar *device = NULL;
+
+		/* make sure there is only one file in the remaining list for
+		 * specifying the source device. It could be extended to let
+		 * the user specify the destination device as well */
+		if (files
+		&&  files [0] != NULL
+		&&  files [1] == NULL)
+			device = files [0];
+
+		brasero_app_check (app, device, TRUE);
+		return;
 	}
 	else if (open_ncb) {
 		GSList *list = NULL;
@@ -246,7 +281,7 @@ brasero_app_parse_options (BraseroApp *app)
 
 		list = g_slist_prepend (NULL, "burn:///");
 
-		/* in this case we can also add the files */
+		/* in this case we can also add the files from the command line */
 		for (iter = files; iter && *iter; iter ++) {
 			GFile *file;
 			gchar *uri;
@@ -260,25 +295,33 @@ brasero_app_parse_options (BraseroApp *app)
 
 		/* reverse to keep the order of files */
 		list = g_slist_reverse (list);
+		brasero_project_manager_set_oneshot (BRASERO_PROJECT_MANAGER (manager), TRUE);
 		brasero_project_manager_data (BRASERO_PROJECT_MANAGER (manager), list);
 		g_slist_free (list);
+		return;
 	}
 	else if (files) {
-		BraseroProjectType type;
+		if (g_strv_length (files) == 1) {
+			BraseroProjectType type;
 
-	    	if (g_strv_length (files) > 1)
-			BRASERO_PROJECT_OPEN_LIST (manager, brasero_project_manager_data, files);
+			brasero_project_manager_set_oneshot (BRASERO_PROJECT_MANAGER (manager), TRUE);
+			type = brasero_project_manager_open_uri (BRASERO_PROJECT_MANAGER (manager), files [0]);
 
-		type = brasero_project_manager_open_uri (BRASERO_PROJECT_MANAGER (manager), files [0]);
-		if (type == BRASERO_PROJECT_TYPE_INVALID)
+			/* Fallback if it hasn't got a suitable URI */
+			if (type == BRASERO_PROJECT_TYPE_INVALID)
+				BRASERO_PROJECT_OPEN_LIST (manager, brasero_project_manager_data, files);
+		}
+		else {
+			brasero_project_manager_set_oneshot (BRASERO_PROJECT_MANAGER (manager), TRUE);
 			BRASERO_PROJECT_OPEN_LIST (manager, brasero_project_manager_data, files);
+		}
 	}
 	else {
 		brasero_project_manager_empty (BRASERO_PROJECT_MANAGER (manager));
 	    	load_default_project = TRUE;
 	}
 
-    	brasero_session_load (app, load_default_project);
+	brasero_app_run (app, load_default_project);
 }
 
 int
@@ -342,14 +385,8 @@ main (int argc, char **argv)
 	if (app == NULL)
 		return 1;
 
-	gtk_widget_realize (app);
-
 	brasero_app_parse_options (BRASERO_APP (app));
 
-	gtk_widget_show (app);
-
-	gtk_main ();
-
 	brasero_burn_library_shutdown ();
 
 #ifdef BUILD_GNOME2
