@@ -101,12 +101,14 @@ brasero_checksum_image_read (BraseroChecksumImage *self,
 		/* ... or an error =( */
 		if (read_bytes == -1) {
 			if (errno != EAGAIN && errno != EINTR) {
+                                int errsv = errno;
+
 				g_set_error (error,
 					     BRASERO_BURN_ERROR,
 					     BRASERO_BURN_ERROR_GENERAL,
 					     _("data could not be read from the pipe (%i: %s)"),
-					     errno,
-					     strerror (errno));
+					     errsv,
+					     g_strerror (errsv));
 				return -1;
 			}
 		}
@@ -149,13 +151,15 @@ brasero_checksum_image_write (BraseroChecksumImage *self,
 
 		if (written != bytes_remaining) {
 			if (errno != EINTR && errno != EAGAIN) {
+                                int errsv = errno;
+
 				/* unrecoverable error */
 				g_set_error (error,
 					     BRASERO_BURN_ERROR,
 					     BRASERO_BURN_ERROR_GENERAL,
 					     _("the data couldn't be written to the pipe (%i: %s)"),
-					     errno,
-					     strerror (errno));
+					     errsv,
+					     g_strerror (errsv));
 				return BRASERO_BURN_ERR;
 			}
 		}
@@ -278,6 +282,7 @@ brasero_checksum_image_checksum_file_input (BraseroChecksumImage *self,
 
 	fd_in = open (path, O_RDONLY);
 	if (!fd_in) {
+                int errsv;
 		gchar *name = NULL;
 
 		if (errno == ENOENT)
@@ -285,12 +290,13 @@ brasero_checksum_image_checksum_file_input (BraseroChecksumImage *self,
 
 		name = g_path_get_basename (path);
 
+                errsv = errno;
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
 			     _("the file %s couldn't be read (%s)"),
 			     name,
-			     strerror (errno));
+			     g_strerror (errsv));
 		g_free (name);
 		g_free (path);
 
@@ -590,6 +596,34 @@ brasero_checksum_image_start (BraseroJob *job,
 	return BRASERO_BURN_OK;
 }
 
+static BraseroBurnResult
+brasero_checksum_image_activate (BraseroJob *job,
+				 GError **error)
+{
+	BraseroBurnFlag flags = BRASERO_BURN_FLAG_NONE;
+	BraseroTrack *track = NULL;
+	BraseroJobAction action;
+
+	brasero_job_get_current_track (job, &track);
+	brasero_job_get_action (job, &action);
+
+	if (action == BRASERO_JOB_ACTION_IMAGE
+	&&  brasero_track_get_checksum_type (track) != BRASERO_CHECKSUM_NONE) {
+		BRASERO_JOB_LOG (job, "There is a checksum already");
+		/* if there is a checksum already, if so no need to redo one */
+		return BRASERO_BURN_NOT_RUNNING;
+	}
+
+	flags = BRASERO_BURN_FLAG_NONE;
+	brasero_job_get_flags (job, &flags);
+	if (flags & BRASERO_BURN_FLAG_DUMMY) {
+		BRASERO_JOB_LOG (job, "Dummy operation, skipping");
+		return BRASERO_BURN_NOT_RUNNING;
+	}
+
+	return BRASERO_BURN_OK;
+}
+
 static BraseroBurnResult
 brasero_checksum_image_clock_tick (BraseroJob *job)
 {
@@ -679,6 +713,7 @@ brasero_checksum_image_class_init (BraseroChecksumImageClass *klass)
 	parent_class = g_type_class_peek_parent (klass);
 	object_class->finalize = brasero_checksum_image_finalize;
 
+	job_class->activate = brasero_checksum_image_activate;
 	job_class->start = brasero_checksum_image_start;
 	job_class->stop = brasero_checksum_image_stop;
 	job_class->clock_tick = brasero_checksum_image_clock_tick;
@@ -697,11 +732,16 @@ brasero_checksum_image_export_caps (BraseroPlugin *plugin, gchar **error)
 			       0);
 
 	/* For images we can process (thus generating a sum on the fly or simply
-	 * test images. */
+	 * test images). */
 	input = brasero_caps_image_new (BRASERO_PLUGIN_IO_ACCEPT_FILE|
 					BRASERO_PLUGIN_IO_ACCEPT_PIPE,
 					BRASERO_IMAGE_FORMAT_BIN);
 	brasero_plugin_process_caps (plugin, input);
+
+	brasero_plugin_set_process_flags (plugin,
+					  BRASERO_PLUGIN_RUN_PREPROCESSING|
+					  BRASERO_PLUGIN_RUN_BEFORE_TARGET);
+
 	brasero_plugin_check_caps (plugin,
 				   BRASERO_CHECKSUM_MD5|
 				   BRASERO_CHECKSUM_SHA1|
@@ -709,10 +749,6 @@ brasero_checksum_image_export_caps (BraseroPlugin *plugin, gchar **error)
 				   input);
 	g_slist_free (input);
 
-	brasero_plugin_set_process_flags (plugin,
-					  BRASERO_PLUGIN_RUN_FIRST|
-					  BRASERO_PLUGIN_RUN_LAST);
-
 	/* add some configure options */
 	checksum_type = brasero_plugin_conf_option_new (GCONF_KEY_CHECKSUM_TYPE,
 							_("Hashing algorithm to be used:"),
