@@ -279,7 +279,8 @@ brasero_libburn_add_file_track (struct burn_session *session,
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     strerror (errno));
+			     "%s",
+			     g_strerror (errno));
 		return BRASERO_BURN_ERR;
 	}
 
@@ -399,6 +400,7 @@ brasero_libburn_setup_session_file (BraseroLibburn *self,
 		}
 		else if (type.type == BRASERO_TRACK_TYPE_IMAGE) {
 			gchar *imagepath;
+			gint64 size;
 			gint mode;
 
 			if (type.subtype.img_format == BRASERO_IMAGE_FORMAT_BIN) {
@@ -415,12 +417,21 @@ brasero_libburn_setup_session_file (BraseroLibburn *self,
 			if (!imagepath)
 				return BRASERO_BURN_ERR;
 
+			result = brasero_track_get_image_size (track,
+							       NULL,
+							       NULL,
+							       &size,
+							       error);
+			if (result != BRASERO_BURN_OK)
+				return BRASERO_BURN_ERR;
+
 			result = brasero_libburn_add_file_track (session,
 								 imagepath,
 								 mode,
-								 -1,
+								 size,
 								 priv->pvd,
 								 error);
+			g_free (imagepath);
 		}
 		else
 			BRASERO_JOB_NOT_SUPPORTED (self);
@@ -465,11 +476,13 @@ brasero_libburn_start_record (BraseroLibburn *self,
 			      GError **error)
 {
 	guint64 rate;
+	gint64 blocks = 0;
 	BraseroMedia media;
 	BraseroBurnFlag flags;
 	BraseroBurnResult result;
 	BraseroLibburnPrivate *priv;
 	struct burn_write_opts *opts;
+	gchar reason [BURN_REASONS_LEN];
 
 	priv = BRASERO_LIBBURN_PRIVATE (self);
 
@@ -478,9 +491,7 @@ brasero_libburn_start_record (BraseroLibburn *self,
 	brasero_job_get_media (BRASERO_JOB (self), &media);
 
 	if (flags & (BRASERO_BURN_FLAG_MERGE|BRASERO_BURN_FLAG_APPEND)
-	&& (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS)
-	||  BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_RESTRICTED)
-	||  BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS_DL))
+	&&  BRASERO_MEDIUM_RANDOM_WRITABLE (media)
 	&& (media & BRASERO_MEDIUM_HAS_DATA))
 		priv->pvd = g_new0 (unsigned char, BRASERO_PVD_SIZE);
 
@@ -495,10 +506,7 @@ brasero_libburn_start_record (BraseroLibburn *self,
 	opts = burn_write_opts_new (priv->ctx->drive);
 	burn_write_opts_set_perform_opc (opts, 0);
 
-	if (!BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS)
-	&&  !BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_RESTRICTED)
-	&&  !BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS_DL)
-	&&  (flags & BRASERO_BURN_FLAG_DAO))
+	if (flags & BRASERO_BURN_FLAG_DAO)
 		burn_write_opts_set_write_type (opts,
 						BURN_WRITE_SAO,
 						BURN_BLOCK_SAO);
@@ -507,8 +515,11 @@ brasero_libburn_start_record (BraseroLibburn *self,
 						BURN_WRITE_TAO,
 						BURN_BLOCK_MODE1);
 
-		/* we also set the start block to write from if MERGE is set */
-		if (flags & BRASERO_BURN_FLAG_MERGE) {
+		/* we also set the start block to write from if MERGE is set.
+		 * That only for random writable media; for other media libburn
+		 * handles all by himself where to start writing. */
+		if (BRASERO_MEDIUM_RANDOM_WRITABLE (media)
+		&& (flags & BRASERO_BURN_FLAG_MERGE)) {
 			gint64 address = 0;
 
 			brasero_job_get_next_writable_address (BRASERO_JOB (self), &address);
@@ -524,6 +535,23 @@ brasero_libburn_start_record (BraseroLibburn *self,
 
 	brasero_job_get_rate (BRASERO_JOB (self), &rate);
 	burn_drive_set_speed (priv->ctx->drive, rate, 0);
+
+	if (burn_precheck_write (opts, priv->ctx->disc, reason, 0) < 1) {
+		g_set_error (error,
+			     BRASERO_BURN_ERROR,
+			     BRASERO_BURN_ERROR_GENERAL,
+			     reason);
+		return BRASERO_BURN_ERR;
+	}
+
+	/* If we're writing to a disc remember that the session can't be under
+	 * 300 sectors (= 614400 bytes) */
+	brasero_job_get_session_output_size (BRASERO_JOB (self), &blocks, NULL);
+	if (blocks < 300)
+		brasero_job_set_output_size_for_current_track (BRASERO_JOB (self),
+							       300L - blocks,
+							       614400L - blocks * 2048);
+
 	burn_disc_write (opts, priv->ctx->disc);
 	burn_write_opts_free (opts);
 
@@ -727,6 +755,7 @@ brasero_libburn_clock_tick (BraseroJob *job)
 
 	/* Double check that everything went well */
 	if (!burn_drive_wrote_well (priv->ctx->drive)) {
+		BRASERO_JOB_LOG (job, "Something went wrong");
 		brasero_job_error (job,
 				   g_error_new (BRASERO_BURN_ERROR,
 						BRASERO_BURN_ERROR_GENERAL,
@@ -833,7 +862,7 @@ brasero_libburn_export_caps (BraseroPlugin *plugin, gchar **error)
 					  BRASERO_MEDIUM_HAS_DATA|
 					  BRASERO_MEDIUM_BLANK;
 	const BraseroMedia media_dvd_rw_plus = BRASERO_MEDIUM_DVD|
-					       BRASERO_MEDIUM_DVD_DL|
+					       BRASERO_MEDIUM_DUAL_L|
 					       BRASERO_MEDIUM_PLUS|
 					       BRASERO_MEDIUM_RESTRICTED|
 					       BRASERO_MEDIUM_REWRITABLE|
@@ -851,9 +880,11 @@ brasero_libburn_export_caps (BraseroPlugin *plugin, gchar **error)
 			       "Philippe Rouquier",
 			       15);
 
+	/* libburn has no OVERBURN capabilities */
+
 	/* CD(R)W */
-	BRASERO_PLUGIN_ADD_STANDARD_CDR_FLAGS (plugin);
-	BRASERO_PLUGIN_ADD_STANDARD_CDRW_FLAGS (plugin);
+	BRASERO_PLUGIN_ADD_STANDARD_CDR_FLAGS (plugin, BRASERO_BURN_FLAG_OVERBURN);
+	BRASERO_PLUGIN_ADD_STANDARD_CDRW_FLAGS (plugin, BRASERO_BURN_FLAG_OVERBURN);
 
 	/* audio support for CDs only */
 	input = brasero_caps_audio_new (BRASERO_PLUGIN_IO_ACCEPT_PIPE|
@@ -878,15 +909,15 @@ brasero_libburn_export_caps (BraseroPlugin *plugin, gchar **error)
 	brasero_plugin_link_caps (plugin, output, input);
 	g_slist_free (output);
 
-	BRASERO_PLUGIN_ADD_STANDARD_DVDR_PLUS_FLAGS (plugin);
-	BRASERO_PLUGIN_ADD_STANDARD_DVDR_FLAGS (plugin);
+	BRASERO_PLUGIN_ADD_STANDARD_DVDR_PLUS_FLAGS (plugin, BRASERO_BURN_FLAG_OVERBURN);
+	BRASERO_PLUGIN_ADD_STANDARD_DVDR_FLAGS (plugin, BRASERO_BURN_FLAG_OVERBURN);
 
 	/* ... and DVD-RW (sequential) */
 	output = brasero_caps_disc_new (media_dvd_rw);
 	brasero_plugin_link_caps (plugin, output, input);
 	g_slist_free (output);
 
-	BRASERO_PLUGIN_ADD_STANDARD_DVDRW_FLAGS (plugin);
+	BRASERO_PLUGIN_ADD_STANDARD_DVDRW_FLAGS (plugin, BRASERO_BURN_FLAG_OVERBURN);
 
 	/* for DVD+/-RW restricted */
 	output = brasero_caps_disc_new (media_dvd_rw_plus);
@@ -894,8 +925,8 @@ brasero_libburn_export_caps (BraseroPlugin *plugin, gchar **error)
 	g_slist_free (output);
 	g_slist_free (input);
 
-	BRASERO_PLUGIN_ADD_STANDARD_DVDRW_RESTRICTED_FLAGS (plugin);
-	BRASERO_PLUGIN_ADD_STANDARD_DVDRW_PLUS_FLAGS (plugin);
+	BRASERO_PLUGIN_ADD_STANDARD_DVDRW_RESTRICTED_FLAGS (plugin, BRASERO_BURN_FLAG_OVERBURN);
+	BRASERO_PLUGIN_ADD_STANDARD_DVDRW_PLUS_FLAGS (plugin, BRASERO_BURN_FLAG_OVERBURN);
 
 	/* add blank caps */
 	output = brasero_caps_disc_new (BRASERO_MEDIUM_CD|
